using System;
using Zcu.Mve.Core;
using Zcu.Mve.Numerics;
using System.ComponentModel;

namespace Examples
{
  /// <summary>
  /// Module creates 100 samples of connected function in &lt;0, 10&gt; interval.
  /// Sample module with NonAutoUpdate and NonEvokeUpdate ports.
  /// </summary>
	[ModuleInfo("Milan Frank",
    "Module creates given number of samples of connected function.",
     CategoryType.Filter, CategoryData.Number)]
  [PortInfo("graph", "Output generated graph.")]
  [PortInfo("x", "Output indepenedent variable.")]
  [PortInfo("y", "Input dependent variable.")]
  public class GenerateGraph : Zcu.Mve.Core.Module
	{
		private int numSamps = 100;

        /// <summary>
        /// Number of asked samples.
        /// </summary>
		[Description("Number of asked samples.")]
		public int NumSamps
		{
			get { return numSamps; }
			set {
				if (value > 0)
				{
					numSamps = value;
				}
				else
				{
					throw new MveException(this, "Number of samples must be positive.");
				}
				 }
		}
	

		/// <summary>
		/// Creates ports.
		/// </summary>
    public GenerateGraph()
		{
      AddOutPort("graph", typeof(VectorND));
      AddOutPort("Sample number", typeof(Integer), false);
      AddInPort("y", typeof(Scalar), true, false);
		}
	
    /// <summary>
    /// Generates samples acording to dependent variable.
    /// </summary>
    public override void Execute()
    {
      VectorND vector = new VectorND(new double [numSamps]);
      Integer x = new Integer();
      Scalar y;

      for (int i = 0; i < numSamps; i++)
      {
        x.Value = i ;
		SetOutput("Sample number", x);
        // Execution of submap
        UpdateInput("y");
        y = (Scalar) GetInput("y");
        vector[i] = y.Value;
      }

      SetOutput("graph", vector);
    }
  }
}
