using System;
using System.Collections.Generic;
using System.Text;
using Zcu.Mve.Core;
using Zcu.Mve.Numerics;
using Zcu.Mve.Visualization;

namespace Examples
{
	/// <summary>
	/// A module that returns inverse body diagonal of a TriangleMesh. Useful for coordinate normalization.
	/// </summary>
    [ModuleInfo("Libor Va", "A module that returns inverse body diagonal of a TriangleMesh. Useful for coordinate normalization.", CategoryType.Filter, CategoryData.Mesh)]
	[PortInfo("Mesh", "Input mesh that will be used.")]
	[PortInfo("Inverse diagonal", "A scalar value that is equal to 1/(length of body diagonal) of the input mesh")]
	class InverseTriangleMeshDiagonal:Module
	{
		private Scalar result;
		private bool resultSet;

		/// <summary>
		/// Default constructor
		/// </summary>
		public InverseTriangleMeshDiagonal()
		{
			AddInPort("Mesh", typeof(TriangleMesh));
			AddOutPort("Inverse diagonal", typeof(Scalar));
		}

		/// <summary>
		/// performs the computation, retrieves the body diagonal from the BoundingBox property of the mesh and computes its inverse value.
		/// </summary>
		public override void Execute()
		{
			if ((!IsAllDataSame()) || (!resultSet))
			{
				TriangleMesh input = (TriangleMesh)GetInput("Mesh");
				double diagonal = input.BoundingBox.MaxDiag;
				diagonal = 1 / diagonal;
				result = new Scalar(diagonal);
				SetOutput("Inverse diagonal", result);
				resultSet = true;
			}
			else
			{
				SetOutput("Inverse diagonal", result, true);
			}
		}
	}
}
