using System;
using Zcu.Mve.Core;
using Zcu.Mve.Visualization;
using Zcu.Mve.Numerics;

namespace Zcu.Mve.Testing
{
    /// <summary>
    /// Source of a convolution mask
    /// </summary>
    [ModuleInfo("Milan Frank", "Zdroj konvolun masky", CategoryType.Source, CategoryData.Number)]
    [PortInfo("Mask", "Konvolun maska.")]
    public class ConvSource : Module
    {
        internal UniformDataArray2D mask = new UniformDataArray2D(typeof(Scalar), 3, 3);

        /// <summary>
        /// Creates a new instance of the module. Adds input and output ports, initializes the mask to a point sample
        /// </summary>
        public ConvSource()
        {
            AddOutPort("Mask", typeof(UniformDataArray2D));

            // implicitn hodnoty
            mask[0, 0] = new Scalar(0.0);
            mask[0, 1] = new Scalar(0.0);
            mask[0, 2] = new Scalar(0.0);
            mask[1, 0] = new Scalar(0.0);
            mask[1, 1] = new Scalar(1.0);
            mask[1, 2] = new Scalar(0.0);
            mask[2, 0] = new Scalar(0.0);
            mask[2, 1] = new Scalar(0.0);
            mask[2, 2] = new Scalar(0.0);
        }

        /// <summary>
        /// Sets the output
        /// </summary>
        public override void Execute()
        {
            SetOutput("Mask", mask);
        }

        /// <summary>
        /// Creates a setup dialog for the module.
        /// </summary>
        /// <returns>New instance of the ConvSourceSetup</returns>
        public override ModuleSetup InvokeSetup()
        {
            return new ConvSourceSetup(this);
        }

        /// <inheritdoc />
        public override void WriteConfig(System.Xml.XmlElement config)
        {
            config.SetAttribute("pos00", ((double)((Scalar)mask[0, 0])).ToString(Globals.nfi));
            config.SetAttribute("pos01", ((double)((Scalar)mask[0, 1])).ToString(Globals.nfi));
            config.SetAttribute("pos02", ((double)((Scalar)mask[0, 2])).ToString(Globals.nfi));
            config.SetAttribute("pos10", ((double)((Scalar)mask[1, 0])).ToString(Globals.nfi));
            config.SetAttribute("pos11", ((double)((Scalar)mask[1, 1])).ToString(Globals.nfi));
            config.SetAttribute("pos12", ((double)((Scalar)mask[1, 2])).ToString(Globals.nfi));
            config.SetAttribute("pos20", ((double)((Scalar)mask[2, 0])).ToString(Globals.nfi));
            config.SetAttribute("pos21", ((double)((Scalar)mask[2, 1])).ToString(Globals.nfi));
            config.SetAttribute("pos22", ((double)((Scalar)mask[2, 2])).ToString(Globals.nfi));
        }

        /// <inheritdoc />
        public override void ReadConfig(System.Xml.XmlElement config)
        {
            string str;
            UniformDataArray2D tmpMask = new UniformDataArray2D(typeof(Scalar), 3, 3);

            if ((str = config.GetAttribute("pos00")) == null) return;
            tmpMask[0, 0] = (Scalar)Double.Parse(str, Globals.nfi);
            if ((str = config.GetAttribute("pos01")) == null) return;
            tmpMask[0, 1] = (Scalar)Double.Parse(str, Globals.nfi);
            if ((str = config.GetAttribute("pos02")) == null) return;
            tmpMask[0, 2] = (Scalar)Double.Parse(str, Globals.nfi);
            if ((str = config.GetAttribute("pos10")) == null) return;
            tmpMask[1, 0] = (Scalar)Double.Parse(str, Globals.nfi);
            if ((str = config.GetAttribute("pos11")) == null) return;
            tmpMask[1, 1] = (Scalar)Double.Parse(str, Globals.nfi);
            if ((str = config.GetAttribute("pos12")) == null) return;
            tmpMask[1, 2] = (Scalar)Double.Parse(str, Globals.nfi);
            if ((str = config.GetAttribute("pos20")) == null) return;
            tmpMask[2, 0] = (Scalar)Double.Parse(str, Globals.nfi);
            if ((str = config.GetAttribute("pos21")) == null) return;
            tmpMask[2, 1] = (Scalar)Double.Parse(str, Globals.nfi);
            if ((str = config.GetAttribute("pos22")) == null) return;
            tmpMask[2, 2] = (Scalar)Double.Parse(str, Globals.nfi);

            this.mask = tmpMask;
        }
    }
}
