using System;
using Zcu.Mve.Numerics;

namespace Zcu.Mve.Testing
{
    /// <summary>
    /// Pendulum module
    /// </summary>
    public class Pendulum : Zcu.Mve.Core.Module
    {
        private PendulumPanel panel;
        internal double pendulumPosition;

        /// <summary>
        /// Constructor.
        /// </summary>
        public Pendulum()
        {
            this.SimulationStarted += new SimulationStartedEventHandler(SimStart);
            this.SimulationEnding += new SimulationEndingEventHandler(SimEnd);
            AddInPort("pendulumPosition", typeof(Scalar));
            panel = new PendulumPanel(this);
        }

        /// <inheritdoc />
        public override void Execute()
        {
            pendulumPosition = ((Scalar)GetInput("pendulumPosition")).Value;

            if (panel.IsDisposed == false)
            {
                panel.Refresh();
            }

            System.Threading.Thread.Sleep(25);
        }

        /// <summary>
        /// Shows the window at the simulation start.
        /// </summary>
        /// <param name="source">event source</param>
        public void SimStart(object source)
        {
            if (panel.IsDisposed)
            {
                panel = new PendulumPanel(this);
            }
            panel.Show();
        }

        /// <summary>
        /// Closes the window at the simulation end.
        /// </summary>
        /// <param name="source">event source</param>
        public void SimEnd(object source)
        {
            panel.Dispose();
            /*
            if (panel.IsDisposed)
            {
              panel = new PositionRendererPanel(this);
            }
            panel.Show();
            */
            Console.WriteLine("Pendulum: simulation finished.");
        }
    }
}
