using System;
using Zcu.Mve.Core;
using Zcu.Mve.Visualization;
using Zcu.Mve.Numerics;

namespace Zcu.Mve.Testing
{
    /// <summary>
    /// Converts UniformDataArray2D to RegGrid2D.
    /// </summary>
    public class UniformArray2RegGrid : Module
    {
        RegGrid2D output = null;
        string attrName = "values";

        private bool paramsChanged = true;

        /// <summary>
        /// Gets/sets the output attribute name.
        /// </summary>
        public string AttrName
        {
            get
            {
                return (attrName);
            }
            set
            {
                if (value != "")
                {
                    attrName = value;
                    paramsChanged = true;
                }
                else
                {
                    throw new MveException(this, "Output attribute name cannot be an empty string.");
                }
            }
        }

        /// <summary>
        /// Constructor.
        /// </summary>
        public UniformArray2RegGrid()
        {
            AddInPort("Input array", typeof(UniformDataArray2D));
            AddOutPort("Output grid", typeof(RegGrid2D));
        }

        /// <inheritdoc />
        public override void Execute()
        {
            if ((!IsDataSame("Input array")) || (output == null) || (paramsChanged))
            {
                UniformDataArray2D input = (UniformDataArray2D)GetInput("Input array");
                output = new RegGrid2D(input.Height, input.Width);
                output.AddCellAttrs(attrName, input);
                SetOutput("Output grid", output);
                paramsChanged = false;
            }
            else
            {
                SetOutput("Output grid", output, true);
            }
        }
    }
}
