using System;
using System.Collections.Generic;
using System.Text;
using Zcu.Mve.Core;
using Zcu.Mve.Visualization;
using Zcu.Mve.Numerics;

namespace Zcu.Mve.Visualization.Processing
{
    class AttributeHistogram:Module
    {
        #region Private fields
        private bool paramsChanged = true;
        private UniformDataArray result;
        private int classCount = 100;
        private string attributeName = "";
        #endregion

        #region Properties        
        public int ClassCount
        {
            get { return (classCount); }
            set
            {
                if (value <= 0)
                {
                    throw new MveException(this, "Class count must be set to a value that is gereater than zero");
                }
                else
                {
                    classCount = value;
                    paramsChanged = true;
                }
            }
        }        

        public string AttributeName
        {
            get { return (attributeName); }
            set
            {
                attributeName = value;
                paramsChanged = true;
            }
        }
        #endregion

        public AttributeHistogram()
        {
            AddInPort("Mesh", typeof(UnstrGrid));
            AddOutPort("Histogram", typeof(UniformDataArray));
        }        

        public override void Execute()
        {
            if (paramsChanged || (!IsAllDataSame()))
            {
                UnstrGrid input = (UnstrGrid)GetInput("Mesh");
                UniformDataArray attribute;
                if (attributeName == "")
                {
                    UniformDataArray[] attrs = input.GetPointAttrs(typeof(Scalar));
                    if (attrs.Length == 0)
                    {
                        throw new MveException(this, "No Scalar point attributes found.");
                    }
                    attribute = attrs[0];
                }
                else
                {
                    attribute = input.GetPointAttrs(attributeName);
                }

                double min = double.MaxValue;
                double max = double.MinValue;

                for (int i = 0; i < attribute.Count; i++)
                {
                    double value = ((Scalar)attribute[i]).Value;
                    if (value < min) min = value;
                    if (value > max) max = value;
                }

                double step = (max - min) / classCount;

                step *= 1.0001;

                int[] histogram = new int[classCount];

                for (int i = 0; i < attribute.Count; i++)
                {
                    double value = ((Scalar)attribute[i]).Value;
                    int classId = (int)((value - min) / step);
                    histogram[classId]++;
                }

                result = new UniformDataArray(typeof(Point2D), classCount);
                for (int i = 0; i < classCount; i++)
                {
                    result[i] = new Point2D(min + i * step, histogram[i]);
                }

                SetOutput("Histogram", result);
                paramsChanged = false;
            }
            else
            {
                SetOutput("Histogram", result, true);
            }
        }
    }
}
