using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Zcu.Mve.Core;
using Zcu.Mve.Visualization;
using System.ComponentModel;
using System.Collections;
using Zcu.Mve.Numerics;

namespace Zcu.Mve.Visualization.Processing
{
    /// <summary>
    ///Module which maps ColorRGBA value on chosen atribute of UnstrGrid type. It takes atribute selected by user, sends out it's remaped scalar values to any color palette and returned color values are saved as new, user named attribute. 
    /// </summary>
    [ModuleInfo("Jan tefny", "Module which maps ColorRGBA value on chosen atribute of UnstrGrid type. It takes atribute selected by user, sends out it's remaped scalar values to any color palette and returned color values are saved as new, user named attribute.", CategoryType.Filter, CategoryData.Mesh)]
    [PortInfo("UnstrGridIN", "UnstrGrid Input")]
    [PortInfo("UnstrGridOUT", "UnstrGrid Output - compound of input UnstrGrid + new color attribute.")]
    [PortInfo("ScalarOUT", "Output port with scalar values for ColorPalette.")]
    [PortInfo("ColorIN", "ColorRGBA value returned by palette.")]
    public class ColorMapingUnstr : Zcu.Mve.Core.Module
    {
        #region Variables declaration

        private String atributeName;
        private String newatributeName;
        private Boolean forcedPaletteReload;

        /// <summary>
        /// Name of the scalar attribute to be processed.
        /// </summary>
        public String AtributeName
        {
            set { atributeName = value; }
            get { return atributeName; }
        }

        /// <summary>
        /// Name of the new attribute containing colors which will be created.
        /// </summary>
        public String NewAtributeName
        {
            set { newatributeName = value; }
            get { return newatributeName; }
        }

        /// <summary>
        /// Sets whether the palette will be reloaded each time the module runs.
        /// </summary>
        public Boolean ForcedPaletteReload
        {
            set { forcedPaletteReload = value; }
            get { return forcedPaletteReload; }
        }

        private UnstrGrid outputGrid;
        #endregion

        /// <summary>
        /// Default constructor.
        /// </summary>
        public ColorMapingUnstr()
        {
            AddInPort("ColorRGBAIN", typeof(ColorRGBA),true, false);
            AddInPort("UnstrGridIN", typeof(UnstrGrid),true);
            AddOutPort("UnstrGridOut", typeof(UnstrGrid));
            AddOutPort("ScalarOut", typeof(Scalar),false);
        }

        List<ColorRGBA> polebarev = null;

        /// <summary>
        /// Entry point of module. By parameters choose CCL with or without preprocessing.
        /// </summary>
        public override void Execute()
        {
            if (IsDataSame("UnstrGridIN") && (!forcedPaletteReload))
            {
                SetOutput("UnstrGridOut", outputGrid, true);
            }
            else
            {
                if ((polebarev == null) || (forcedPaletteReload))
                {
                    polebarev = SamplePalet();
                }                
                
                ExecuteUnstrGrid(polebarev);                
            }                       
        }

        /// <summary>
        /// Load of UnstrGridInput and reading of attributes.
        /// </summary>
        private void ExecuteUnstrGrid(List<ColorRGBA> polebarev)
        {
            UnstrGrid inputUnstrGrid = (UnstrGrid)this.GetInput("UnstrGridIN");//load of input UnstrGrid
            
            if (inputUnstrGrid == null)//if no input Unstrgrid
            {
                throw new MveException(this, "Invalid input type or no input");//throw exception
            }


            UniformDataArray atributes;
            if ((atributeName == "") || (atributeName == null))
            {
                UniformDataArray[] atributesArray = inputUnstrGrid.GetPointAttrs(typeof(Scalar));
                if (atributesArray.Length == 0)
                {
                    atributesArray = inputUnstrGrid.GetPointAttrs(typeof(Integer));
                    if (atributesArray.Length == 0)
                    {
                        throw new MveException(this, "No scalar attribute in the data");
                    }
                    else
                    {
                        atributes = atributesArray[0];
                        CreateOutputUnstrGridFromUnstrGrid(atributes, inputUnstrGrid, polebarev);
                    }
                }
                else
                {
                    atributes = atributesArray[0];
                    CreateOutputUnstrGridFromUnstrGrid(atributes, inputUnstrGrid, polebarev);
                }
            }
            else
            {
                ICollection<string> ar = inputUnstrGrid.GetPointAttrsIds();//read names of attributes in unstrgrid
                if (ar.Contains(atributeName))
                {
                    atributes = inputUnstrGrid.GetPointAttrs(atributeName);
                    CreateOutputUnstrGridFromUnstrGrid(atributes, inputUnstrGrid, polebarev);
                }
            }            
        }

        private List<ColorRGBA> SamplePalet()
        {
            Scalar SampleRatio = (1 / 1000.0f), SampleBorder = 0;
            List<ColorRGBA> polebarev = new List<ColorRGBA>();


            for (int i = 0; i < 1000; i++)
            {
                SetOutput("ScalarOut", SampleBorder);
                SampleBorder += SampleRatio;
                UpdateInput("ColorRGBAIN");
                polebarev.Add((ColorRGBA)GetInput("ColorRGBAIN"));

            }
            return polebarev;
        }

        /// <summary>
        /// Method which tests which type of attribute data we have to process. It remaps values of attribute between zero and one and send scalar value on output,then gets ColorRGBA type to colorsAr.
        /// </summary>
        private void CreateOutputUnstrGridFromUnstrGrid(UniformDataArray atributes, UnstrGrid inputUnsterGrid, List<ColorRGBA> polebarev)
        {
            List<ColorRGBA> colorsAr = new List<ColorRGBA>();//list of colors

            outputGrid = inputUnsterGrid.ShallowCopy();
            
            if (atributes[0].GetType().ToString() == "Zcu.Mve.Numerics.Integer")//test on type of data(Integer)
            {
                double min = (double)(Integer)atributes[0];
                double max = (double)(Integer)atributes[0];
                double pom, pom2;
                for (int i = 0; i < atributes.Count; i++) //founding of minimum and maximum in attribute values
                {
                    if ((Integer)atributes[i] < min) min = (int)(Integer)atributes[i];
                    else if ((Integer)atributes[i] > max) max = (int)(Integer)atributes[i];
                }

                for (int i = 0; i < atributes.Count; i++)//remaping and getting color value
                {
                    pom2 = (Integer)atributes[i];

                    if (max - min != 0)
                    {
                        pom = ((pom2 + (-min)) / (max - min)) * 999;
                    }
                    else pom = 0;

                    pom = Math.Round(pom);

                    colorsAr.Add(polebarev[(int)pom]);
                }
            }
            else if (atributes[0].GetType().ToString() == "Zcu.Mve.Numerics.Scalar")//test on type of data(Scalar)
            {
                Scalar min = (Scalar)atributes[0];
                Scalar max = (Scalar)atributes[0];
                double pom, pom2;
                for (int i = 0; i < atributes.Count; i++) //founding of minimum and maximum in attribute values
                {
                    if ((Scalar)atributes[i] < min)
                        min = (Scalar)atributes[i];
                    else if ((Scalar)atributes[i] > max)
                        max = (Scalar)atributes[i];

                }

                for (int i = 0; i < atributes.Count; i++)//remaping and getting color value
                {
                    pom2 = (Scalar)atributes[i];

                    if (max - min != 0)
                    {
                        pom = ((pom2 + (-min)) / (max - min)) * 999;
                    }
                    else pom = 0;

                    pom = Math.Round(pom);

                    colorsAr.Add(polebarev[(int)pom]);
                }
            }
            else
            {
                throw new MveException(this, "Invalid atribute type! Supported types are Zcu.Mve.Numerics.Integer or Zcu.Mve.Numerics.Scalar.");
            }

            outputGrid.AddPointAttrs(newatributeName , UniformDataArray.FromList(colorsAr));//adding new attribute to output

            SetOutput("UnstrGridOut", outputGrid);//setting of output
        }
    }
}
