using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

//File with data types of ColorPointList and bod used by EditablePalette and PaletteGUI.

namespace Zcu.Mve.Visualization.Processing
{
    /// <summary>
    /// Data type, it consists of other data type ColorPoint. It is bidirectional connected list.
    /// </summary>
    public class ColorPointList 
    {
        ColorPoint first;
        ColorPoint last;

        /// <summary>
        /// constructs a new instance of the list
        /// </summary>
        /// <param name="first">Starting element of the list.</param>
        /// <param name="last">Ending element of the list.</param>
        public ColorPointList(ColorPoint first, ColorPoint last)//for every point in seznam point on predecessor and succesor
        {
            this.first = first;
            this.last = last;
        }

        /// <summary>
        /// Selects a ColorPoint at a given position, within given tolerance.
        /// </summary>
        /// <param name="x">X position of the selected point</param>
        /// <param name="y">X position of the selected point</param>
        /// <param name="epsilon">Tolerance</param>
        /// <returns>Selected ColorPoint</returns>
        public ColorPoint Select(int x, int y, int epsilon)//x pos, y pos, epsilon (range around point tfor drawing
        {
            ColorPoint pom = first;
            ColorPoint left = first;

            while (pom != null)
            {
                if (Math.Abs(pom.x - x) < epsilon)
                {
                    return pom;
                }
                else if (pom.x < x)
                {
                    left = pom;
                }
                else break;


                pom = pom.right;
            }
            float pomocna = 1.0f / (left.right.x - left.x);


            ColorPoint n = new ColorPoint(x, y, Color.FromArgb(255, (int)(pomocna * (left.right.x - x) * left.color.R + pomocna * (x - left.x) * left.right.color.R), (int)(pomocna * (left.right.x - x) * left.color.G + pomocna * (x - left.x) * left.right.color.G), (int)(pomocna * (left.right.x - x) * left.color.B + pomocna * (x - left.x) * left.right.color.B)), left, left.right);
            left.right.left = n;
            left.right = n;

            return n;
        }

        /// <summary>
        /// returns the first element of the list
        /// </summary>
        public ColorPoint First
        {
            get
            {
                return first;
            }
        }
    }

    /// <summary>
    /// Data type for every control point(CP) in palette
    /// </summary>
    public class ColorPoint
    {
        /// <summary>
        /// x position
        /// </summary>
        public int x;

        /// <summary>
        /// y position
        /// </summary>
        public int y;

        /// <summary>
        /// color of CP
        /// </summary>
        public Color color;

        /// <summary>
        /// predecessor in the list
        /// </summary>
        public ColorPoint left;

        /// <summary>
        /// succesor in the list
        /// </summary>
        public ColorPoint right;

        /// <summary>
        /// Constructs a new instance of color point
        /// </summary>
        /// <param name="x">X position of the new ColorPoint</param>
        /// <param name="y">Y position of the new ColorPoint (represents opacity)</param>
        /// <param name="color">Color of the new ColorPoint</param>
        /// <param name="left">Predecessor in the list of ColorPoint instances</param>
        /// <param name="right">Successor in the list of ColorPoint instances</param>
        public ColorPoint(int x, int y, Color color, ColorPoint left, ColorPoint right)
        {
            this.x = x;
            this.y = y;
            this.color = color;
            this.left = left;
            this.right = right;
        }
    }
}
