using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using Zcu.Mve.Visualization;
using System.ComponentModel;

namespace Zcu.Mve.Visualization
{
  /// <summary>
  /// Converts ColorRGBA from one data type to another. Access this class through the TypeDescriptor.
  /// </summary>
  public class ColorRGBAConverter : ColorConverter
  {
    /// <summary>
    /// Converts the given object to the converter's native type.
    /// </summary>
    /// <param name="context">
    /// A TypeDescriptor that provides a format context. 
    /// You can use this object to get additional information about the environment
    /// from which this converter is being invoked.
    /// </param>
    /// <param name="culture">A CultureInfo that specifies the culture to represent the color.</param>
    /// <param name="value">The object to convert.</param>
    /// <returns>An Object representing the converted value.</returns>
    public override object ConvertFrom(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value)
    {
      Color color = (Color) base.ConvertFrom(context, culture, value);

      return new ColorRGBA(color.R, color.G, color.B, color.A);
    }

    /// <summary>
    /// Converts the specified object to another type.
    /// </summary>
    /// <param name="context">
    /// A formatter context. Use this object to extract additional information about the environment from which this converter is being invoked.
    /// Always check whether this value is a null reference. Also, properties on the context object may return a null reference. 
    /// </param>
    /// <param name="culture">A CultureInfo that specifies the culture to represent the color.</param>
    /// <param name="value">The object to convert.</param>
    /// <param name="destinationType">The type to convert the object to.</param>
    /// <returns>An Object representing the converted value.</returns>
    public override object ConvertTo(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, Type destinationType)
    {
      ColorRGBA color = (ColorRGBA) value;
      value = Color.FromArgb(color.Alpha, color.Red, color.Green, color.Blue);

      return base.ConvertTo(context, culture, value, destinationType);
    }
  }
}
