using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Zcu.Mve.Visualization.Renderers
{
    class ImageForm : Form
    {
        private StatusBar statusBar1;
        private StatusBarPanel panel1;
        private TrackBar trackBarSlice;
        private Label labelPlane;
        private ComboBox comboBoxPlane;
        private CheckBox checkBoxInvert;
        private Button buttonZoomIn, buttonZoomOut;
        private VolumeSliceViewer viewer;

        public ImageForm(VolumeSliceViewer viewer)
        {
            this.viewer = viewer;
            this.FormBorderStyle = FormBorderStyle.FixedSingle;

            UpdateWindowTitle();

            this.MouseWheel += new MouseEventHandler(ImageForm_MouseWheel);

            this.trackBarSlice = new TrackBar();
            this.trackBarSlice.Location = new Point(0, viewer.bmp.Height);
            this.trackBarSlice.Minimum = 1;
            if (viewer.plane == Plane.YZ)
                this.trackBarSlice.Maximum = viewer.width;
            else if (viewer.plane == Plane.XZ)
                this.trackBarSlice.Maximum = viewer.height;
            else if (viewer.plane == Plane.XY)
                this.trackBarSlice.Maximum = viewer.slices;
            this.trackBarSlice.Size = new Size(200, 20);
            this.trackBarSlice.TabIndex = 0;
            this.trackBarSlice.TickStyle = TickStyle.None;
            this.trackBarSlice.Value = viewer.imageIndex + 1;
            this.trackBarSlice.SmallChange = 1;
            this.trackBarSlice.LargeChange = 10;
            this.trackBarSlice.ValueChanged += new EventHandler(trackBar_ValueChanged);
            this.Controls.Add(this.trackBarSlice);

            this.labelPlane = new Label();
            this.labelPlane.AutoSize = true;
            this.labelPlane.Location = new Point(trackBarSlice.Location.X + trackBarSlice.Width + 5, trackBarSlice.Location.Y + 10);
            this.labelPlane.Size = new Size(10, 13);
            this.labelPlane.TabIndex = 1;
            this.labelPlane.Text = "Plane ";
            this.Controls.Add(this.labelPlane);

            this.comboBoxPlane = new ComboBox();
            this.comboBoxPlane.FormattingEnabled = true;
            this.comboBoxPlane.Location = new Point(labelPlane.Location.X + labelPlane.Width + 5, trackBarSlice.Location.Y + 5);
            this.comboBoxPlane.Size = new Size(40, 10);
            this.comboBoxPlane.TabIndex = 2;
            this.comboBoxPlane.Items.Add("YZ");
            this.comboBoxPlane.Items.Add("XZ");
            this.comboBoxPlane.Items.Add("XY");
            this.comboBoxPlane.SelectedIndex = (byte)viewer.plane;
            this.comboBoxPlane.SelectedIndexChanged += new EventHandler(this.comboBoxPlane_SelectedIndexChanged);
            this.Controls.Add(this.comboBoxPlane);

            this.checkBoxInvert = new CheckBox();
            this.checkBoxInvert.AutoSize = true;
            this.checkBoxInvert.Location = new Point(labelPlane.Location.X, comboBoxPlane.Location.Y + comboBoxPlane.Height + 5);
            this.checkBoxInvert.Size = new Size(80, 17);
            this.checkBoxInvert.TabIndex = 3;
            this.checkBoxInvert.Text = "Invert ";
            this.checkBoxInvert.Checked = viewer.invert;
            this.checkBoxInvert.CheckAlign = ContentAlignment.MiddleRight;
            this.checkBoxInvert.CheckedChanged += new EventHandler(this.checkBoxInvert_CheckedChanged);
            this.Controls.Add(this.checkBoxInvert);

            this.buttonZoomIn = new Button();
            this.buttonZoomIn.Location = new Point(comboBoxPlane.Location.X + comboBoxPlane.Width + 5, trackBarSlice.Location.Y + 5);
            this.buttonZoomIn.Text = "Zoom in";
            this.buttonZoomIn.AutoSize = true;
            this.buttonZoomIn.TabIndex = 4;
            this.buttonZoomIn.Click += new EventHandler(this.buttonZoomIn_Click);
            this.Controls.Add(this.buttonZoomIn);

            this.buttonZoomOut = new Button();
            this.buttonZoomOut.Location = new Point(buttonZoomIn.Location.X, buttonZoomIn.Location.Y + buttonZoomIn.Height + 5);
            this.buttonZoomOut.Text = "Zoom out";
            this.buttonZoomOut.AutoSize = true;
            this.buttonZoomOut.TabIndex = 5;
            this.buttonZoomOut.Click += new EventHandler(this.buttonZoomOut_Click);
            this.Controls.Add(this.buttonZoomOut);

            this.panel1 = new StatusBarPanel();
            this.panel1.AutoSize = StatusBarPanelAutoSize.Spring;
            this.panel1.BorderStyle = StatusBarPanelBorderStyle.None;
            this.panel1.Text = " Resolution:  " + viewer.bmp.Width + " x " + viewer.bmp.Height + "   Dataset:  " + viewer.datasetName;
            this.panel1.AutoSize = StatusBarPanelAutoSize.Contents;
            this.statusBar1 = new StatusBar();
            this.statusBar1.Dock = DockStyle.Bottom;
            this.statusBar1.ShowPanels = true;
            this.statusBar1.Panels.Add(panel1);
            this.Controls.Add(statusBar1);

            ResizeForm();
        }

        private void SwitchCamera()
        {
            if (viewer.plane == Plane.YZ)
            {
                if (viewer.imageIndex > viewer.width)
                    viewer.imageIndex = viewer.width - 1;

                this.trackBarSlice.Maximum = viewer.width;
                panel1.Text = " Resolution:  " + viewer.height + " x " + viewer.slices + "   Dataset:  " + viewer.datasetName;
            }
            else if (viewer.plane == Plane.XZ)
            {
                if (viewer.imageIndex > viewer.height)
                    viewer.imageIndex = viewer.height - 1;

                this.trackBarSlice.Maximum = viewer.height;
                panel1.Text = " Resolution:  " + viewer.width + " x " + viewer.slices + "   Dataset:  " + viewer.datasetName;
            }
            else if (viewer.plane == Plane.XY)
            {
                if (viewer.imageIndex > viewer.slices)
                    viewer.imageIndex = viewer.slices - 1;

                this.trackBarSlice.Maximum = viewer.slices;
                panel1.Text = " Resolution:  " + viewer.width + " x " + viewer.height + "   Dataset:  " + viewer.datasetName;
            }

            UpdateWindowTitle();
            viewer.bmp = viewer.RegGridNDtoBitmap();
            ResizeForm();
        }

        private void TraverseForward()
        {
            if (viewer.plane == Plane.YZ)
            {
                if (viewer.imageIndex < viewer.width - 1)
                    viewer.imageIndex++;
                else
                    viewer.imageIndex = viewer.width - 1;
            }
            else if (viewer.plane == Plane.XZ)
            {
                if (viewer.imageIndex < viewer.height - 1)
                    viewer.imageIndex++;
                else
                    viewer.imageIndex = viewer.height - 1;
            }
            else if (viewer.plane == Plane.XY)
            {
                if (viewer.imageIndex < viewer.slices - 1)
                    viewer.imageIndex++;
                else
                    viewer.imageIndex = viewer.slices - 1;
            }

            UpdateWindowTitle();
            viewer.bmp = viewer.RegGridNDtoBitmap();
            this.Invalidate();
        }

        private void TraverseBackward()
        {
            if (viewer.plane == Plane.YZ)
            {
                if (viewer.imageIndex > 0)
                    viewer.imageIndex--;
                else
                    viewer.imageIndex = 0;
            }
            else if (viewer.plane == Plane.XZ)
            {
                if (viewer.imageIndex > 0)
                    viewer.imageIndex--;
                else
                    viewer.imageIndex = 0;
            }
            else if (viewer.plane == Plane.XY)
            {
                if (viewer.imageIndex > 0)
                    viewer.imageIndex--;
                else
                    viewer.imageIndex = 0;
            }

            UpdateWindowTitle();
            viewer.bmp = viewer.RegGridNDtoBitmap();
            this.Invalidate();
        }

        private void UpdateWindowTitle()
        {
            if (viewer.scale > 0)
                viewer.zoom = viewer.scale * 100;
            else if (viewer.scale < 0)
                viewer.zoom = 1f / (-viewer.scale) * 100;

            if (viewer.plane == Plane.YZ)
                this.Text = viewer.Name + "   " + (viewer.imageIndex + 1).ToString() + " / " + viewer.width + " (" + viewer.zoom + "%)";
            else if (viewer.plane == Plane.XZ)
                this.Text = viewer.Name + "   " + (viewer.imageIndex + 1).ToString() + " / " + viewer.height + " (" + viewer.zoom + "%)";
            else if (viewer.plane == Plane.XY)
                this.Text = viewer.Name + "   " + (viewer.imageIndex + 1).ToString() + " / " + viewer.slices + " (" + viewer.zoom + "%)";
        }

        private void ResizeForm()
        {
            panel1.AutoSize = StatusBarPanelAutoSize.Contents;
            this.trackBarSlice.Location = new Point(0, viewer.bmp.Height);
            this.labelPlane.Location = new Point(trackBarSlice.Location.X + trackBarSlice.Width + 5, trackBarSlice.Location.Y + 10);
            this.comboBoxPlane.Location = new Point(labelPlane.Location.X + labelPlane.Width + 5, trackBarSlice.Location.Y + 5);
            this.checkBoxInvert.Location = new Point(labelPlane.Location.X, comboBoxPlane.Location.Y + comboBoxPlane.Height + 5);
            this.buttonZoomIn.Location = new Point(comboBoxPlane.Location.X + comboBoxPlane.Width + 5, trackBarSlice.Location.Y + 5);
            this.buttonZoomOut.Location = new Point(buttonZoomIn.Location.X, buttonZoomIn.Location.Y + buttonZoomIn.Height + 5);
            this.ClientSize = new Size(Math.Max(viewer.bmp.Width, buttonZoomIn.Location.X + buttonZoomIn.Width + 10), viewer.bmp.Height + trackBarSlice.Height + 40);

            this.Invalidate();
        }

        #region Events

        protected override void OnLoad(EventArgs e)
        {
            this.SetStyle(ControlStyles.OptimizedDoubleBuffer | ControlStyles.UserPaint | ControlStyles.AllPaintingInWmPaint, true);
            this.UpdateStyles();
        }


        protected override void OnPaint(PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            g.DrawImage(viewer.bmp, 0, 0, viewer.bmp.Width, viewer.bmp.Height);
            base.OnPaint(e);
        }

        private void comboBoxPlane_SelectedIndexChanged(object sender, EventArgs e)
        {
            viewer.plane = (Plane)comboBoxPlane.SelectedIndex;

            SwitchCamera();
        }

        private void checkBoxInvert_CheckedChanged(object sender, EventArgs e)
        {
            viewer.invert = !viewer.invert;
            viewer.bmp = viewer.RegGridNDtoBitmap();
            this.Invalidate();
        }

        private void buttonZoomIn_Click(object sender, EventArgs e)
        {
            if (viewer.scale == -2)
                viewer.scale = 1;
            else
                viewer.scale++;

            UpdateWindowTitle();
            viewer.bmp = viewer.RegGridNDtoBitmap();
            ResizeForm();
        }

        private void buttonZoomOut_Click(object sender, EventArgs e)
        {
            if (viewer.scale == 1)
                viewer.scale = -2;
            else
                viewer.scale--;

            UpdateWindowTitle();
            viewer.bmp = viewer.RegGridNDtoBitmap();
            ResizeForm();
        }

        private void trackBar_ValueChanged(object sender, EventArgs e)
        {
            viewer.imageIndex = trackBarSlice.Value - 1;
            UpdateWindowTitle();
            viewer.bmp = viewer.RegGridNDtoBitmap();
            this.Invalidate();
        }

        private void ImageForm_MouseWheel(object sender, MouseEventArgs e)
        {
            trackBarSlice.Value = viewer.imageIndex + 1;
            if (e.Delta > 0)
                TraverseForward();
            else if (e.Delta < 0)
                TraverseBackward();
        }
        #endregion
      
        private void InitializeComponent()
        {
            this.SuspendLayout();
            // 
            // ImageForm
            // 
            this.ClientSize = new System.Drawing.Size(284, 264);
            this.Name = "ImageForm";
            this.ResumeLayout(false);
        }
    }
}