/*
 *		Jiri Skala		A01216
 *
 *		KIV / GSVD		2004/05
 */
using System;

namespace Zcu.Mve.Visualization.Processing
{
	/// <summary>
	/// <i>CalculateDisplace</i> module setup dialog.
	/// </summary>
	public class CalculateDisplaceSetup : Zcu.Mve.Core.ModuleSetup
	{
		#region Frame components

		private System.Windows.Forms.RadioButton radioButtonGray;
		private System.Windows.Forms.RadioButton radioButtonRed;
		private System.Windows.Forms.RadioButton radioButtonGreen;
		private System.Windows.Forms.RadioButton radioButtonBlue;
		private System.Windows.Forms.GroupBox groupBox1;
		private System.Windows.Forms.GroupBox groupBox2;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.TextBox textBoxColorAttr;
		private System.Windows.Forms.CheckBox checkBoxColorAttr;
		private System.Windows.Forms.TextBox textBoxScalarAttr;
		private System.Windows.Forms.CheckBox checkBoxScalarAttr;

		#endregion

		/// <summary>
		/// Reference to module which we set up.
		/// </summary>
		private CalculateDisplace parentModule;

		/// <summary>
		/// Constructor. Stores parent module reference, initializes GUI
		/// and loads parent module setup.
		/// </summary>
		/// <param name="parentModule">Parent module reference.</param>
		public CalculateDisplaceSetup(CalculateDisplace parentModule)
		{
			// Required for Windows Form Designer support
			InitializeComponent();

			// store parent module reference
			this.parentModule = parentModule;

			// check appropriate color function radio button
			switch (parentModule.SelectedColorFunction)
			{
				case ColorFunction.Gray:
					radioButtonGray.Checked = true;
					break;

				case ColorFunction.Red:
					radioButtonRed.Checked = true;
					break;

				case ColorFunction.Green:
					radioButtonGreen.Checked = true;
					break;

				case ColorFunction.Blue:
					radioButtonBlue.Checked = true;
					break;
			}

			// set color attribute name
			bool shouldBeDefault;
			textBoxColorAttr.Text = parentModule.GetColorAttrName(out shouldBeDefault);
			checkBoxColorAttr.Checked = shouldBeDefault;

			// set scalar attribute name
			textBoxScalarAttr.Text = parentModule.GetScalarAttrName(out shouldBeDefault);
			checkBoxScalarAttr.Checked = shouldBeDefault;
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.radioButtonGray = new System.Windows.Forms.RadioButton();
			this.radioButtonRed = new System.Windows.Forms.RadioButton();
			this.radioButtonGreen = new System.Windows.Forms.RadioButton();
			this.radioButtonBlue = new System.Windows.Forms.RadioButton();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.groupBox2 = new System.Windows.Forms.GroupBox();
			this.checkBoxScalarAttr = new System.Windows.Forms.CheckBox();
			this.textBoxScalarAttr = new System.Windows.Forms.TextBox();
			this.label2 = new System.Windows.Forms.Label();
			this.checkBoxColorAttr = new System.Windows.Forms.CheckBox();
			this.textBoxColorAttr = new System.Windows.Forms.TextBox();
			this.label1 = new System.Windows.Forms.Label();
			this.groupBox1.SuspendLayout();
			this.groupBox2.SuspendLayout();
			this.SuspendLayout();
			// 
			// radioButtonGray
			// 
			this.radioButtonGray.Location = new System.Drawing.Point(16, 24);
			this.radioButtonGray.Name = "radioButtonGray";
			this.radioButtonGray.Size = new System.Drawing.Size(72, 24);
			this.radioButtonGray.TabIndex = 1;
			this.radioButtonGray.Text = "Lightness";
			// 
			// radioButtonRed
			// 
			this.radioButtonRed.Location = new System.Drawing.Point(16, 48);
			this.radioButtonRed.Name = "radioButtonRed";
			this.radioButtonRed.Size = new System.Drawing.Size(112, 24);
			this.radioButtonRed.TabIndex = 2;
			this.radioButtonRed.Text = "Red channel only";
			// 
			// radioButtonGreen
			// 
			this.radioButtonGreen.Location = new System.Drawing.Point(16, 72);
			this.radioButtonGreen.Name = "radioButtonGreen";
			this.radioButtonGreen.Size = new System.Drawing.Size(120, 24);
			this.radioButtonGreen.TabIndex = 3;
			this.radioButtonGreen.Text = "Green channel only";
			// 
			// radioButtonBlue
			// 
			this.radioButtonBlue.Location = new System.Drawing.Point(16, 96);
			this.radioButtonBlue.Name = "radioButtonBlue";
			this.radioButtonBlue.Size = new System.Drawing.Size(112, 24);
			this.radioButtonBlue.TabIndex = 4;
			this.radioButtonBlue.Text = "Blue channel only";
			// 
			// groupBox1
			// 
			this.groupBox1.Controls.Add(this.radioButtonRed);
			this.groupBox1.Controls.Add(this.radioButtonGreen);
			this.groupBox1.Controls.Add(this.radioButtonBlue);
			this.groupBox1.Controls.Add(this.radioButtonGray);
			this.groupBox1.Location = new System.Drawing.Point(8, 8);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(184, 128);
			this.groupBox1.TabIndex = 5;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "Elevation map color processing";
			// 
			// groupBox2
			// 
			this.groupBox2.Controls.Add(this.checkBoxScalarAttr);
			this.groupBox2.Controls.Add(this.textBoxScalarAttr);
			this.groupBox2.Controls.Add(this.label2);
			this.groupBox2.Controls.Add(this.checkBoxColorAttr);
			this.groupBox2.Controls.Add(this.textBoxColorAttr);
			this.groupBox2.Controls.Add(this.label1);
			this.groupBox2.Location = new System.Drawing.Point(200, 8);
			this.groupBox2.Name = "groupBox2";
			this.groupBox2.Size = new System.Drawing.Size(200, 128);
			this.groupBox2.TabIndex = 6;
			this.groupBox2.TabStop = false;
			this.groupBox2.Text = "Attribute names";
			// 
			// checkBoxScalarAttr
			// 
			this.checkBoxScalarAttr.Location = new System.Drawing.Point(112, 96);
			this.checkBoxScalarAttr.Name = "checkBoxScalarAttr";
			this.checkBoxScalarAttr.Size = new System.Drawing.Size(80, 24);
			this.checkBoxScalarAttr.TabIndex = 5;
			this.checkBoxScalarAttr.Text = "Use default";
			this.checkBoxScalarAttr.CheckedChanged += new System.EventHandler(this.checkBox_CheckedChanged);
			// 
			// textBoxScalarAttr
			// 
			this.textBoxScalarAttr.Location = new System.Drawing.Point(8, 96);
			this.textBoxScalarAttr.Name = "textBoxScalarAttr";
			this.textBoxScalarAttr.Size = new System.Drawing.Size(96, 20);
			this.textBoxScalarAttr.TabIndex = 4;
			this.textBoxScalarAttr.Text = "textBox2";
			// 
			// label2
			// 
			this.label2.Location = new System.Drawing.Point(8, 72);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(168, 23);
			this.label2.TabIndex = 3;
			this.label2.Text = "Output elevation scalar attribute";
			this.label2.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// checkBoxColorAttr
			// 
			this.checkBoxColorAttr.Location = new System.Drawing.Point(112, 40);
			this.checkBoxColorAttr.Name = "checkBoxColorAttr";
			this.checkBoxColorAttr.Size = new System.Drawing.Size(80, 24);
			this.checkBoxColorAttr.TabIndex = 2;
			this.checkBoxColorAttr.Text = "Use default";
			this.checkBoxColorAttr.CheckedChanged += new System.EventHandler(this.checkBox_CheckedChanged);
			// 
			// textBoxColorAttr
			// 
			this.textBoxColorAttr.Location = new System.Drawing.Point(8, 40);
			this.textBoxColorAttr.Name = "textBoxColorAttr";
			this.textBoxColorAttr.Size = new System.Drawing.Size(96, 20);
			this.textBoxColorAttr.TabIndex = 1;
			this.textBoxColorAttr.Text = "textBox1";
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(8, 16);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(152, 23);
			this.label1.TabIndex = 0;
			this.label1.Text = "Elevation map color attribute";
			this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// CalculateDisplaceSetup
			// 
			this.Controls.Add(this.groupBox2);
			this.Controls.Add(this.groupBox1);
			this.Name = "CalculateDisplaceSetup";
			this.Size = new System.Drawing.Size(408, 144);
			this.groupBox1.ResumeLayout(false);
			this.groupBox2.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// Accepts changes and performs parent module setup.
		/// </summary>
		/// <returns>Returns true if setup is correct. Otherwise returns false.</returns>
		public override bool OnOkClick()
		{
			// set selected color processing function
			if (radioButtonGray.Checked)
				parentModule.SelectedColorFunction = ColorFunction.Gray;
			else if (radioButtonRed.Checked)
				parentModule.SelectedColorFunction = ColorFunction.Red;
			else if (radioButtonGreen.Checked)
				parentModule.SelectedColorFunction = ColorFunction.Green;
			else
				parentModule.SelectedColorFunction = ColorFunction.Blue;

			// set color attribute name
			parentModule.SetColorAttrName(textBoxColorAttr.Text, checkBoxColorAttr.Checked);

			// set scalar attribute name
			parentModule.SetScalarAttrName(textBoxScalarAttr.Text, checkBoxScalarAttr.Checked);

			return true;
		}

		/// <summary>
		/// Enables / disables default color / scalar attribute names.
		/// </summary>
		private void checkBox_CheckedChanged(object sender, System.EventArgs e)
		{
			textBoxColorAttr.Enabled = ! checkBoxColorAttr.Checked;
			textBoxScalarAttr.Enabled = ! checkBoxScalarAttr.Checked;
		}
	}
}