/*
 *		Jiri Skala		A01216
 *
 *		KIV / GSVD		2004/05
 */
using System;

namespace Zcu.Mve.Visualization.Processing
{
	/// <summary>
	/// <i>DisplaceMap</i> module setup dialog.
	/// </summary>
	public class DisplaceMapSetup : Zcu.Mve.Core.ModuleSetup
	{
		#region Frame components

		private System.Windows.Forms.GroupBox groupBox1;
		private System.Windows.Forms.RadioButton radioButton1stQuadrant;
		private System.Windows.Forms.GroupBox groupBox2;
		private System.Windows.Forms.RadioButton radioButtonCellSize;
		private System.Windows.Forms.RadioButton radioButtonMeshSize;
		private System.Windows.Forms.CheckBox checkBoxSquare;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.RadioButton radioButtonCentered;
		private System.Windows.Forms.Label label5;
		private System.Windows.Forms.NumericUpDown numUpDnElevFactor;
		private System.Windows.Forms.TextBox textBoxElevAttr;
		private System.Windows.Forms.CheckBox checkBoxElevAttr;
		private System.Windows.Forms.NumericUpDown numUpDnY;
		private System.Windows.Forms.NumericUpDown numUpDnX;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.GroupBox groupBox3;

		#endregion

		/// <summary>
		/// Reference to module which we set up.
		/// </summary>
		private DisplaceMap parentModule;

		/// <summary>
		/// Constructor. Stores parent module reference, initializes GUI
		/// and loads parent module setup.
		/// </summary>
		/// <param name="parentModule">Parent module reference.</param>
		public DisplaceMapSetup(DisplaceMap parentModule)
		{
			// Required for Windows Form Designer support
			InitializeComponent();

			// store parent module reference
			this.parentModule = parentModule;

			// set elevation factor
			numUpDnElevFactor.Value = (decimal)parentModule.ElevationFactor;

			// check appropriate radio button according to grid positioning
			if (parentModule.GridCentered)
				radioButtonCentered.Checked = true;
			else
				radioButton1stQuadrant.Checked = true;

			// get point elevation attribute name
			bool shouldBeDefault;
			textBoxElevAttr.Text = parentModule.GetElevAttrName(out shouldBeDefault);
			checkBoxElevAttr.Checked = shouldBeDefault;

			// get whether cells should be square
			checkBoxSquare.Checked = parentModule.SquareCells;

			// get output mesh size
			double x, y;
			if (parentModule.GetSize(out x, out y))
				radioButtonMeshSize.Checked = true;
			else
				radioButtonCellSize.Checked = true;
			// show mesh size
			numUpDnX.Value = (decimal)x;
			numUpDnY.Value = (decimal)y;
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.radioButtonCentered = new System.Windows.Forms.RadioButton();
			this.radioButton1stQuadrant = new System.Windows.Forms.RadioButton();
			this.groupBox2 = new System.Windows.Forms.GroupBox();
			this.groupBox3 = new System.Windows.Forms.GroupBox();
			this.checkBoxSquare = new System.Windows.Forms.CheckBox();
			this.label2 = new System.Windows.Forms.Label();
			this.radioButtonMeshSize = new System.Windows.Forms.RadioButton();
			this.radioButtonCellSize = new System.Windows.Forms.RadioButton();
			this.label1 = new System.Windows.Forms.Label();
			this.numUpDnY = new System.Windows.Forms.NumericUpDown();
			this.numUpDnX = new System.Windows.Forms.NumericUpDown();
			this.label5 = new System.Windows.Forms.Label();
			this.numUpDnElevFactor = new System.Windows.Forms.NumericUpDown();
			this.checkBoxElevAttr = new System.Windows.Forms.CheckBox();
			this.textBoxElevAttr = new System.Windows.Forms.TextBox();
			this.label3 = new System.Windows.Forms.Label();
			this.groupBox1.SuspendLayout();
			this.groupBox2.SuspendLayout();
			this.groupBox3.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.numUpDnY)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numUpDnX)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numUpDnElevFactor)).BeginInit();
			this.SuspendLayout();
			// 
			// groupBox1
			// 
			this.groupBox1.Controls.Add(this.radioButtonCentered);
			this.groupBox1.Controls.Add(this.radioButton1stQuadrant);
			this.groupBox1.Location = new System.Drawing.Point(8, 24);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(216, 80);
			this.groupBox1.TabIndex = 0;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "Output mesh position";
			// 
			// radioButtonCentered
			// 
			this.radioButtonCentered.Location = new System.Drawing.Point(16, 48);
			this.radioButtonCentered.Name = "radioButtonCentered";
			this.radioButtonCentered.Size = new System.Drawing.Size(112, 24);
			this.radioButtonCentered.TabIndex = 1;
			this.radioButtonCentered.Text = "Centered at origin";
			// 
			// radioButton1stQuadrant
			// 
			this.radioButton1stQuadrant.Location = new System.Drawing.Point(16, 24);
			this.radioButton1stQuadrant.Name = "radioButton1stQuadrant";
			this.radioButton1stQuadrant.Size = new System.Drawing.Size(168, 24);
			this.radioButton1stQuadrant.TabIndex = 0;
			this.radioButton1stQuadrant.Text = "From origin to first quadrant";
			// 
			// groupBox2
			// 
			this.groupBox2.Controls.Add(this.groupBox3);
			this.groupBox2.Controls.Add(this.groupBox1);
			this.groupBox2.Location = new System.Drawing.Point(8, 96);
			this.groupBox2.Name = "groupBox2";
			this.groupBox2.Size = new System.Drawing.Size(232, 264);
			this.groupBox2.TabIndex = 1;
			this.groupBox2.TabStop = false;
			this.groupBox2.Text = "Regular 2D grid input specific";
			// 
			// groupBox3
			// 
			this.groupBox3.Controls.Add(this.checkBoxSquare);
			this.groupBox3.Controls.Add(this.label2);
			this.groupBox3.Controls.Add(this.radioButtonMeshSize);
			this.groupBox3.Controls.Add(this.radioButtonCellSize);
			this.groupBox3.Controls.Add(this.label1);
			this.groupBox3.Controls.Add(this.numUpDnY);
			this.groupBox3.Controls.Add(this.numUpDnX);
			this.groupBox3.Location = new System.Drawing.Point(8, 112);
			this.groupBox3.Name = "groupBox3";
			this.groupBox3.Size = new System.Drawing.Size(216, 144);
			this.groupBox3.TabIndex = 7;
			this.groupBox3.TabStop = false;
			this.groupBox3.Text = "Output mesh size";
			// 
			// checkBoxSquare
			// 
			this.checkBoxSquare.Location = new System.Drawing.Point(16, 112);
			this.checkBoxSquare.Name = "checkBoxSquare";
			this.checkBoxSquare.Size = new System.Drawing.Size(197, 24);
			this.checkBoxSquare.TabIndex = 2;
			this.checkBoxSquare.Text = "Square cells, preserve aspect ratio";
			this.checkBoxSquare.CheckedChanged += new System.EventHandler(this.checkBoxSquare_CheckedChanged);
			// 
			// label2
			// 
			this.label2.Location = new System.Drawing.Point(16, 80);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(16, 23);
			this.label2.TabIndex = 6;
			this.label2.Text = "X";
			this.label2.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// radioButtonMeshSize
			// 
			this.radioButtonMeshSize.Location = new System.Drawing.Point(16, 48);
			this.radioButtonMeshSize.Name = "radioButtonMeshSize";
			this.radioButtonMeshSize.Size = new System.Drawing.Size(136, 24);
			this.radioButtonMeshSize.TabIndex = 1;
			this.radioButtonMeshSize.Text = "Set overall mesh size";
			// 
			// radioButtonCellSize
			// 
			this.radioButtonCellSize.Location = new System.Drawing.Point(16, 24);
			this.radioButtonCellSize.Name = "radioButtonCellSize";
			this.radioButtonCellSize.Size = new System.Drawing.Size(120, 24);
			this.radioButtonCellSize.TabIndex = 0;
			this.radioButtonCellSize.Text = "Set single cell size";
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(120, 80);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(16, 23);
			this.label1.TabIndex = 5;
			this.label1.Text = "Y";
			this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// numUpDnY
			// 
			this.numUpDnY.DecimalPlaces = 1;
			this.numUpDnY.Increment = new System.Decimal(new int[] {
																	   5,
																	   0,
																	   0,
																	   0});
			this.numUpDnY.Location = new System.Drawing.Point(136, 80);
			this.numUpDnY.Maximum = new System.Decimal(new int[] {
																	 500,
																	 0,
																	 0,
																	 0});
			this.numUpDnY.Name = "numUpDnY";
			this.numUpDnY.Size = new System.Drawing.Size(64, 20);
			this.numUpDnY.TabIndex = 4;
			// 
			// numUpDnX
			// 
			this.numUpDnX.DecimalPlaces = 1;
			this.numUpDnX.Increment = new System.Decimal(new int[] {
																	   5,
																	   0,
																	   0,
																	   0});
			this.numUpDnX.Location = new System.Drawing.Point(32, 80);
			this.numUpDnX.Maximum = new System.Decimal(new int[] {
																	 500,
																	 0,
																	 0,
																	 0});
			this.numUpDnX.Name = "numUpDnX";
			this.numUpDnX.Size = new System.Drawing.Size(64, 20);
			this.numUpDnX.TabIndex = 3;
			// 
			// label5
			// 
			this.label5.Location = new System.Drawing.Point(16, 8);
			this.label5.Name = "label5";
			this.label5.Size = new System.Drawing.Size(88, 23);
			this.label5.TabIndex = 14;
			this.label5.Text = "Elevation factor";
			this.label5.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// numUpDnElevFactor
			// 
			this.numUpDnElevFactor.DecimalPlaces = 1;
			this.numUpDnElevFactor.Increment = new System.Decimal(new int[] {
																				5,
																				0,
																				0,
																				0});
			this.numUpDnElevFactor.Location = new System.Drawing.Point(104, 8);
			this.numUpDnElevFactor.Maximum = new System.Decimal(new int[] {
																			  500,
																			  0,
																			  0,
																			  0});
			this.numUpDnElevFactor.Minimum = new System.Decimal(new int[] {
																			  500,
																			  0,
																			  0,
																			  -2147483648});
			this.numUpDnElevFactor.Name = "numUpDnElevFactor";
			this.numUpDnElevFactor.Size = new System.Drawing.Size(72, 20);
			this.numUpDnElevFactor.TabIndex = 15;
			// 
			// checkBoxElevAttr
			// 
			this.checkBoxElevAttr.Location = new System.Drawing.Point(152, 64);
			this.checkBoxElevAttr.Name = "checkBoxElevAttr";
			this.checkBoxElevAttr.Size = new System.Drawing.Size(80, 24);
			this.checkBoxElevAttr.TabIndex = 1;
			this.checkBoxElevAttr.Text = "Use default";
			this.checkBoxElevAttr.CheckedChanged += new System.EventHandler(this.checkBoxElevAttr_CheckedChanged);
			// 
			// textBoxElevAttr
			// 
			this.textBoxElevAttr.Location = new System.Drawing.Point(16, 64);
			this.textBoxElevAttr.Name = "textBoxElevAttr";
			this.textBoxElevAttr.Size = new System.Drawing.Size(128, 20);
			this.textBoxElevAttr.TabIndex = 0;
			this.textBoxElevAttr.Text = "textBox1";
			// 
			// label3
			// 
			this.label3.Location = new System.Drawing.Point(16, 40);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(128, 23);
			this.label3.TabIndex = 2;
			this.label3.Text = "Elevation attribute name";
			this.label3.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// DisplaceMapSetup
			// 
			this.Controls.Add(this.groupBox2);
			this.Controls.Add(this.textBoxElevAttr);
			this.Controls.Add(this.label5);
			this.Controls.Add(this.label3);
			this.Controls.Add(this.numUpDnElevFactor);
			this.Controls.Add(this.checkBoxElevAttr);
			this.Name = "DisplaceMapSetup";
			this.Size = new System.Drawing.Size(248, 368);
			this.groupBox1.ResumeLayout(false);
			this.groupBox2.ResumeLayout(false);
			this.groupBox3.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.numUpDnY)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numUpDnX)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numUpDnElevFactor)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// Accepts changes and performs parent module setup.
		/// </summary>
		/// <returns>Returns true if setup is correct. Otherwise returns false.</returns>
		public override bool OnOkClick()
		{
			// set elevation factor
			parentModule.ElevationFactor = (double)numUpDnElevFactor.Value;

			// set grid positioning
			parentModule.GridCentered = radioButtonCentered.Checked;

			// set point elevation attribute name
			parentModule.SetElevAttrName(textBoxElevAttr.Text, checkBoxElevAttr.Checked);

			// set output mesh size
			parentModule.SetSize((double)numUpDnX.Value, (double)numUpDnY.Value,
				radioButtonMeshSize.Checked);

			// set whether cells should be square
			parentModule.SquareCells = checkBoxSquare.Checked;

			return true;
		}

		/// <summary>
		/// Enables / disables default elevation attribute name.
		/// </summary>
		private void checkBoxElevAttr_CheckedChanged(object sender, System.EventArgs e)
		{
			textBoxElevAttr.Enabled = ! checkBoxElevAttr.Checked;
		}

		/// <summary>
		/// Enables / disables Y size setting.
		/// </summary>
		private void checkBoxSquare_CheckedChanged(object sender, System.EventArgs e)
		{
			numUpDnY.Enabled = ! checkBoxSquare.Checked;
		}
	}
}