using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Zcu.Mve.Core;

namespace Zcu.Mve.Visualization.Processing
{
    /// <summary>
    /// Gui for cone glyph
    /// </summary>
    public partial class ConeGui : GlyphGui
    {
        #region Variables Declaration
        // cone attributes
        private bool capping = Cone.DefCapping;
        /// <summary>
        /// If true cone with capping will geberated
        /// </summary>
        public bool Capping
        {
            get { return capping; }
        }

        private double radius = Cone.DefRadius;
        /// <summary>
        /// Cone Radius
        /// </summary>
        public double Radius
        {
            get { return radius; }
        }

        private double heigth = Cone.DefHeigth;
        /// <summary>
        /// Cone Height
        /// </summary>
        public double Heigth
        {
            get { return heigth; }
        }

        private int resolution = Cone.DefResolution;
        /// <summary>
        /// Cone resolution
        /// </summary>
        public int Resolution
        {
            get { return resolution; }
        }

        private Center center = Cone.DefCenter;
        /// <summary>
        /// Cone center
        /// </summary>
        public Center Center
        {
            get { return center; }
        }
        #endregion
        
        /// <summary>
        /// Default constructor
        /// </summary>
        public ConeGui()
        {
            InitializeComponent();
            InitGuiValues();
        }

        /// <summary>
        /// Return an cone of currnet arrow glyph paramatres
        /// </summary>
        /// <returns>An cone of currnet arrow glyph paramatres</returns>
        public override List<string> GetParametrs()
        {
            List<string> paramets = new List<string>();
            paramets.Add(resolution.ToString());
            paramets.Add(radius.ToString(Globals.nfi));
            paramets.Add(heigth.ToString(Globals.nfi));
            paramets.Add(center.x.ToString(Globals.nfi));
            paramets.Add(center.y.ToString(Globals.nfi));
            paramets.Add(center.z.ToString(Globals.nfi));
            paramets.Add(capping.ToString());

            return paramets;
        }

        /// <summary>
        /// Sets cone gui attributes according to values in input array
        /// </summary>
        /// <param name="paramets"> new gui values</param>
        public override void SetParametrs(List<string> paramets)
        {
            resolution = Int32.Parse(paramets[0], Globals.nfi);
            radius = Double.Parse(paramets[1], Globals.nfi);
            heigth = Double.Parse(paramets[2], Globals.nfi);
            center.x = Double.Parse(paramets[3], Globals.nfi);
            center.y = Double.Parse(paramets[4], Globals.nfi);
            center.z = Double.Parse(paramets[5], Globals.nfi);
            capping = Boolean.Parse(paramets[6]);
            InitGuiValues();
        }


        /// <summary>
        /// Sets cone attributes to gui components
        /// </summary>
        private void InitGuiValues()
        {
            resolutionTBar.Value = resolution;
            resolutionTBox.Text = resolution.ToString(Globals.nfi);
            radiusTBox.Text = radius.ToString(Globals.nfi);
            heightTBox.Text = heigth.ToString(Globals.nfi);
            centerXTBox.Text = center.x.ToString(Globals.nfi);
            centerYTBox.Text = center.y.ToString(Globals.nfi);
            centerZTBox.Text = center.z.ToString(Globals.nfi);
            cappingChBox.Checked = capping;
        }

        private void resolutionTBar_Scroll(object sender, EventArgs e)
        {
            resolutionTBox.Text = resolutionTBar.Value.ToString();
            resolutionTBox_LostFocus(null, null);
        }

        private void radiusTBox_LostFocus(object sender, EventArgs e)
        {
            bool succeeded = ParseDouble(ref radius, radiusTBox.Text, "Radius must be positive double", false);
            if (!succeeded)
            {
                radiusTBox.Text = Cone.DefRadius.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }
        }

        private void heightTBox_LostFocus(object sender, EventArgs e)
        {
            bool succeeded = ParseDouble(ref heigth, heightTBox.Text, "Height must be positive double", false);
            if (!succeeded)
            {
                heightTBox.Text = Cone.DefHeigth.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }
        }

        private void centerXTBox_LostFocus(object sender, EventArgs e)
        {
            bool succeeded = ParseDouble(ref center.x, centerXTBox.Text, "Coordinates must be positive integers", true);
            if (!succeeded)
            {
                centerXTBox.Text = Cone.DefCenter.x.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }
        }

        private void centerYTBox_LostFocus(object sender, EventArgs e)
        {
            bool succeeded = ParseDouble(ref center.y, centerYTBox.Text, "Coordinates must be positive integers", true);
            if (!succeeded)
            {
                centerYTBox.Text = Cone.DefCenter.y.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }
        }

        private void centerZTBox_LostFocus(object sender, EventArgs e)
        {
            bool succeeded = ParseDouble(ref center.z, centerZTBox.Text, "Coordinates must be positive integers", true);
            if (!succeeded)
            {
                centerZTBox.Text = Cone.DefCenter.z.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }
        }

        private void cappingChBox_CheckedChanged(object sender, EventArgs e)
        {
            capping = cappingChBox.Checked;
            glyphParametrsChanged = true;
        }

        private void resolutionTBox_LostFocus(object sender, EventArgs e)
        {
            bool succeeded = ParseInt(ref resolution, resolutionTBox.Text, "Coordinates must be positive integers");
            if (!succeeded)
            {
                resolutionTBox.Text = Cone.DefResolution.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }
        }
    }
}
