using System;
using System.Collections.Generic;
using Zcu.Mve.Visualization;
using System.Collections;

namespace Zcu.Mve.Visualization.Processing
{
    /// <summary>
    /// Represents cylinder glyph
    /// </summary>
    class Cylinder : Glyph3D
    {
        #region Variables Declaration
        // default culinder attributes values
        private static double defRadius = 0.01;
        /// <summary>
        /// Default Cylinder radius
        /// </summary>
        public static double DefRadius
        {
            get { return defRadius; }
        }

        private static bool defCapping = true;
        /// <summary>
        /// Default Cylinder capping
        /// </summary>
        public static bool DefCapping
        {
            get { return defCapping; }
        }

        private static double defHeigth = 0.1;
        /// <summary>
        /// Default Cylinder Height
        /// </summary>
        public static double DefHeigth
        {
            get { return defHeigth; }
        }

        private static int defResolution = 6;
        /// <summary>
        /// Default Cylinder resolution
        /// </summary>
        public static int DefResolution
        {
            get { return defResolution; }
        }

        private static Center defCenter = new Center(0, 0, 0);
        /// <summary>
        /// Default Cylinder center
        /// </summary>
        public static Center DefCenter
        {
            get { return defCenter; }
        }
        #endregion

        /// <summary>
        /// default constructor
        /// </summary>
        /// <param name="resolution"> cylinder resolution</param>
        /// <param name="heigth"> cylinder height</param>
        /// <param name="radius"> cylinder radius</param>
        /// <param name="capping"> if true cylinder bottom capping is used</param>
        /// <param name="center"> center of the glyph</param>
        public Cylinder(int resolution, double heigth, double radius, bool capping, Center center)
        {
            double zTop = heigth / 2.0;
            double zBottom = heigth / -2.0;

            double phi = 0;
            double phiStep = 2 * Math.PI / resolution;
            for (int i = 0; i < resolution; i++)
            {
                glyphPoints.Add(new Point3D(radius * Math.Sin(phi) + center.x, radius * Math.Cos(phi) + center.y, zTop + center.z));
                glyphPoints.Add(new Point3D(radius * Math.Sin(phi) + center.x, radius * Math.Cos(phi) + center.y, zBottom + center.z));
                phi += phiStep;
            }

            glyphPoints.Add(new Point3D(0 + center.x, 0 + center.y, zTop + center.z));
            glyphPoints.Add(new Point3D(0 + center.x, 0 + center.y, zBottom + center.z));

            int index;
            if (capping)
            {
                for (int i = 0; i < resolution - 1; i++)
                {
                    index = i * 2;
                    glyphCells.Add(new Triangle(index + 2, resolution * 2, index));
                    glyphCells.Add(new Triangle(index + 1, resolution * 2 + 1, index + 3));
                }
                glyphCells.Add(new Triangle(0, resolution * 2, (resolution - 1) * 2));
                glyphCells.Add(new Triangle((resolution - 1) * 2 + 1, resolution * 2 + 1, 1));

            }
            for (int i = 0; i < (resolution - 1) * 2; i += 2)
            {
                glyphCells.Add(new Triangle(i, i + 1, i + 2));
                glyphCells.Add(new Triangle(i + 2, i + 1, i + 3));

            }
            glyphCells.Add(new Triangle((resolution - 1) * 2, (resolution - 1) * 2 + 1, 0));
            glyphCells.Add(new Triangle(0, (resolution - 1) * 2 + 1, 1));
        }
    }
}
