using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Zcu.Mve.Core;

namespace Zcu.Mve.Visualization.Processing
{
    /// <summary>
    /// Gui for cylinder glyph
    /// </summary>
    public partial class CylinderGui : GlyphGui
    {
        #region Variables Declarations
        // cylinder glyph attributes 
        private bool capping = Cylinder.DefCapping;
        /// <summary>
        /// If true cylinders with capping are generated
        /// </summary>
        public bool Capping
        {
            get { return capping; }
        }

        private double radius = Cylinder.DefRadius;
        /// <summary>
        /// Radius of the cylinder
        /// </summary>
        public double Radius
        {
            get { return radius; }
        }

        private double height = Cylinder.DefHeigth;
        /// <summary>
        /// Height of the cylinder
        /// </summary>
        public double Heigth
        {
            get { return height; }
        }

        private int resolution = Cylinder.DefResolution;
        /// <summary>
        /// Resoulution of the cylinder
        /// </summary>
        public int Resolution
        {
            get { return resolution; }
        }

        private Center center = Cylinder.DefCenter;
        /// <summary>
        /// Center of the cylinder
        /// </summary>
        public Center Center
        {
            get { return center; }
        }
        #endregion

        /// <summary>
        /// Default constructor
        /// </summary>
        public CylinderGui()
        {
            InitializeComponent();
            InitGuiValues();
        }

        /// <summary>
        /// Return an cylinder of currnet arrow glyph paramatres
        /// </summary>
        /// <returns>An cylinder of currnet arrow glyph paramatres</returns>
        public override List<string> GetParametrs()
        {
            List<string> paramets = new List<string>();
            paramets.Add(height.ToString(Globals.nfi));
            paramets.Add(radius.ToString(Globals.nfi));
            paramets.Add(center.x.ToString(Globals.nfi));
            paramets.Add(center.y.ToString(Globals.nfi));
            paramets.Add(center.z.ToString(Globals.nfi));
            paramets.Add(capping.ToString());
            return paramets;
        }

        /// <summary>
        /// Sets cylinder gui attributes according to values in input array
        /// </summary>
        /// <param name="paramets"> new gui values</param>
        public override void SetParametrs(List<string> paramets)
        {
            height = Double.Parse(paramets[0], Globals.nfi);
            radius = Double.Parse(paramets[1], Globals.nfi);
            center.x = Double.Parse(paramets[2], Globals.nfi);
            center.y = Double.Parse(paramets[3], Globals.nfi);
            center.z = Double.Parse(paramets[4], Globals.nfi);
            capping = Boolean.Parse(paramets[5]);
            InitGuiValues();
        }

        /// <summary>
        /// Sets cylinder attributes to gui components
        /// </summary>
        private void InitGuiValues()
        {
            resolutionTBox.Text = resolution.ToString(Globals.nfi);
            resolutionTBar.Value = resolution;
            heightTBox.Text = height.ToString(Globals.nfi);
            radiusTBox.Text = radius.ToString(Globals.nfi);
            centerXTBox.Text = center.x.ToString(Globals.nfi);
            centerYTBox.Text = center.y.ToString(Globals.nfi);
            centerZTBox.Text = center.z.ToString(Globals.nfi);
            cappingCBox.Checked = capping;
        }

        private void resolutionTBox_LostFocus(object sender, EventArgs e)
        {
            bool succeeded = ParseInt(ref resolution, resolutionTBox.Text, "Coordinates must be positive integers");
            if (!succeeded)
            {
                resolutionTBox.Text = Cylinder.DefResolution.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }
        }

        private void radiusTBox_LostFocus(object sender, EventArgs e)
        {
            bool succeeded = ParseDouble(ref radius, radiusTBox.Text, "Coordinates must be positive integers", false);
            if (!succeeded)
            {
                radiusTBox.Text = Cylinder.DefRadius.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }
        }

        private void heightTBox_LostFocus(object sender, EventArgs e)
        {
            bool succeeded = ParseDouble(ref height, heightTBox.Text, "Coordinates must be positive integers", false);
            if (!succeeded)
            {
                heightTBox.Text = Cylinder.DefHeigth.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }
        }

        private void centerXTBox_LostFocus(object sender, EventArgs e)
        {
            bool succeeded = ParseDouble(ref center.x, centerXTBox.Text, "Coordinates must be positive integers", true);
            if (!succeeded)
            {
                centerXTBox.Text = Cylinder.DefCenter.x.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }
        }

        private void centerYTBox_LostFocus(object sender, EventArgs e)
        {
            bool succeeded = ParseDouble(ref center.y, centerYTBox.Text, "Coordinates must be positive integers", true);
            if (!succeeded)
            {
                centerYTBox.Text = Cylinder.DefCenter.y.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }
        }

        private void centeZTBox_LostFocus(object sender, EventArgs e)
        {
            bool succeeded = ParseDouble(ref center.z, centerZTBox.Text, "Coordinates must be positive integers", true);
            if (!succeeded)
            {
                centerZTBox.Text = Cylinder.DefCenter.z.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }
        }

        private void cappingCBox_CheckedChanged(object sender, EventArgs e)
        {
            capping = cappingCBox.Checked;
            glyphParametrsChanged = true;
        }

        private void resolutionTBar_Scroll(object sender, EventArgs e)
        {
            resolutionTBox.Text = resolutionTBar.Value.ToString();
            resolutionTBox_LostFocus(null, null);
        }
    }
}
