using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace Zcu.Mve.Visualization.Processing
{
    /// <summary>
    /// Structure representing glyphs centers
    /// </summary>
    public struct Center
    {
        /// <summary>
        /// Default contructor
        /// </summary>
        /// <param name="x"> x coordinate of the center</param>
        /// <param name="y"> y coordinate of the center</param>
        /// <param name="z"> z coordinate of the center</param>
        public Center(double x, double y, double z)
        {
            this.x = x;
            this.y = y;
            this.z = z;
        }
        /// <summary>
        /// X coordinate of the center
        /// </summary>
        public double x;
        /// <summary>
        /// Y coordinate of the center
        /// </summary>
        public double y;
        /// <summary>
        /// Z coordinate of the center
        /// </summary>
        public double z;
    }

    /// <summary>
    /// Ancester for all glyphs guis, providing basic functions for text data parsing
    /// </summary>
    public abstract class GlyphGui : UserControl
    {
        /// <summary>
        /// true if glyphs parameters have changed
        /// </summary>
        protected bool glyphParametrsChanged = false;
        /// <summary>
        /// return true if glyphs parameters have changed
        /// </summary>
        public bool GlyphParametrsChanged
        {
            get { return glyphParametrsChanged; }
            set { glyphParametrsChanged = value; } 
        }
        /// <summary>
        /// Function for parsing int values
        /// </summary>
        /// <param name="value"> return value</param>
        /// <param name="text"> text to be parsed</param>
        /// <param name="errorMessage"> text that appears in the error if parsing fails</param>
        /// <returns>true if parsing succeeded</returns>
        protected bool ParseInt(ref int value, string text, string errorMessage)
        {
            int tmp;
            try
            {
                tmp = Int32.Parse(text);
                if (tmp >= 0)
                {
                    value = tmp;
                    return true;
                }
                else
                    MessageBox.Show(errorMessage);
            }
            catch (Exception ex)
            {
                MessageBox.Show(errorMessage + ex.Message);
               
            }
            return false;
        }

        /// <summary>
        /// Function that return array of glyphGui parametrs
        /// </summary>
        /// <returns></returns>
        public abstract List<string> GetParametrs();

        /// <summary>
        /// Function that sets new glyph parameters
        /// </summary>
        /// <param name="parametrs"> array that contains new parametrs</param>
        public abstract void SetParametrs(List<string> parametrs);

        /// <summary>
        /// Function for parsing int values
        /// </summary>
        /// <param name="value"> return value</param>
        /// <param name="text"> text to be parsed</param>
        /// <param name="errorMessage"> text that appears in the error if parsing fails</param>
        /// <param name="minus"> if true negativ values are accepted</param>
        /// <returns> true if parsing succeeded</returns>
        protected bool ParseDouble(ref double value, string text, string errorMessage, bool minus)
        {
            double tmp;
            try
            {
                tmp = Double.Parse(text);
                if (minus || (tmp >= 0))
                {
                    value = tmp;
                    return true;
                }
                else
                    MessageBox.Show(errorMessage);
            }
            catch (Exception ex)
            {
                MessageBox.Show(errorMessage+ex.Message );
            }
            return false;
        }   
    }
}
