using System;
using System.Collections.Generic;
using System.Text;
using Zcu.Mve.Core;
using Zcu.Mve.Visualization;
using Zcu.Mve.Numerics;
using System.Collections;
namespace Zcu.Mve.Visualization.Processing
{
    /// <summary>
    /// Module can transfoms Normal3D or Vector3D or Scalar or Integer point atributes to glyphs
    /// </summary>
    [ModuleInfo("Vaclav Bystricky", " Module can transfoms Normal3D or Vector3D or Scalar or Integer point atributes to glyphs", CategoryType.Filter, CategoryData.Volume, IconName = "Glyph.ico")]
    [PortInfo("RegGridNDIn", "RegGridND from which point attributes will be created glyphs ")]
    [PortInfo("UnstrGridGlyphIn", "User defined glyph")]
    [PortInfo("UnstrGridOut", "Output Unstrgrid that contains glyphs")]
    class GlyphsFromRegGridNd : Glyphs
    {
        #region Init

        /// <summary>
        /// Function that adds ports to module
        /// </summary>
        protected override void AddPorts()
        {
            AddInPort("RegGridNDIn", typeof(RegGridND));
            AddInPort("UnstrGridGlyphIn", typeof(UnstrGrid), false);
            AddOutPort("UnstrGridOut", typeof(UnstrGrid));
        }

        /// <summary>
        /// Function tha check input attribute parametrs and creates glyphs
        /// </summary>
        protected override void ProccesInput()
        {
            RegGridND inputRegGridND = (RegGridND)this.GetInput("RegGridNDIn");
            if (inputRegGridND == null)
            {
                throw new MveException(this, "Invalid input, only Unstrgrid is needed");
            }

            if (inputRegGridND.Dimensions.Length != 3)// check grid dimesion
                throw new MveException(this, "Incorrect grid Dimension: " + inputRegGridND.Dimensions.Length + ", only 3D grid are allowed");
            ICollection ar = inputRegGridND.GetPointAttrIds();
            String[] pointAtrIDs = new string[ar.Count];
            ar.CopyTo(pointAtrIDs, 0);
            for (int i = 0; i < pointAtrIDs.Length; i++)
            {
                if (pointAtrIDs[i].Equals(AttributeName))//check atribute name
                {
                    MveArray attributes = inputRegGridND.GetPointAttrs(pointAtrIDs[i]);
                    if (attributes.ElemType.Name.Equals(AttributeType))//check atribute type
                    {
                        CreateOutputUnstrGridFromRegGridND(ref inputRegGridND, ref  attributes);// computes glyphs
                        return;
                    }
                }
            }
            // if attributes do not match, an exception is thrown
            throw new MveException(this, "Atribute " + AttributeName + " (" + AttributeType + ") wasn't found");
        }

        /// <summary>
        /// Creates glyphs from inputReggridND points and their attributes.
        /// </summary>
        /// <param name="inputReggridND"> Grid from which glyph posotions will be determined</param>
        /// <param name="attributes"> atrbutes that deternime glyph size and direction</param>
        private void CreateOutputUnstrGridFromRegGridND(ref RegGridND inputReggridND, ref MveArray attributes)
        {
            GetInputGlyph();
            outputGrid = new UnstrGrid();// output unstrgrid

            int numOfGlyphs;
            if (MaskPoints)
                numOfGlyphs = Math.Min(MaxGlyphs, GetNumOFPoints(ref inputReggridND));
            else
                numOfGlyphs = GetNumOFPoints(ref inputReggridND);

            UniformDataArray glyphsPoints = new UniformDataArray(typeof(Point3D), numOfGlyphs *glyph.GlyphPoints.Count);// glyph points
            UniformDataArray glyphsCells = new UniformDataArray(typeof(Triangle),numOfGlyphs * glyph.GlyphCells.Count);// glyph cells

            UniformDataArray glyphSourcePointsIndexes = null;// indexces of points from which glyphs were created
            UniformDataArray originalAtributes = null;
            if (KeepAtributes)
                originalAtributes = new UniformDataArray( attributes.ElemType, numOfGlyphs * glyph.GlyphPoints.Count);
            // computation of glyphs
            AddGlyphs(ref glyphsPoints, ref glyphsCells, ref inputReggridND, ref attributes, ref glyph,
                      ref glyphSourcePointsIndexes, ref originalAtributes);

            outputGrid.AddCells("Triangle", glyphsCells);
            outputGrid.Points = glyphsPoints;
            if (AddIndexes)
             //   outputGrid.AddPointAttrs(indexAttributeName, UniformDataArray.FromList(glyphSourcePointsIndexes));
            if (KeepAtributes)//add orig attributes
                   outputGrid.AddPointAttrs(OriginalAttributeName, originalAtributes);
            if (!outputGrid.CheckConsistence())
            {
                throw new MveException(this, "");
            }
            SetOutput("UnstrGridOut", outputGrid);
        }
        #endregion
        #region AddGlyphs

        /// <summary>
        ///  Computes index of next point. Can work in random or normal mode.
        /// </summary>
        /// <param name="step"> </param>
        /// <param name="rand"> generator of random numbers</param>
        /// <param name="xTmp"> x coordinate used for storing x in float </param>
        /// <param name="yTmp"> y coordinate used for storing y in float </param>
        /// <param name="zTmp"> z coordinate used for storing z in float </param>
        /// <param name="xMax"> maximal vaule of x coordinate</param>
        /// <param name="yMax"> maximal vaule of y coordinate</param>
        /// <param name="zMax"> maximal vaule of z coordinate</param>
        private void GetIndex(float step, ref Random rand, ref float xTmp, ref float yTmp, ref float zTmp
                         , int xMax, int yMax, int zMax)
        {
            if (RandomMode && step != 1)// random mode
            {
                int index;
                do
                {
                    xTmp = rand.Next(xMax);
                    yTmp = rand.Next(yMax);
                    zTmp = rand.Next(zMax);
                    index = (int)(zTmp + yTmp * zMax + xTmp * yMax * zMax);
                } while (glyph.UsedIndexes[index] != false);
                glyph.UsedIndexes[index] = true;
            }
            else// normal mode
            {
                xTmp += step;
                if (xTmp >= xMax)
                {
                    int tmp = (int)(xTmp / xMax);
                    xTmp -= (tmp * xMax);
                    zTmp += tmp;
                    if (zTmp >= zMax)
                    {
                        tmp = (int)(zTmp / zMax);
                        zTmp -= (tmp * zMax);
                        yTmp += tmp;
                    }
                }
            }
        }

        /// <summary>
        /// Adds points of glyphs to the list outputpoints, otionlaly fill also colors and pointIndexes
        /// </summary>
        /// <param name="outputPoints"> output list of glyphs points</param>
        /// <param name="inputGrid">input grid from which glyph positions will be detrmined</param>
        /// <param name="inputAttributes">input attributes from which glyph attributes scale and Orientation will be determined</param>
        /// <param name="glyph"> glyph that will be used</param>
        /// <param name="outputIndexes"> output list of indexes  of points from which glyphs were created</param>
        /// <param name="outputAttributes"> output list of attributes from which glyphs were created</param>
        private void AddPoints(ref UniformDataArray outputPoints, ref UniformDataArray outputIndexes,
                                ref RegGridND inputGrid, ref MveArray inputAttributes, ref Glyph3D glyph,
                                ref UniformDataArray outputAttributes)
        {
            if (inputAttributes.ElemType.Name.Equals("Normal3D"))//process Normal3D attributes
                AddPointsWithNomalAtr(ref outputPoints, ref outputIndexes, ref inputGrid, ref inputAttributes,
                                      ref glyph, ref outputAttributes);
            else if (inputAttributes.ElemType.Name.Equals("Vector3D"))//process Vecto3D attributes
                AddPointsWithVectorAtr(ref outputPoints, ref outputIndexes, ref inputGrid, ref inputAttributes,
                                       ref glyph, ref outputAttributes);
            else if (inputAttributes.ElemType.Name.Equals("Scalar"))//process Scalar attributes
                AddPointsWithScalarAtr(ref outputPoints, ref outputIndexes, ref inputGrid, ref inputAttributes,
                                       ref glyph, ref outputAttributes);
            else if (inputAttributes.ElemType.Name.Equals("Integer"))//process Integer attributes
                AddPointsWithIntegerAtr(ref outputPoints, ref outputIndexes, ref inputGrid, ref inputAttributes,
                                        ref glyph, ref outputAttributes);
            else
            {
                throw new MveException(this, "Invalid atribute type: " + inputAttributes.ElemType.Name);
            }
        }

        /// <summary>
        /// Creates lists of glyph points and point attributes from Normal3D attributes
        /// </summary>
        /// <param name="outputPoints"> output list of glyphs points</param>
        /// <param name="inputGrid">input grid from which glyph positions will be detrmined</param>
        /// <param name="inputAttributes">input attributes from which glyph attributes scale and Orientation will be determined</param>
        /// <param name="glyph"> glyph that will be used</param>
        /// <param name="outputIndexes"> output list of indexes  of points from which glyphs were created</param>
        /// <param name="outputAttributes"> output list of attributes used for glyph generation</param>
        private void AddPointsWithNomalAtr(ref UniformDataArray outputPoints, ref UniformDataArray outputIndexes,
                                            ref RegGridND inputGrid, ref MveArray inputAttributes,
                                            ref Glyph3D glyph, ref UniformDataArray outputAttributes)
        {
            Point3D position;
            Transform3D transformation = new Transform3D();
            Normal3D normal;
            float min = 0, max = 1, scale, intervalLength;
            Random rand = new Random(System.Environment.TickCount);
            float step;
            float progress = 0;
            float progressStep;
            int numOfGlyphs;

            int gridPointsCount = GetNumOFPoints(ref inputGrid);
            // determine number of glyphs that will be created and step
            if (MaxGlyphs < gridPointsCount && MaskPoints)
            {
                numOfGlyphs = MaxGlyphs;
                step = (float)gridPointsCount / (float)MaxGlyphs;
            }
            else
            {
                numOfGlyphs = gridPointsCount;
                step = 1;
            }
            progressStep = 1 / (float)numOfGlyphs;
            //detemine max and min normal length
            if (UseScale)
            {
                normal = (Normal3D)inputAttributes[0, 0, 0];
                min = max = (float)normal.Abs();
                float actualLength;
                for (int i = 1; i < inputAttributes.Lengths[0]; i++)
                {
                    for (int j = 1; j < inputAttributes.Lengths[1]; j++)
                    {
                        for (int k = 1; k < inputAttributes.Lengths[2]; k++)
                        {
                            actualLength = (float)((Normal3D)inputAttributes[i, j, k]).Abs();
                            if (actualLength < min)
                                min = actualLength;
                            else if (actualLength > max)
                                max = actualLength;
                        }
                    }
                }
            }
            intervalLength = max - min;// intervalLength will specify length af interval from min to max
            // add points and their attributes
            float xTmp = 0;
            float yTmp = 0;
            float zTmp = 0;
            int xMax = inputGrid.Dimensions[0] + 1;
            int yMax = inputGrid.Dimensions[1] + 1;
            int zMax = inputGrid.Dimensions[2] + 1;
            int x = 0;
            int y = 0;
            int z = 0;
            int index = 0;
            for (int m = 0; m < numOfGlyphs; m++)
            {
                x = (int)Math.Floor(xTmp);
                z = (int)Math.Floor(zTmp);
                y = (int)Math.Floor(yTmp);
                GetIndex(step, ref rand, ref xTmp, ref yTmp, ref zTmp, xMax, yMax, zMax);
                normal = (Normal3D)inputAttributes[x,y,z];
                position = (Point3D)inputGrid.GetPoint(new int[] { x, y, z });
                transformation.SetTranslation((float)position.X, (float)position.Y, (float)position.Z);
                if (Orient)// rotate glyph
                {
                    if (normal.Abs() > 0)
                    {
                        Transform3D tr = AddRotation(normal);
                        transformation = transformation * tr;
                    }
                }
                if (UseScale)// add scale accroding to relative vector atribute lenght
                {
                    if (intervalLength != 0)
                    {
                        scale = (float)(normal.Abs() + (-min)) / intervalLength;
                        scale += 0.1f;
                        transformation.AddScale(scale, scale, scale);
                    }
                }
                if (UseGlobalScale)// add global scale
                {
                    transformation.AddScale((float)GlobalScale, (float)GlobalScale, (float)GlobalScale);
                }
                for (int l = 0; l < glyph.GlyphPoints.Count; l++)// add transformed glyph points to the list
                {
                    outputPoints[index] = (transformation.Transform(glyph.GlyphPoints[l]));
                    
                    //  if (AddIndexes)
                    //     outputIndexes.Add(new Integer(index));
                    if (KeepAtributes)
                        outputAttributes[index] = normal;
                    index++;
                }
                ProgressInfo(progress, 1, 2);
                progress += progressStep;
            }
        }



        /// <summary>
        /// Creates lists of glyph points and point attributes from Vector3D attributes
        /// </summary>
        /// <param name="outputPoints"> output list of glyphs points</param>
        /// <param name="inputGrid">input grid from which glyph positions will be detrmined</param>
        /// <param name="inputAttributes">input attributes from which glyph attributes scale and Orientation will be determined</param>
        /// <param name="glyph"> glyph that will be used</param>
        /// <param name="outputIndexes"> output list of indexes  of points from which glyphs were created</param>
        /// <param name="outputAttributes"> output list of attributes used for glyph generation</param>
        private void AddPointsWithVectorAtr(ref UniformDataArray outputPoints, ref UniformDataArray outputIndexes,
                                            ref RegGridND inputGrid, ref MveArray inputAttributes,
                                            ref Glyph3D glyph, ref UniformDataArray outputAttributes)
        {
            Point3D position;
            Transform3D transformation = new Transform3D();
            Vector3D vector;
            float min = 0, max = 1, scale, intervalLength;
            Random rand = new Random(System.Environment.TickCount);
            float step;
            float progress = 0;
            float progressStep;
            int numOfGlyphs;

            int gridPointsCount = GetNumOFPoints(ref inputGrid);
            // determine number of glyphs that will be created and step
            if (MaxGlyphs < gridPointsCount && MaskPoints)
            {
                numOfGlyphs = MaxGlyphs;
                step = (float)gridPointsCount / (float)MaxGlyphs;
            }
            else
            {
                numOfGlyphs = gridPointsCount;
                step = 1;
            }
            progressStep = 1 / (float)numOfGlyphs;
            //detemine max and min normal length

            if (UseScale)
            {
                vector = (Vector3D)inputAttributes[0, 0, 0];
                min = max = (float)vector.Abs();
                float actualLength;
                for (int i = 1; i < inputAttributes.Lengths[0]; i++)
                {
                    for (int j = 1; j < inputAttributes.Lengths[1]; j++)
                    {
                        for (int k = 1; k < inputAttributes.Lengths[2]; k++)
                        {
                            actualLength = (float)((Vector3D)inputAttributes[i, j, k]).Abs();
                            if (actualLength < min)
                                min = actualLength;
                            else if (actualLength > max)
                                max = actualLength;
                        }
                    }
                }
            }
            intervalLength = max - min;// intervalLength will specify length af interval from min to max
            // add points and their attributes
            float xTmp = 0;
            float yTmp = 0;
            float zTmp = 0;
            int xMax = inputGrid.Dimensions[0] + 1;
            int yMax = inputGrid.Dimensions[1] + 1;
            int zMax = inputGrid.Dimensions[2] + 1;
            int x = 0;
            int y = 0;
            int z = 0;
            int index = 0;
            for (int m = 0; m < numOfGlyphs; m++)
            {
                x = (int)Math.Floor(xTmp);
                z = (int)Math.Floor(zTmp);
                y = (int)Math.Floor(yTmp);
                GetIndex(step, ref rand, ref xTmp, ref yTmp, ref zTmp, xMax, yMax, zMax);
                vector = (Vector3D)inputAttributes[x, y, z];
                position = (Point3D)inputGrid.GetPoint(new int[] { x, y, z });
                transformation.SetTranslation((float)position.X, (float)position.Y, (float)position.Z);
                if (Orient)// rotate glyph
                {
                    if (vector.Abs() > 0)
                    {
                        Transform3D tr = AddRotation(new Normal3D(vector.X, vector.Y, vector.Z));
                        transformation = transformation * tr;
                    }
                }
                if (UseScale)// add scale accroding to relative vector atribute lenght
                {
                    if (intervalLength != 0)
                    {
                        scale = (float)(vector.Abs() + (-min)) / intervalLength;
                        scale += 0.1f;
                        transformation.AddScale(scale, scale, scale);
                    }
                }
                if (UseGlobalScale)// add global scale
                {
                    transformation.AddScale((float)GlobalScale, (float)GlobalScale, (float)GlobalScale);
                }
                for (int l = 0; l < glyph.GlyphPoints.Count; l++)
                {
                    outputPoints[index] = (transformation.Transform(glyph.GlyphPoints[l]));
                    
                    //  if (AddIndexes)
                    //     outputIndexes.Add(new Integer(index));
                    if (KeepAtributes)
                        outputAttributes[index] = vector;
                    index++;
                }
                ProgressInfo(progress, 1, 2);
                progress += progressStep;
            }
        }

        /// <summary>
        /// Creates lists of glyph points and point attributes from Scalar attributes
        /// </summary>
        /// <param name="outputPoints"> output list of glyphs points</param>
        /// <param name="inputGrid">input grid from which glyph positions will be detrmined</param>
        /// <param name="inputAttributes">input attributes from which glyph attributes scale and Orientation will be determined</param>
        /// <param name="glyph"> glyph that will be used</param>
        /// <param name="outputIndexes"> output list of indexes  of points from which glyphs were created</param>
        /// <param name="outputAttributes"> output list of attributes used for glyph generation</param>
        private void AddPointsWithScalarAtr(ref UniformDataArray outputPoints, ref UniformDataArray outputIndexes,
                                            ref RegGridND inputGrid, ref MveArray inputAttributes,
                                            ref Glyph3D glyph, ref UniformDataArray outputAttributes)
        {
            Point3D position;
            Transform3D transformation = new Transform3D();
            Scalar scalar;
            float min = 0, max = 1, scale, intervalLength;
            Random rand = new Random(System.Environment.TickCount);
            float step;
            float progress = 0;
            float progressStep;
            int numOfGlyphs;

            int gridPointsCount = GetNumOFPoints(ref inputGrid);
            // determine number of glyphs that will be created and step
            if (MaxGlyphs < gridPointsCount && MaskPoints)
            {
                numOfGlyphs = MaxGlyphs;
                step = (float)gridPointsCount / (float)MaxGlyphs;
            }
            else
            {
                numOfGlyphs = gridPointsCount;
                step = 1;
            }
            progressStep = 1 / (float)numOfGlyphs;
            //detemine max and min normal length

            if (UseScale)
            {
                scalar = (Scalar)inputAttributes[0, 0, 0];
                min = max = (float)scalar;
                float actualLength;
                for (int i = 1; i < inputAttributes.Lengths[0]; i++)
                {
                    for (int j = 1; j < inputAttributes.Lengths[1]; j++)
                    {
                        for (int k = 1; k < inputAttributes.Lengths[2]; k++)
                        {
                            actualLength = (float)((Scalar)inputAttributes[i, j, k]);
                            if (actualLength < min)
                                min = actualLength;
                            else if (actualLength > max)
                                max = actualLength;
                        }
                    }
                }
            }
            intervalLength = max - min;// intervalLength will specify length af interval from min to max
            // add points and their attributes
            float xTmp = 0;
            float yTmp = 0;
            float zTmp = 0;
            int xMax = inputGrid.Dimensions[0] + 1;
            int yMax = inputGrid.Dimensions[1] + 1;
            int zMax = inputGrid.Dimensions[2] + 1;
            int x = 0;
            int y = 0;
            int z = 0;
            int index = 0;
            for (int m = 0; m < numOfGlyphs; m++)
            {
                x = (int)Math.Floor(xTmp);
                z = (int)Math.Floor(zTmp);
                y = (int)Math.Floor(yTmp);
                GetIndex(step, ref rand, ref xTmp, ref yTmp, ref zTmp, xMax, yMax, zMax);
                scalar = (Scalar)inputAttributes[x, y, z];
                position = (Point3D)inputGrid.GetPoint(new int[] { x, y, z });
                transformation.SetTranslation((float)position.X, (float)position.Y, (float)position.Z);
                
                if (UseScale)// add scale accroding to relative vector atribute lenght
                {
                    if (intervalLength != 0)
                    {
                        scale = (float)(scalar.Value + (-min)) / intervalLength;
                        scale += 0.1f;
                        transformation.AddScale(scale, scale, scale);
                    }
                }
                if (UseGlobalScale)// add global scale
                {
                    transformation.AddScale((float)GlobalScale, (float)GlobalScale, (float)GlobalScale);
                }
                for (int l = 0; l < glyph.GlyphPoints.Count; l++)
                {
                    outputPoints[index] = (transformation.Transform(glyph.GlyphPoints[l]));
                    
                    //  if (AddIndexes)
                    //     outputIndexes.Add(new Integer(index));
                    if (KeepAtributes)
                        outputAttributes[index] = scalar;
                    index++;
                }
                ProgressInfo(progress, 1, 2);
                progress += progressStep;
            }
        }

        /// <summary>
        /// Creates lists of glyph points and point attributes from Integer attributes
        /// </summary>
        /// <param name="outputPoints"> output list of glyphs points</param>
        /// <param name="inputGrid">input grid from which glyph positions will be detrmined</param>
        /// <param name="inputAttributes">input attributes from which glyph attributes scale and Orientation will be determined</param>
        /// <param name="glyph"> glyph that will be used</param>
        /// <param name="outputIndexes"> output list of indexes  of points from which glyphs were created</param>
        /// <param name="outputAttributes"> output list of attributes used for glyph generation</param>
        private void AddPointsWithIntegerAtr(ref UniformDataArray outputPoints, ref UniformDataArray outputIndexes,
                                             ref RegGridND inputGrid, ref MveArray inputAttributes,
                                             ref Glyph3D glyph, ref UniformDataArray outputAttributes)
        {
            Point3D position;
            Transform3D transformation = new Transform3D();
            Integer integer;
            int min = 0, max = 1, intervalLength;
            float scale;
            Random rand = new Random(System.Environment.TickCount);
            float step;
            float progress = 0;
            float progressStep;
            int numOfGlyphs;

            int gridPointsCount = GetNumOFPoints(ref inputGrid);
            // determine number of glyphs that will be created and step
            if (MaxGlyphs < gridPointsCount && MaskPoints)
            {
                numOfGlyphs = MaxGlyphs;
                step = (float)gridPointsCount / (float)MaxGlyphs;
            }
            else
            {
                numOfGlyphs = gridPointsCount;
                step = 1;
            }
            progressStep = 1 / (float)numOfGlyphs;
            //detemine max and min normal length

            if (UseScale)
            {
                integer = (Integer)inputAttributes[0, 0, 0];
                min = max = integer;
                int actualLength;
                for (int i = 1; i < inputAttributes.Lengths[0]; i++)
                {
                    for (int j = 1; j < inputAttributes.Lengths[1]; j++)
                    {
                        for (int k = 1; k < inputAttributes.Lengths[2]; k++)
                        {
                            actualLength = ((Integer)inputAttributes[i, j, k]);
                            if (actualLength < min)
                                min = actualLength;
                            else if (actualLength > max)
                                max = actualLength;
                        }
                    }
                }
            }
            intervalLength = max - min;// intervalLength will specify length af interval from min to max
            // add points and their attributes
            float xTmp = 0;
            float yTmp = 0;
            float zTmp = 0;
            int xMax = inputGrid.Dimensions[0] + 1;
            int yMax = inputGrid.Dimensions[1] + 1;
            int zMax = inputGrid.Dimensions[2] + 1;
            int x = 0;
            int y = 0;
            int z = 0;
            int index = 0;
            for (int m = 0; m < numOfGlyphs; m++)
            {
                x = (int)Math.Floor(xTmp);
                z = (int)Math.Floor(zTmp);
                y = (int)Math.Floor(yTmp);
                GetIndex(step, ref rand, ref xTmp, ref yTmp, ref zTmp, xMax, yMax, zMax);
                integer = (Integer)inputAttributes[x, y, z];
                position = (Point3D)inputGrid.GetPoint(new int[] { x, y, z });
                transformation.SetTranslation((float)position.X, (float)position.Y, (float)position.Z);
                
                if (UseScale)// add scale accroding to relative vector atribute lenght
                {
                    if (intervalLength != 0)
                    {
                        scale = (float)(integer.Value + (-min)) / intervalLength;
                        scale += 0.1f;
                        transformation.AddScale(scale, scale, scale);
                    }
                }
                if (UseGlobalScale)// add global scale
                {
                    transformation.AddScale((float)GlobalScale, (float)GlobalScale, (float)GlobalScale);
                }
                for (int l = 0; l < glyph.GlyphPoints.Count; l++)
                {
                    outputPoints[index] = (transformation.Transform(glyph.GlyphPoints[l]));
                    
                    //  if (AddIndexes)
                    //     outputIndexes.Add(new Integer(index));
                    if (KeepAtributes)
                        outputAttributes[index] = integer;
                    index++;
                }
                ProgressInfo(progress, 1, 2);
                progress += progressStep;
            }
        }


        /// <summary>
        /// Fills list of cells a points according to input parameters
        /// </summary>
        /// <param name="outputPoints"> output list of glyphs points</param>
        /// <param name="outputCells"> output list of glyphs cells</param>
        /// <param name="inputGrid">input grid from which glyph positions will be detrmined</param>
        /// <param name="inputAtributes">input attributes from which glyph attributes scale and Orientation will be determined</param>
        /// <param name="glyph"> glyph that will be used</param>
        /// <param name="outputIndexes">output list of indexes  of points from which glyphs were created</param>
        /// <param name="outputAttributes"> output list of attributes used for glyph generation</param>
        private void AddGlyphs(ref UniformDataArray outputPoints, ref UniformDataArray outputCells,
                                     ref RegGridND inputGrid, ref MveArray inputAtributes,
                                     ref Glyph3D glyph, ref UniformDataArray outputIndexes, ref UniformDataArray outputAttributes)
        {
            int numOfGlyphs = GetNumOFPoints(ref inputGrid);
            if (RandomMode)
            {
                glyph.UsedIndexes = new bool[numOfGlyphs];
            }
            AddCells(ref outputCells, numOfGlyphs, ref glyph);// add cells
            AddPoints(ref outputPoints, ref  outputIndexes, ref inputGrid, ref  inputAtributes,
                      ref  glyph, ref outputAttributes);
        }

        /// <summary>
        /// Function that return number of points in inputGrid
        /// </summary>
        /// <param name="inputGrid"></param>
        /// <returns></returns>
        private int GetNumOFPoints(ref RegGridND inputGrid)
        {
            int tmp = 1;
            for (int i = 0; i < inputGrid.Dimensions.Length; i++)
            {
                tmp *= (inputGrid.Dimensions[i] + 1);
            }
            return tmp;
        }
        #endregion
    }
}