using System;
using System.Collections.Generic;
using System.Text;
using Zcu.Mve.Core;
using Zcu.Mve.Visualization;
using Zcu.Mve.Numerics;
using System.Collections;
namespace Zcu.Mve.Visualization.Processing
{
    /// <summary>
    /// Module can transfoms Normal3D or Vector3D or Scalar or Integer point atributes to glyphs
    /// </summary>
    [ModuleInfo("Vaclav Bystricky", " Module can transfoms Normal3D or Vector3D or Scalar or Integer point atributes to glyphs", CategoryType.Filter, CategoryData.Mesh, IconName = "Glyph.ico")]
    [PortInfo("UnstrGridIn", "Unstrgrid from which point attributes will be created glyphs ")]
    [PortInfo("UnstrGridGlyphIn", "User defined glyph")]
    [PortInfo("UnstrGridOut", "Output Unstrgrid that contains glyphs")]
    public class GlyphsFromUnstrGrid : Glyphs
    {
        #region Init

        /// <summary>
        /// Function that adds pots to module
        /// </summary>
        protected override void AddPorts()
        {
            AddInPort("UnstrGridIn", typeof(UnstrGrid));
            AddInPort("UnstrGridGlyphIn", typeof(UnstrGrid), false);
            AddOutPort("UnstrGridOut", typeof(UnstrGrid));
        }

        /// <summary>
        /// Function that checks input parametrs and creates glyphs
        /// </summary>
        protected override void ProccesInput()
        {
            UnstrGrid inputUnstrGrid = (UnstrGrid)this.GetInput("UnstrGridIn");
            if (inputUnstrGrid == null)
            {
                throw new MveException(this, "Invalid input, only Unstrgrid is neede");
            }
            ICollection<string> ar = inputUnstrGrid.GetPointAttrsIds();
            String[] pointAtrIDs = new string[ar.Count];
            ar.CopyTo(pointAtrIDs, 0);
            for (int i = 0; i < pointAtrIDs.Length; i++)
            {
                if (pointAtrIDs[i].Equals(AttributeName))//check atribute name
                {
                    UniformDataArray attributes = inputUnstrGrid.GetPointAttrs(pointAtrIDs[i]);
                    if (attributes.ElemType.Name.Equals(AttributeType))//check atribute type
                    {
                        if (inputUnstrGrid.Points.ElemType.Name.Equals("Point3D"))//check point type
                        {
                            CreateOutputUnstrGridFromUnstrGrid(inputUnstrGrid.Points, attributes);// computes glyphs
                        }
                        else
                        {
                            throw new MveException(this, "Invalid point type: " + inputUnstrGrid.Points.ElemType.Name + ", only Point3D type is allowed");
                        }
                        return;
                    }
                }
            }
            // if attributes do not match, an exception is thrown
            throw new MveException(this, "Atribute " + AttributeName + " (" + AttributeType + ") wasn't found");
        }

        /// <summary>
        /// Function that creates glyphs from input points and their attributes
        /// </summary>
        /// <param name="points"> points that determine glyphs positions</param>
        /// <param name="aributes"> attributes that determine glyphs size and direction </param>
        private void CreateOutputUnstrGridFromUnstrGrid(UniformDataArray points, MveArray aributes)
        {
            GetInputGlyph();
            outputGrid = new UnstrGrid();// output unstrgrid

            int numOFGlyphs;
            if (MaskPoints)
                numOFGlyphs = Math.Min(MaxGlyphs, points.Count);
            else
                numOFGlyphs = points.Count;

            UniformDataArray glyphsPoints = new UniformDataArray(typeof(Point3D), numOFGlyphs * glyph.GlyphPoints.Count);// glyph points
            UniformDataArray glyphsCells = new UniformDataArray(typeof(Triangle), numOFGlyphs * glyph.GlyphCells.Count);// glyph cells
            UniformDataArray glyphSourcePointsIndexes = null; ;// indexces of points from which glyphs were created
            UniformDataArray originalAtributes = null; // list of original attributes that will be hand over to glyph points
            if (KeepAtributes)
                originalAtributes = new UniformDataArray(aributes.ElemType, numOFGlyphs * glyph.GlyphPoints.Count);
            if (AddIndexes)
                glyphSourcePointsIndexes = new UniformDataArray(typeof(Integer), numOFGlyphs * glyph.GlyphPoints.Count);
            // computation of glyphs
            AddGlyphs(ref glyphsPoints, ref glyphsCells, ref points, ref aributes, ref glyph, 
                      ref glyphSourcePointsIndexes, ref originalAtributes);

            outputGrid.AddCells("Triangle", glyphsCells);//add cells
            outputGrid.Points = glyphsPoints;// add points
            if(AddIndexes)
                outputGrid.AddPointAttrs(IndexAttributeName, glyphSourcePointsIndexes);//add indexes
            if(KeepAtributes)
                outputGrid.AddPointAttrs(OriginalAttributeName, originalAtributes);
            SetOutput("UnstrGridOut", outputGrid);
        }
        #endregion
        #region AddGlyphs

        /// <summary>
        /// Compute new index of points from which a glyph will be created
        /// </summary>
        /// <param name="step"> float value deteminig step between two following indexes </param>
        /// <param name="rand"> random number generator</param>
        /// <param name="maxIndex"> maximum value of index that can be generated</param>
        /// <param name="floatIndex"> number deterniming current float value of index</param>
        /// <param name="randomIndex"> index to the field of bool values deterniming if corresponding index was already used</param>
        /// <returns></returns>
        protected int GetIndex(float step, ref Random rand, int maxIndex, ref float floatIndex, ref int randomIndex)
        {
            int index;
            int maxMiss = 10000;
            if (RandomMode && (step != 1)) // random mode
            {
                int tmp = rand.Next(0, maxIndex);
                int randomMiss = 0;
                while ((randomMiss < maxMiss) && (glyph.UsedIndexes[tmp] == true))
                {
                    tmp = rand.Next(0, maxIndex);
                    randomMiss++;
                }
                if (randomMiss != maxMiss)
                {
                    index = tmp;
                    glyph.UsedIndexes[tmp] = true;
                }
                else
                {
                    while (glyph.UsedIndexes[randomIndex] == true)
                    {
                        randomIndex++;
                    }
                    index = randomIndex;
                    glyph.UsedIndexes[randomIndex] = true;
                }
            }
            else// normal mode
            {
                index = (int)Math.Floor(floatIndex);
                floatIndex += step;
            }
            return index;
        }

        /// <summary>
        /// Adds points of glyphs to the list outputpoints, otionlaly fill also colors and pointIndexes
        /// </summary>
        /// <param name="outputPoints"> output list of glyphs points</param>
        /// <param name="outputIndexes"> output list of indexes of points from which glyphs were created</param>
        /// <param name="inputPoints">input points from which glyph positions will be detrmined</param>
        /// <param name="inputAttributes">input attributes from which glyph attributes scale and Orientation will be determined</param>
        /// <param name="glyph"> glyph that will be used</param>
        /// <param name="outputAttributes"> output list of attributes from which glyphs were created</param>
        private void AddPoints(ref UniformDataArray outputPoints, ref UniformDataArray outputIndexes,
                                ref UniformDataArray inputPoints, ref MveArray inputAttributes, ref Glyph3D glyph,
                                ref UniformDataArray outputAttributes)
        {
            if (inputAttributes.ElemType.Name.Equals("Normal3D"))//process Normal3D attributes
                AddPointsWithNomalAtr(ref outputPoints, ref outputIndexes, ref inputPoints, ref inputAttributes,
                                      ref glyph, ref outputAttributes);
            else if (inputAttributes.ElemType.Name.Equals("Vector3D"))//process Vecto3D attributes
                AddPointsWithVectorAtr(ref outputPoints, ref outputIndexes, ref inputPoints, ref inputAttributes,
                                       ref glyph, ref outputAttributes);
            else if (inputAttributes.ElemType.Name.Equals("Scalar"))//process Scalar attributes
                AddPointsWithScalarAtr(ref outputPoints, ref outputIndexes, ref inputPoints, ref inputAttributes,
                                       ref glyph, ref outputAttributes);
            else if (inputAttributes.ElemType.Name.Equals("Integer"))//process Integer attributes
                AddPointsWithIntegerAtr(ref outputPoints, ref outputIndexes, ref inputPoints, ref inputAttributes,
                                        ref glyph, ref outputAttributes);
            else
            {
                throw new MveException(this, "Invalid atribute type: " + inputAttributes.ElemType.Name);
            }
        }

        /// <summary>
        /// Creates lists of glyph points and point attributes from Normal3D attributes
        /// </summary>
        /// <param name="outputPoints"> output list of glyphs points</param>
        /// <param name="inputPoints">input points from which glyph positions will be detrmined</param>
        /// <param name="inputAttributes">input attributes from which glyph attributes scale and Orientation will be determined</param>
        /// <param name="glyph"> glyph that will be used</param>
        /// <param name="outputIndexes"> output list of indexes  of points from which glyphs were created</param>
        /// <param name="outputAttributes"> output list of attributes used for glyph generation</param>
        private void AddPointsWithNomalAtr(ref UniformDataArray outputPoints, ref UniformDataArray outputIndexes,
                                            ref UniformDataArray inputPoints, ref MveArray inputAttributes,
                                            ref Glyph3D glyph, ref UniformDataArray outputAttributes)
        {
            Point3D position;
            Transform3D transformation = new Transform3D();
            Normal3D normal;
            float min = 0, max = 1, scale;
            Random rand = new Random(System.Environment.TickCount);
            float step;
            float progress = 0;
            float progressStep;
            float floatIndex = 0;
            int numOfGlyphs;
            float intervalLength;
            // determine number of glyphs that will be created and step
            if (MaxGlyphs < inputPoints.Count && MaskPoints)
            {
                numOfGlyphs = MaxGlyphs;
                step = (float)inputPoints.Count / (float)MaxGlyphs;
            }
            else
            {
                numOfGlyphs = inputPoints.Count;
                step = 1;
            }
            progressStep = 1 / (float)numOfGlyphs;
            //detemine max and min normal length
            if (UseScale)
            {
                normal = (Normal3D)inputAttributes[0];
                min = max = (float)normal.Abs();
                float actualLength;
                for (int i = 1; i < inputPoints.Count; i++)
                {
                    actualLength = (float)((Normal3D)inputAttributes[i]).Abs();
                    if (actualLength < min)
                        min = actualLength;
                    else if (actualLength > max)
                        max = actualLength;
                }
            }
            intervalLength = max - min;
            // add points and their attributes
            int randomIndex = 0;
            int indexToGlyphPointsArray = 0;
            for (int i = 0; i < numOfGlyphs; i++)
            {
                int index;
                index = GetIndex(step, ref rand, inputPoints.Count - 1, ref floatIndex, ref randomIndex);
                normal = (Normal3D)inputAttributes[index];
                position = (Point3D)inputPoints[index];
                transformation.SetTranslation((float)position.X, (float)position.Y, (float)position.Z);

                if (Orient)// rotate glyph
                {
                    if (normal.Abs() > 0)
                    {
                        Transform3D tr = AddRotation(normal);
                        transformation = transformation * tr;
                    }
                }
                if (UseScale)// add scale accroding to relative vector atribute lenght
                {
                    if (intervalLength != 0)
                    {
                        scale = (float)(normal.Abs() + (-min)) / intervalLength;
                        scale += 0.1f;
                        transformation.AddScale(scale, scale, scale);
                    }
                }

                if (UseGlobalScale)// add global scale
                {
                    transformation.AddScale((float)GlobalScale, (float)GlobalScale, (float)GlobalScale);
                }
                for (int j = 0; j < glyph.GlyphPoints.Count; j++)
                {
                    outputPoints[indexToGlyphPointsArray] = (transformation.Transform(glyph.GlyphPoints[j]));
                   
                    if (AddIndexes)
                        outputIndexes[indexToGlyphPointsArray] = new Integer(index);
                    if (KeepAtributes)
                        outputAttributes[indexToGlyphPointsArray] = normal;
                    indexToGlyphPointsArray++;
                }
                ProgressInfo(progress, 1, 2);
                progress += progressStep;
            }
        }

        /// <summary>
        /// Creates lists of glyph points and point attributes from Vector3D attributes
        /// </summary>
        /// <param name="outputPoints"> output list of glyphs points</param>
        /// <param name="inputPoints">input points from which glyph positions will be detrmined</param>
        /// <param name="inputAttributes">input attributes from which glyph attributes scale and Orientation will be determined</param>
        /// <param name="glyph"> glyph that will be used</param>
        /// <param name="outputIndexes"> output list of indexes  of points from which glyphs were created</param>
        /// <param name="outputAttributes"> output list of attributes used for glyph generation</param>
        private void AddPointsWithVectorAtr(ref UniformDataArray outputPoints, ref UniformDataArray outputIndexes,
                                            ref UniformDataArray inputPoints, ref MveArray inputAttributes,
                                            ref Glyph3D glyph, ref UniformDataArray outputAttributes)
        {
            Point3D position;
            Transform3D transformation = new Transform3D();
            Vector3D vector;
            Random rand = new Random(System.Environment.TickCount);
            float step, floatIndex = 0;
            float min = 0, max = 1, scale, actualLength;
            float progress = 0;
            float progressStep;
            float intervalLength;
            int numOfGlyphs = inputPoints.Count;

            // determine number of glyphs to  insert and step
            if (MaxGlyphs < inputPoints.Count && MaskPoints)
            {
                numOfGlyphs = MaxGlyphs;
                step = (float)inputPoints.Count / (float)MaxGlyphs;
            }
            else
            {
                numOfGlyphs = inputPoints.Count;
                step = 1;
            }
            progressStep = 1 / (float)numOfGlyphs;
            // find min and max length of vector atribute
            if (UseScale)
            {
                min = max = (float)((Vector3D)inputAttributes[0]).Abs();
                for (int i = 1; i < inputPoints.Count; i++)
                {
                    actualLength = (float)((Vector3D)inputAttributes[i]).Abs();
                    if (actualLength < min)
                        min = actualLength;
                    else if (actualLength > max)
                        max = actualLength;
                }
            }
            intervalLength = max - min;
            int randomIndex = 0;
            int indexToGlyphPointsArray = 0;
            // add points and their attributes
            for (int i = 0; i < numOfGlyphs; i++)
            {
                int index;
                index = GetIndex(step, ref rand, inputPoints.Count - 1, ref floatIndex, ref randomIndex);
                position = (Point3D)inputPoints[index];
                transformation.SetTranslation((float)position.X, (float)position.Y, (float)position.Z);
                vector = (Vector3D)inputAttributes[index];
                if (Orient)// rotate glyph
                {
                    if (vector.Abs() > 0)
                    {
                        Transform3D tr = AddRotation(new Normal3D(vector.X, vector.Y, vector.Z));
                        transformation = transformation * tr;
                    }
                }
                if (UseScale)// add scale accroding to relative vector atribute lenght
                {
                    if (intervalLength != 0)
                    {
                        scale = (float)(vector.Abs() + (-min)) / intervalLength;
                        scale += 0.1f;
                        transformation.AddScale(scale, scale, scale);
                    }

                }
                if (UseGlobalScale)// add global scale
                {
                    transformation.AddScale((float)GlobalScale, (float)GlobalScale, (float)GlobalScale);
                }
                for (int j = 0; j < glyph.GlyphPoints.Count; j++)
                {
                    outputPoints[indexToGlyphPointsArray] = (transformation.Transform(glyph.GlyphPoints[j]));
                   
                    if (AddIndexes)
                        outputIndexes[indexToGlyphPointsArray] = new Integer(index);
                    if (KeepAtributes)
                        outputAttributes[indexToGlyphPointsArray] = vector;
                    indexToGlyphPointsArray++;
                }
                ProgressInfo(progress, 1, 2);
                progress += progressStep;
            }
        }

        /// <summary>
        /// Creates lists of glyph points and point attributes from Scalar attributes
        /// </summary>
        /// <param name="outputPoints"> output list of glyphs points</param>
        /// <param name="inputPoints">input points from which glyph positions will be detrmined</param>
        /// <param name="inputAttributes">input attributes from which glyph attributes scale and Orientation will be determined</param>
        /// <param name="glyph"> glyph that will be used</param>
        /// <param name="outputIndexes"> output list of indexes  of points from which glyphs were created</param>
        /// <param name="outputAttributes"> output list of attributes used for glyph generation</param>
        private void AddPointsWithScalarAtr(ref UniformDataArray outputPoints, ref UniformDataArray outputIndexes,
                                            ref UniformDataArray inputPoints, ref MveArray inputAttributes,
                                            ref Glyph3D glyph, ref UniformDataArray outputAttributes)
        {
            Point3D position;
            Transform3D transformation = new Transform3D();
            Random rand = new Random(System.Environment.TickCount);
            float progress = 0;
            float progressStep;
            float intervalLength;
            float min = 0, max = 1, actualLength, scale;
            int numOfGlyphs = inputPoints.Count;
            float step, floatIndex = 0;

            // determine number of glyphs to  insert and step
            if (MaxGlyphs < inputPoints.Count && MaskPoints)
            {
                numOfGlyphs = MaxGlyphs;
                step = (float)inputPoints.Count / (float)MaxGlyphs;
            }
            else
            {
                numOfGlyphs = inputPoints.Count;
                step = 1;
            }
            progressStep = 1 / (float)numOfGlyphs;
            // find min and max value of scalar atribute
            if (UseScale)
            {
                min = max = (Scalar)inputAttributes[0];
                for (int i = 1; i < inputPoints.Count; i++)
                {
                    actualLength = (Scalar)inputAttributes[i];
                    if (actualLength < min)
                        min = actualLength;
                    else if (actualLength > max)
                        max = actualLength;
                }
            }
            intervalLength = max - min;
            int randomIndex = 0;
            int indexToGlyphPointsArray = 0;
            //add glyph points and their attributes
            for (int i = 0; i < numOfGlyphs; i++)
            {
                int index;
                index = GetIndex(step, ref rand, inputPoints.Count - 1, ref floatIndex, ref randomIndex);
                position = (Point3D)inputPoints[index];
                transformation.SetTranslation((float)position.X, (float)position.Y, (float)position.Z);
                if (UseScale)// add scale accroding to relative vector atribute lenght
                {
                    if (intervalLength != 0)
                    {
                        scale = (float)((Scalar)inputAttributes[index] + (-min)) / intervalLength;
                        scale += 0.1f;
                        transformation.AddScale(scale, scale, scale);
                    }

                }
                if (UseGlobalScale)// add global scale
                {
                    transformation.AddScale((float)GlobalScale, (float)GlobalScale, (float)GlobalScale);
                }
                for (int j = 0; j < glyph.GlyphPoints.Count; j++)
                {
                    outputPoints[indexToGlyphPointsArray] = (transformation.Transform(glyph.GlyphPoints[j]));
                   
                    if (AddIndexes)
                        outputIndexes[indexToGlyphPointsArray] = new Integer(index);
                    if (KeepAtributes)
                        outputAttributes[indexToGlyphPointsArray] = (Scalar)inputAttributes[index];
                    indexToGlyphPointsArray++;
                }
                ProgressInfo(progress, 1, 2);
                progress += progressStep;
            }
        }

        /// <summary>
        /// Creates lists of glyph points and point attributes from Integer attributes
        /// </summary>
        /// <param name="outputPoints"> output list of glyphs points</param>
        /// <param name="inputPoints">input points from which glyph positions will be detrmined</param>
        /// <param name="inputAttributes">input attributes from which glyph attributes scale and Orientation will be determined</param>
        /// <param name="glyph"> glyph that will be used</param>
        /// <param name="outputIndexes"> output list of indexes  of points from which glyphs were created</param>
        /// <param name="outputAttributes"> output list of attributes used for glyph generation</param>
        private void AddPointsWithIntegerAtr(ref UniformDataArray outputPoints, ref UniformDataArray outputIndexes,
                                              ref UniformDataArray inputPoints, ref MveArray inputAttributes,
                                              ref Glyph3D glyph, ref UniformDataArray outputAttributes)
        {
            Point3D position;
            Transform3D transformation = new Transform3D();
            Random rand = new Random(System.Environment.TickCount);
            float progress = 0;
            float progressStep;
            float intervalLength;
            float step, floatIndex = 0;
            float min = 0, max = 1, actualLength, scale;
            int numOfGlyphs = inputPoints.Count;

            // determine number of glyphs to  insert and step
            if (MaxGlyphs < inputPoints.Count && MaskPoints)
            {
                numOfGlyphs = MaxGlyphs;
                step = (float)inputPoints.Count / (float)MaxGlyphs;
            }
            else
            {
                numOfGlyphs = inputPoints.Count;
                step = 1;
            }
            progressStep = 1 / (float)numOfGlyphs;
            // find min and max value of integer attributes
            if (UseScale)
            {
                min = max = (Integer)inputAttributes[0];
                for (int i = 1; i < inputPoints.Count; i++)
                {
                    actualLength = (Integer)inputAttributes[i];
                    if (actualLength < min)
                        min = actualLength;
                    else if (actualLength > max)
                        max = actualLength;
                }
            }
            intervalLength = max - min;
            int randomIndex = 0;
            int indexToGlyphPointsArray = 0;
            // no optimalization is used 
            // add glyph points and thei attributes
            for (int i = 0; i < numOfGlyphs; i++)
            {
                int index;
                index = GetIndex(step, ref rand, inputPoints.Count - 1, ref floatIndex, ref randomIndex);
                position = (Point3D)inputPoints[index];
                transformation.SetTranslation((float)position.X, (float)position.Y, (float)position.Z);
                if (UseScale)// add scale accroding to relative vector atribute lenght
                {
                    if (intervalLength != 0)
                    {
                        scale = (float)((Integer)inputAttributes[index] + (-min)) / intervalLength;
                        scale += 0.1f;
                        transformation.AddScale(scale, scale, scale);
                    }
                }
                if (UseGlobalScale)//add global scaling
                {
                    transformation.AddScale((float)GlobalScale, (float)GlobalScale, (float)GlobalScale);
                }
                for (int j = 0; j < glyph.GlyphPoints.Count; j++)
                {
                    outputPoints[indexToGlyphPointsArray] = (transformation.Transform(glyph.GlyphPoints[j]));
                   
                    if (AddIndexes)
                        outputIndexes[indexToGlyphPointsArray] = new Integer(index);
                    if (KeepAtributes)
                        outputAttributes[indexToGlyphPointsArray] = (Integer)inputAttributes[index];
                    indexToGlyphPointsArray++;
                }
                ProgressInfo(progress, 1, 2);
                progress += progressStep;
            }
        }


        /// <summary>
        /// Fills list of cells a points according to input parameters
        /// </summary>
        /// <param name="outputPoints"> output list of glyphs points</param>
        /// <param name="outputCells"> output list of glyphs cells</param>
        /// <param name="inputPoints">input points from which glyph positions will be detrmined</param>
        /// <param name="inputAtributes">input attributes from which glyph attributes scale and Orientation will be determined</param>
        /// <param name="glyph"> glyph that will be used</param>
        /// <param name="outputIndexes">output list of indexes  of points from which glyphs were created</param>
        /// <param name="outputAttributes"> output list of attributes used for glyph generation</param>
        private void AddGlyphs(ref UniformDataArray outputPoints, ref UniformDataArray outputCells,
                                     ref UniformDataArray inputPoints, ref MveArray inputAtributes,
                                     ref Glyph3D glyph, ref UniformDataArray outputIndexes, ref UniformDataArray outputAttributes)
        {
            if(RandomMode)
                glyph.UsedIndexes = new bool[inputPoints.Count];
            AddCells(ref outputCells, inputPoints.Count, ref glyph);// add cells
            AddPoints(ref outputPoints, ref  outputIndexes, ref  inputPoints, ref  inputAtributes,
                      ref  glyph, ref outputAttributes);
        }
        #endregion
    }
}
