using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Zcu.Mve.Core;

namespace Zcu.Mve.Visualization.Processing
{
    /// <summary>
    /// Gui for glyph modul
    /// </summary>
    public partial class ModulGui : ModuleSetup
    {
        #region Variables Declaration
        // state variables
        private bool attributeNameChanged = true;

        private bool OrientChange = false;

        private bool AddIndexesChanged = false;

        private bool keepAttributesChange = false;

        private bool maxNumOfGlyphsChanged = false;

        private bool maskPoitsChanged = false;

        private bool indexAttributeNameChange = false;

        private bool OriginalAttributeNameChange= false;

        private bool RandomModeChanged = false;

        private double GlobalScale;
        private bool GlobalScaleChanged = false;

        private bool attributeTypeChanged = false;
        private bool UseScaleChanged = false;

        private bool glytphTypeChange = true;
        private bool UseGlobalScaleChanged = false;
        // Glyph guis
        private GlyphGui glyphGui;
        private SphereGui sphereGui = new SphereGui();
        private BoxGui boxGui = new BoxGui();
        private ConeGui coneGui = new ConeGui();
        private ArrowGui arrowGui = new ArrowGui();
        private CylinderGui cylinderGui = new CylinderGui();
        // module
        private Glyphs glyphs;
        #endregion

        /// <summary>
        /// Default contructor
        /// </summary>
        /// <param name="glyphs"></param>
        public ModulGui(Glyphs glyphs)
        {
            this.glyphs = glyphs;
            InitializeComponent();
            AddToolTips();
        }

        /// <summary>
        /// Adds tooltips to gui controls
        /// </summary>
        private void AddToolTips()
        {
            // Create the ToolTip and associate with the Form container.
            ToolTip toolTip1 = new ToolTip();
            // Set up the delays for the ToolTip.
            toolTip1.AutoPopDelay = 5000;
            toolTip1.InitialDelay = 1000;
            toolTip1.ReshowDelay = 500;
            // Force the ToolTip text to be displayed whether or not the form is active.
            toolTip1.ShowAlways = true;
            // Set up the ToolTip text for gui controls
            toolTip1.SetToolTip(MaxGlyphsNumUD, "Maximal number of glyphs that will be generated");
            toolTip1.SetToolTip(atributeNameTB, "Name of Atribute that will be used for glyph generation");
            toolTip1.SetToolTip(glyphTypeCB, " Type of glyphs that will be generated (Arrow, Box, Cone, Sphere, Cylinder)");
            toolTip1.SetToolTip(MaskPointsCBox, " If true, number of glyphs will be limited by Maximal number of Glyphs");
            toolTip1.SetToolTip(RandomModeCBox, " If true, glyphs will generated in random order, else evenly");
            toolTip1.SetToolTip(OrientCBox, " If true, glyphs will be Oriented according to vector atribute directions");
            toolTip1.SetToolTip(keepAttributesCBox, " If true, glyph points will contain atributes from which they were generated");
            toolTip1.SetToolTip(scaleCBox, " If true, glyphs will be scale relativly to maximum a and minimun values in proccesed attribute");
            toolTip1.SetToolTip(GlobalScaleCBox, " If true, global scaling will be performed");
            toolTip1.SetToolTip(GlobalScaleTBox, " value of global scale factor");
            toolTip1.SetToolTip(AttributeTypeComBox, " Type of proccesed attribute (Vector3D, Normal3D, Scalar, Integer)");
            toolTip1.SetToolTip(AddIndexesCBox1, " If true, indexes of points from which were glzphs created will be added to output points attributes");
            toolTip1.SetToolTip(originalAttrTBox, " Name of attribute generated from input attribute values and added to glyph points");
            toolTip1.SetToolTip(indexAttrTBox, "Name of attribute generated from input points indexes and added to glyph points");
        }
        
        /// <summary>
        /// Actions performed when Ok button is pressed
        /// </summary>
        /// <returns> true if all parametrs are correct</returns>
        public override bool OnOkClick()
        {
            if (OriginalAttributeNameChange)
            {
                OriginalAttributeNameChange = false;
                glyphs.ValueChanged = true;
                glyphs.OriginalAttributeName = originalAttrTBox.Text;
            }
            if (indexAttributeNameChange)
            {
                indexAttributeNameChange = false;
                glyphs.ValueChanged = true;
                glyphs.IndexAttributeName = indexAttrTBox.Text;
            }
            if (AddIndexesChanged)
            {
                AddIndexesChanged = false;
                glyphs.ValueChanged = true;
                glyphs.AddIndexes = AddIndexesCBox1.Checked;
            }
            if (keepAttributesChange)
            {
                keepAttributesChange = false;
                glyphs.ValueChanged = true;
                glyphs.KeepAtributes = keepAttributesCBox.Checked;
            }
            if (maxNumOfGlyphsChanged)
            {
                maxNumOfGlyphsChanged = false;
                glyphs.ValueChanged = true;
                glyphs.MaxGlyphs = (int)MaxGlyphsNumUD.Value;
            }
            if (GlobalScaleChanged)
            {
                GlobalScaleChanged = false;
                glyphs.ValueChanged = true;
                glyphs.GlobalScale = GlobalScale;
            }
            if (UseGlobalScaleChanged)
            {
                UseGlobalScaleChanged = false;
                glyphs.ValueChanged = true;
                glyphs.UseGlobalScale = GlobalScaleCBox.Checked;
            }
            if (UseScaleChanged)
            {
                UseScaleChanged = false;
                glyphs.ValueChanged = true;
                glyphs.UseScale = scaleCBox.Checked;
            }
            if (attributeNameChanged)
            {
                if (atributeNameTB.Text == "")
                {
                    MessageBox.Show("Enter the atribute name", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return false;
                }
                attributeNameChanged = false;
                glyphs.ValueChanged = true;
                glyphs.AttributeName = atributeNameTB.Text;
            }
            if (glytphTypeChange)
            {
                glytphTypeChange = false;
                glyphs.ValueChanged = true;
                string selesctedGlyph = (string)glyphTypeCB.SelectedItem;
                if(!CreateGlyph(selesctedGlyph))
                    return false;
            }
            if (glyphGui.GlyphParametrsChanged)
            {
                glyphGui.GlyphParametrsChanged = false;
                glyphs.ValueChanged = true;
                if (!CreateGlyph((string)glyphTypeCB.SelectedItem))
                    return false;

            }
            if (RandomModeChanged)
            {
                RandomModeChanged = false;
                glyphs.ValueChanged = true;
                glyphs.RandomMode = RandomModeCBox.Checked;
            }
            if (maskPoitsChanged)
            {
                maskPoitsChanged = false;
                glyphs.ValueChanged = true;
                glyphs.MaskPoints = MaskPointsCBox.Checked;
            }
            if (OrientChange)
            {
                OrientChange = false;
                glyphs.ValueChanged = true;
                glyphs.Orient = OrientCBox.Checked;
            }
            if (attributeTypeChanged)
            {
                attributeTypeChanged = false;
                glyphs.ValueChanged = true;
                glyphs.AttributeType = (string)AttributeTypeComBox.SelectedItem;
            }
            return true;
        }

        /// <summary>
        /// Returns current glyph gui parametrs
        /// </summary>
        /// <returns></returns>
        public List<string> GetGlyphParamatrs()
        {
            return glyphGui.GetParametrs();
        }

        /// <summary>
        /// Sets new valurs to gui compnents
        /// </summary>
        /// <param name="glyphGuiParametrs"></param>
        public void SetDefaultValues(List<string> glyphGuiParametrs)
        {
            MaxGlyphsNumUD.Value = (decimal)glyphs.MaxGlyphs;
            atributeNameTB.Text = glyphs.AttributeName;
            glyphTypeCB.SelectedItem = glyphs.GlyphType;
            MaskPointsCBox.Checked = glyphs.MaskPoints;
            RandomModeCBox.Checked = glyphs.RandomMode;
            OrientCBox.Checked = glyphs.Orient;
            keepAttributesCBox.Checked = glyphs.KeepAtributes;
            scaleCBox.Checked = glyphs.UseScale;
            GlobalScaleCBox.Checked = glyphs.UseGlobalScale;
            GlobalScaleTBox.Text = glyphs.GlobalScale.ToString();
            SetGlyphGui(glyphs.GlyphType);
            if(glyphGuiParametrs != null)
                glyphGui.SetParametrs(glyphGuiParametrs);
            CreateGlyph(glyphs.GlyphType);
            AttributeTypeComBox.SelectedItem = glyphs.AttributeType;
            AddIndexesCBox1.Checked = glyphs.AddIndexes;
            originalAttrTBox.Text = glyphs.OriginalAttributeName;
            indexAttrTBox.Text = glyphs.IndexAttributeName;
        }

        /// <summary>
        /// Actions perfomed when storno button was clicked
        /// </summary>
        /// <returns> true if all actions passed</returns>
        public override bool OnStornoClick()
        {
            SetDefaultValues(glyphGui.GetParametrs());
            return base.OnStornoClick();
        }

        /// <summary>
        /// Creates new glyph according to glyph type parametr and pertinent glyph gui parametrs
        /// </summary>
        /// <param name="glyphType"> name of glyph that should be created</param>
        /// <returns></returns>
        private bool CreateGlyph(string glyphType)
        {
            switch (glyphType)
            {
                case "Sphere": glyphs.Glyph = new Sphere(sphereGui.PhiRes, sphereGui.ThetaRes, sphereGui.Radius, sphereGui.SphereCenter);
                    glyphs.GlyphType = glyphType;
                    break;
                case "Cone": glyphs.Glyph = new Cone(coneGui.Resolution, coneGui.Heigth, coneGui.Radius, coneGui.Capping, coneGui.Center);
                    glyphs.GlyphType = glyphType;
                    break;
                case "Box": glyphs.Glyph = new Box(boxGui.XLenght, boxGui.YLenght, boxGui.ZLenght, boxGui.Center);
                    glyphs.GlyphType = glyphType;
                    break;
                case "Cylinder": glyphs.Glyph = new Cylinder(cylinderGui.Resolution, cylinderGui.Heigth, cylinderGui.Radius, cylinderGui.Capping, cylinderGui.Center);
                    glyphs.GlyphType = glyphType;
                    break;
                case "Arrow": glyphs.Glyph = new Arrow(arrowGui.TipResolution, arrowGui.TipRadius, arrowGui.TipLength, arrowGui.ShaftResoluion, arrowGui.ShaftRadius, arrowGui.ArrowLength);
                    glyphs.GlyphType = glyphType;
                    break;
                default: MessageBox.Show("Invalid Glyph type, folloving glyph types are acceptable : \r"
                                        + "Sphere, Cone, Box, Cylinder", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return false;
            }
            return true;
        }

        /// <summary>
        /// Activates glyph gui corresponding to glyphType parametr
        /// </summary>
        /// <param name="glyphType"> name of glyp which gui will be actvated</param>
        private void SetGlyphGui(string glyphType)
        {
            switch (glyphType)
            {
                case "Sphere":
                    tableLayoutPanel1.Controls.Remove(glyphGui);
                    tableLayoutPanel1.Controls.Add(sphereGui, 0, 1);
                    glyphGui = sphereGui;
                    break;
                case "Cone":
                    tableLayoutPanel1.Controls.Remove(glyphGui);
                    tableLayoutPanel1.Controls.Add(coneGui, 0, 1);
                    glyphGui = coneGui;
                    break;
                case "Box":
                    tableLayoutPanel1.Controls.Remove(glyphGui);
                    tableLayoutPanel1.Controls.Add(boxGui, 0, 1);
                    glyphGui = boxGui;
                    break;
                case "Cylinder":
                    tableLayoutPanel1.Controls.Remove(glyphGui);
                    tableLayoutPanel1.Controls.Add(cylinderGui, 0, 1);
                    glyphGui = cylinderGui;
                    break;
                case "Arrow":
                    tableLayoutPanel1.Controls.Remove(glyphGui);
                    tableLayoutPanel1.Controls.Add(arrowGui, 0, 1);
                    glyphGui = arrowGui;
                    break;
                default:
                    MessageBox.Show("Invalid Glyph type, folloving glyph types are acceptable : \r"
                                        + "Sphere, Cone, Box, Cylinder", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    break;
            }
        }

        private void atributeNameTB_LostFocus(object sender, EventArgs e)
        {
            attributeNameChanged = true;
        }

        private void MaxGlyphsNumUD_ValueChanged(object sender, EventArgs e)
        {
            maxNumOfGlyphsChanged = true;
        }

        private void MaskPointsCBox_CheckedChanged(object sender, EventArgs e)
        {
            maskPoitsChanged = true;
        }

        private void RandomModeCBox_CheckedChanged(object sender, EventArgs e)
        {
            RandomModeChanged = true;
        }

        private void glyphTypeCB_SelectedValueChanged(object sender, EventArgs e)
        {

            string selesctedGlyph = (string)glyphTypeCB.SelectedItem;
            SetGlyphGui(selesctedGlyph);
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            OrientChange = true;
        }

        private void checkBox1_CheckedChanged_1(object sender, EventArgs e)
        {
            keepAttributesChange = true;
        }

        private void GlobalScaleTBox_LostFocus(object sender, EventArgs e)
        {
            bool succeded = ParseDouble(ref GlobalScale, GlobalScaleTBox.Text, "Scale mus be positive number", false);
            if (succeded)
                GlobalScaleChanged = true;
            else
                GlobalScaleTBox.Text = Glyphs.DefGlobalScale.ToString();
        }

        private bool ParseDouble(ref double value, string text, string errorMessage, bool minus)
        {
            double tmp;
            try
            {
                tmp = Double.Parse(text, Globals.nfi);
                if (minus || (tmp >= 0))
                {
                    value = tmp;
                    return true;
                }
                else
                    MessageBox.Show(errorMessage);
            }
            catch (Exception ex)
            {
                MessageBox.Show(errorMessage + ex.Message);
            }
            return false;
        }

        private void scaleCBox_CheckedChanged(object sender, EventArgs e)
        {
            UseScaleChanged = true;
        }

        private void GlobalScaleCBox_CheckedChanged(object sender, EventArgs e)
        {
            UseGlobalScaleChanged = true;
        }

        private void glyphTypeCB_SelectedIndexChanged(object sender, EventArgs e)
        {
            glytphTypeChange = true;
        }

        private void AttributeTypeComBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            attributeTypeChanged = true;
        }

        private void AddIndexesCBox1_CheckedChanged(object sender, EventArgs e)
        {
            AddIndexesChanged = true;
        }

        private void originalAttrTBox_TextChanged(object sender, EventArgs e)
        {
            OriginalAttributeNameChange = true;
        }

        private void indexAttrTBox_TextChanged(object sender, EventArgs e)
        {
            indexAttributeNameChange = true;
        }   
    }
}
