using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Zcu.Mve.Core;

namespace Zcu.Mve.Visualization.Processing
{
    /// <summary>
    /// Giu for sphere glyph
    /// </summary>
    public partial class SphereGui : GlyphGui
    {
        # region Variables Declaration
        //sphere attributes
        private int thetaRes = Sphere.DefThetaRes;
        /// <summary>
        /// Sphere phi resolition
        /// </summary>
        public int ThetaRes
        {
            get { return thetaRes; }
        }

        private int phiRes = Sphere.DefPhiRes;
        /// <summary>
        /// Sphere phi resolition
        /// </summary>
        public int PhiRes
        {
            get { return phiRes; }
        }

        private double radius = Sphere.DefRadius;
        /// <summary>
        /// Shere radius
        /// </summary>
        public double Radius
        {
            get { return radius; }
        }

        private Center center = Sphere.DefCenter;
        /// <summary>
        /// Sphere center
        /// </summary>
        public Center SphereCenter
        {
            get { return center; }
        }
        # endregion
        
        /// <summary>
        /// Default constructor
        /// </summary>
        public SphereGui()
        {
            InitializeComponent();
            InitGuiValues();
        }

        /// <summary>
        /// Return an array of currnet arrow glyph paramatres
        /// </summary>
        /// <returns>An array of currnet arrow glyph paramatres</returns>
        public override List<string> GetParametrs()
        {
            List<string> paramets = new List<string>();
            paramets.Add(thetaRes.ToString());
            paramets.Add(radius.ToString(Globals.nfi));
            paramets.Add(phiRes.ToString());
            paramets.Add(center.x.ToString(Globals.nfi));
            paramets.Add(center.y.ToString(Globals.nfi));
            paramets.Add(center.z.ToString(Globals.nfi));
            return paramets;
        }

        /// <summary>
        /// Sets arrow gui attributes according to values in input array
        /// </summary>
        /// <param name="paramets"> new gui values</param>
        public override void SetParametrs(List<string> paramets)
        {
            thetaRes = Int32.Parse(paramets[0]);
            radius = Double.Parse(paramets[1], Globals.nfi);
            phiRes = Int32.Parse(paramets[2]);
            center.x = Double.Parse(paramets[3], Globals.nfi);
            center.y = Double.Parse(paramets[4], Globals.nfi);
            center.z = Double.Parse(paramets[5], Globals.nfi);
            InitGuiValues();
        }

        /// <summary>
        /// Sets arrow attributes to gui components
        /// </summary>
        private void InitGuiValues()
        {
            trackBar2.Value = thetaRes;
            thetaTBox.Text = thetaRes.ToString();
            radiusTBox.Text = radius.ToString(Globals.nfi);
            trackBar1.Value = phiRes;
            phiTBox.Text = phiRes.ToString();
            centerXTBox.Text = center.x.ToString(Globals.nfi);
            centerYTBox.Text = center.y.ToString(Globals.nfi);
            centerZTBox.Text = center.z.ToString(Globals.nfi);
            centerZTBox.LostFocus += new EventHandler(centerZTBox_LostFocus);
        }

        private void trackBar2_Scroll(object sender, EventArgs e)
        {
            thetaTBox.Text = trackBar2.Value.ToString();
            thetaTBox_LostFocus(null, null);
        }

        private void trackBar1_Scroll(object sender, EventArgs e)
        {
            phiTBox.Text = trackBar1.Value.ToString();
            phiTBox_LostFocus(null, null);
        }

        private void radiusTBox_LostFocus(object sender, EventArgs e)
        {
           bool secceeded = ParseDouble(ref radius, radiusTBox.Text, "Radius must be positiv float", false);
           if (!secceeded)
           {
               radiusTBox.Text = Sphere.DefRadius.ToString();
           }
           else
           {
               glyphParametrsChanged = true;
           }
        }

        private void centerXTBox_LostFocus(object sender, EventArgs e)
        {
            bool secceeded = ParseDouble(ref center.x, centerXTBox.Text, "Coordinate must be positiv integer",true);
            if (!secceeded)
            {
                centerXTBox.Text = Sphere.DefCenter.x.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }
        }

        private void centerYTBox_LostFocus(object sender, EventArgs e)
        {
            bool secceeded = ParseDouble(ref center.y, centerYTBox.Text, "Coordinate must be positiv integer",true);
            if (!secceeded)
            {
                centerYTBox.Text = Sphere.DefCenter.y.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }
        }

        private void centerZTBox_LostFocus(object sender, EventArgs e)
        {
            bool secceeded = ParseDouble(ref center.z, centerZTBox.Text, "Coordinate must be positiv integer",true);
            if (!secceeded)
            {
                centerZTBox.Text = Sphere.DefCenter.z.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }
        }

        private void thetaTBox_LostFocus(object sender, EventArgs e)
        {
            bool secceeded = ParseInt(ref thetaRes, thetaTBox.Text, "ThetaRes must be positiv integer");
            if (!secceeded)
            {
                thetaTBox.Text = Sphere.DefThetaRes.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }
        }

        private void phiTBox_LostFocus(object sender, EventArgs e)
        {
            bool secceeded = ParseInt(ref phiRes, phiTBox.Text, "Coordinate must be positiv integer");
            if (!secceeded)
            {
                phiTBox.Text = Sphere.DefPhiRes.ToString();
            }
            else
            {
                glyphParametrsChanged = true;
            }
        }
    }
}
