using System;
using Zcu.Mve.Core;
using System.Collections;
using System.Xml;
using System.Globalization;
using System.Diagnostics;

namespace Zcu.Mve.Visualization
{
	/// <summary>
	/// Tda pedstavuje nepravidelnou 2D mku, kter je dna dvmi double
	/// poli (jedno ve smru x, druh ve smru y) pedstavujcmi rozloen mky.
	/// Mka se skld z pixel, kad pixel je tvoen 4-mi body.
	/// S pixely i s body me bt asociovn libovoln poet atribut.
	/// </summary>
	public class RectilinearGrid2D : Grid2D
	{
		/// <summary>
		/// Pole pedstavujc rozloen bod ve smru osy x.
		/// Hodnoty museji byt v intervalu 0 az 1.
		/// </summary>
		private double[] arrayX;

		/// <summary>
		/// Pole pedstavujc rozloen bod ve smru osy y.
		/// Hodnoty museji byt v intervalu 0 az 1.
		/// </summary>
		private double[] arrayY;

		/// <summary>
		/// Implicitn konstruktor - vytvo przdn objekt (pouv se typicky pi natn dat ze souboru).
		/// </summary>
		public RectilinearGrid2D()
		{
			this.arrayY = null;
			this.arrayX = null;
		} // RectilinearGrid2D()

		/// <summary>
		/// Konstruktor - vytvo przdnou mku.
		/// </summary>
		/// <param name="arrayY">Pole pedstavujc rozloen bod ve smru osy y.</param>
		/// <param name="arrayX">Pole pedstavujc rozloen bod ve smru osy x.</param>
		public RectilinearGrid2D(double[] arrayY, double[] arrayX) : base(arrayY.Length - 1, arrayX.Length - 1)
		{
			this.arrayY = arrayY.Clone();
			this.arrayX = arrayX.Clone();
		} // RectilinearGrid2D()

		/// <summary>
		/// Vrac pole pedstavujc rozloen bod ve smru osy x.
		/// </summary>
		public double[] ArrayX
		{
			get
			{
				return this.arrayX;
			}
		} // ArrayX

		/// <summary>
		/// Vrac pole pedstavujc rozloen bod ve smru osy y.
		/// </summary>
		public double[] ArrayY
		{
			get
			{
				return this.arrayY;
			}
		} // ArrayY

// ===== Metody DataObject ========================================================

		/// <summary>
		/// Metoda pete data z Xml souboru do struktur datovho objektu.
		/// </summary>
		/// <param name="xmlTextReader">Datov tok pro ten dat z Xml souboru.</param>
		public override void ReadData(System.Xml.XmlTextReader xmlTextReader)
		{
			while(xmlTextReader.Read())
				if (xmlTextReader.NodeType == XmlNodeType.Element)
				{
					switch(xmlTextReader.Name)
					{
						case "arrays":			// naten pol arrayX a arrayY
							this.ReadArrays(xmlTextReader);
							InitializeGrid2D(this.arrayX.Length - 1, this.arrayY.Length - 1);
							break;
						case "pointsattrs":		// naten atribut bod
							ReadPointsAttrs(xmlTextReader);
							break;
						case "pixelsattrs":		// naten atribut pixel
							ReadPixelsAttrs(xmlTextReader);
							break;
						default:
							break;
					} // switch
				} // if
		} // ReadData()

		/// <summary>
		/// Metoda zape cel datov obsah datovho objektu do Xml souboru.
		/// </summary>
		/// <param name="xmlTextWriter">Datov tok pro zpis dat do Xml souboru.</param>
		public override void WriteData(System.Xml.XmlTextWriter xmlTextWriter)
		{
			// zapis poli arrayX a arrayY
			xmlTextWriter.WriteStartElement("arrays");
			xmlTextWriter.WriteAttributeString("count", "2");
			WriteArray(xmlTextWriter, this.arrayX);
			WriteArray(xmlTextWriter, this.arrayY);
			xmlTextWriter.WriteEndElement();
			// zapis atributu bodu
			WritePointsAttrs(xmlTextWriter);
			// zapis atributu pixelu
			WritePixelsAttrs(xmlTextWriter);
		} // WriteData()

		/// <summary>
		/// Hlubok kopie datovho objektu.
		/// </summary>
		/// <returns>Kopie datovho objektu (stejn data, jin datov prostor).</returns>
		public override DataObject DeepCopy()
		{
			// kopirovani poli arrayX a arrayY
			double[] newArrayX = new double[this.arrayX.Length];
			double[] newArrayY = new double[this.arrayY.Length];
			for(int i = 0; i < this.arrayX.Length; i++)
				newArrayX[i] = this.arrayX[i];
			for(int i = 0; i < this.arrayY.Length; i++)
				newArrayY[i] = this.arrayY[i];
			RectilinearGrid2D tmp = new RectilinearGrid2D(newArrayX, newArrayY);
			// kopirovani atributu bodu
			for(int i = 0; i < this.pointAttributes.GetLength(0); i++)
				for(int j = 0; j < this.pointAttributes.GetLength(1); j++)
				{
					ArrayList pointAttrList = this.pointAttributes[i, j];
					for (int k = 0; k < pointAttrList.Count; k++)
						tmp.AddPointAttr(i, j, (DataObject) ((DataObject) pointAttrList[k]).DeepCopy());
				}
			// kopirovani atributu pixelu
			for(int i = 0; i < this.pixelAttributes.GetLength(0); i++)
				for(int j = 0; j < this.pixelAttributes.GetLength(1); j++)
				{
					ArrayList pixelAttrList = this.pixelAttributes[i, j];
					for (int k = 0; k < pixelAttrList.Count; k++)
						tmp.AddPixelAttr(i, j, (DataObject) ((DataObject) pixelAttrList[k]).DeepCopy());
				}
			return tmp;
		} // DeepCopy()

		/// <summary>
		/// Prov konzistenci datov struktury.
		/// </summary>
		/// <returns>Vrac true v ppad, e je datov struktura v podku, jinak vrac false.</returns>
		public override bool CheckConsistence()
		{
			bool isOK = base.CheckConsistence();
			if (this.arrayX == null || this.arrayY == null)
				return isOK;
			// kontrola usporadanosti pole arrayX
			for (int i = 0; i < this.arrayX.Length - 1; i++)
				if (this.arrayX[i] >= this.arrayX[i + 1])
				{
					Console.WriteLine("Error! " + this.GetType().Name + ": arrayX[" + i + "] = " + this.arrayX[i] + " >= arrayX[" + (i + 1) + "] = " + this.arrayX[i + 1]);
					isOK = false;
				}
			// kontrola usporadanosti pole arrayY
			for (int i = 0; i < this.arrayY.Length - 1; i++)
				if (this.arrayY[i] >= this.arrayY[i + 1])
				{
					Console.WriteLine("Error! " + this.GetType().Name + ": arrayY[" + i + "] = " + this.arrayY[i] + " >= arrayY[" + (i + 1) + "] = " + this.arrayY[i + 1]);
					isOK = false;
				}
			// kontrola nalezitosti intervalu <0.0, 1.0> pole arrayX
			if (this.arrayX[0] < 0.0 || this.arrayX[0] > 1.0 || this.arrayX[this.arrayX.Length - 1] < 0.0 || this.arrayX[this.arrayX.Length - 1] > 1.0)
			{
				Console.WriteLine("Error! " + this.GetType().Name + ": some element of arrayX is out of range <0.0, 1.0>");
				isOK = false;
			}
			// kontrola nalezitosti intervalu <0.0, 1.0> pole arrayY
			if (this.arrayY[0] < 0.0 || this.arrayY[0] > 1.0 || this.arrayY[this.arrayY.Length - 1] < 0.0 || this.arrayY[this.arrayY.Length - 1] > 1.0)
			{
				Console.WriteLine("Error! " + this.GetType().Name + ": some element of arrayY is out of range <0.0, 1.0>");
				isOK = false;
			}
			return isOK;
		} // CheckConsistence()

// ===== Pomocn metody ===========================================================

		/// <summary>
		/// Pomocn metoda pro naten pol arrayX a arrayY z xml souboru.
		/// </summary>
		/// <param name="xmlTextReader">Datov proud pro ten dat z xml souboru.</param>
		protected void ReadArrays(XmlTextReader xmlTextReader)
		{
			NumberFormatInfo nfi = new CultureInfo("en-US", false).NumberFormat;
			// poet vech pol
			int dimCount = Int32.Parse(xmlTextReader.GetAttribute("count"));
			if (dimCount != 2)
				throw new MveVisualizationException(this.GetType().Name + ": error at reading file. Bad file structure.");
			// poet akutln natench pol
			int i = 0;
			// natej dokud nejsou natena vechna pole nebo nen dosaeno konce xml souboru
			while(xmlTextReader.Read() && i < dimCount)
			{
				if (xmlTextReader.NodeType == XmlNodeType.Element && xmlTextReader.Name == "array")
				{
					string[] tmpArrayString = xmlTextReader.ReadString().Trim().Split(' ');
					double[] tmpArrayDouble = new double[tmpArrayString.Length];
					for (int j = 0; j < tmpArrayString.Length; j++)
						tmpArrayDouble[j] = Double.Parse(tmpArrayString[j], nfi);
					if (i == 0)
						this.arrayX = tmpArrayDouble;
					else if (i == 1)
						this.arrayY = tmpArrayDouble;
					i++;
				}
			} // while
		} // ReadArrays()

		/// <summary>
		/// Pomocn metoda pro zpis pol definujcch mku do souboru.
		/// </summary>
		/// <param name="xmlTextWriter">Datov tok pro zpis dat do Xml souboru.</param>
		/// <param name="array">Pole kter se zape do souboru.</param>
		protected void WriteArray(XmlTextWriter xmlTextWriter, double[] array)
		{
			NumberFormatInfo nfi = new CultureInfo("en-US", false).NumberFormat;
			xmlTextWriter.WriteStartElement("array");
			string tmp = "";
			for (int i = 0; i < array.Length; i++)
				tmp += array[i].ToString(nfi) + " ";
			xmlTextWriter.WriteString(tmp.Substring(0, tmp.Length - 1));
			xmlTextWriter.WriteEndElement();
		} // WriteArray()

// ===== Metody Grid2D ============================================================

		/// <summary>
		/// Vrac bod zadan pozic v mce. Vrcen bod m ji konkrtn souadnice - reln sla v intervalu 0.0 a 1.0
		/// </summary>
		/// <param name="i">Index bodu v mce ve smru osy y. Mus bt v intervalu 0 a height.</param>
		/// <param name="j">Index bodu v mce ve smru osy x. Mus bt v intervalu 0 a widht.</param>
		/// <returns>Bod s konkrtnmi souadnicemi - reln sla v intervalu 0.0 a 1.0</returns>
		public override Point2D GetPoint(int i, int j)
		{
			if (i < this.pointAttributes.GetLength(0) && j < this.pointAttributes.GetLength(1))
				return new Point2D(this.arrayX[j], this.arrayY[i]);
			else
			{
				Debug.WriteLine(this.GetType().Name + ": point [" + i + ", " + j + "] is out of range \n" + this.ToString());
				return null;
			}		
		} // GetPoint()

	} // class RectilinearGrid2D
} // namespace
