using System;
using System.Collections.Generic;
using System.Text;
using Zcu.Mve.Core;
using Zcu.Mve.Visualization;

namespace Zcu.Mve.Visualization.Processing
{
    class TriangleMeshFixer:Module
    {
        public TriangleMeshFixer()
        {
            AddInPort("Input mesh", typeof(TriangleMesh));
            AddOutPort("Output mesh", typeof(TriangleMesh));
        }

        public override void Execute()
        {
            TriangleMesh input = (TriangleMesh)GetInput("Input mesh");
            UniformDataArray points = input.Points;
            UniformDataArray triangles = input.GetCells(typeof(Triangle))[0];
            bool[] usages = new bool[points.Count];
            foreach (Triangle t in triangles)
            {
                usages[t.V1] = true;
                usages[t.V2] = true;
                usages[t.V3] = true;
            }

            int shift = 0;            
            int[] shifts = new int[points.Count];
            for (int i = 0; i < points.Count; i++)
            {
                if (usages[i] == false)
                {
                    shift++;                    
                }
                shifts[i] = shift;
            }
            TriangleMesh result = (TriangleMesh)input.ShallowCopy();
            if (shift>0)
            {
                UniformDataArray newTriangles = new UniformDataArray(typeof(Triangle),triangles.Count);
                for (int i = 0; i < triangles.Count; i++)
                {
                    Triangle t = (Triangle)triangles[i];
                    t.V1 -= shifts[t.V1];
                    t.V2 -= shifts[t.V2];
                    t.V3 -= shifts[t.V3];
                    newTriangles[i] = t;
                }
                UniformDataArray newPoints = new UniformDataArray(points.ElemType, points.Count - shift);
                int index = 0;
                for (int i = 0;i<points.Count;i++)
                {
                    if (usages[i])
                    {
                        newPoints[index] = points[i];
                        index++;
                    }
                }
                string id = input.GetCellIds(typeof(Triangle))[0];                
                result.RemoveCells(id);
                result.AddCells(id, newTriangles);
                result.Points = newPoints;
            }
            SetOutput("Output mesh", result);
        }
    }
}
