//using System;
//using System.ComponentModel;
//using System.Collections.Generic;
//using System.ComponentModel.Design;
//using System.Diagnostics;

//using Zcu.Mve.Core;
//using Zcu.Mve.Numerics;
//using Zcu.Mve.Visualization;

//namespace Zcu.Mve.Visualization.Processing
//{
//    /// <summary>
//    /// <b>Centred Lattice cube</b> is method for iso-surface extraction. Each cell is split into 24 tetraherons
//    /// whereas tetrahedrons trench into neighbouring cell. All tetraherons (24) are same shape. In comparsion
//    /// with Marching Cubes is CCL: generate smooth surface, more computing time expensive, much more tringles 
//    /// in output grid and values in the middle of cells are approximated.
//    /// </summary>
//    [ModuleInfo("Novak Frantisek", "Centred Lattice cube iso-surface extractor.", IconName = "CCL.ico")]
//    [PortInfo("RegGridND", "RegGridND")]
//    [PortInfo("Treshold", "Treshold")]
//    [PortInfo("Out", "TriangleMesh")]
//    public class CCL : Zcu.Mve.Core.Module
//    {
//        #region Variables declaration

//        /// <summary>
//        /// Treshold value in %
//        /// </summary>
//        public static double TresholdValue;
//        private double tresholdPercent = 50;
//        private bool useThresholdAsIsoValue = false;
//        private Point3D delta; // distance between values in grid      
//        private Point3D halfDelta;
//        private Point3D distortion;
//        private Octahedron oHedron = null; // 4 tetrahedrons with shared edges

//        private int kdCells;
//        private int indicesValue;
//        private List<int> indices = null;
//        private List<Point3D> vertices = null;
//        private List<Normal3D> normals = null;
//        private KdCell[] kdCell = null; // kd-tree cells
//        private KdNode[] kdNode = null; // kd-tree nodes

//        private int row;   // X
//        private int slice; // X * Y
//        private int length; // X * Y * Z
//        private double min = double.MaxValue, max = double.MinValue;        
//        private int[] size; // x, y, z
//        private RegGridND data = null; // input data
//        private MveArray vData = null;  // volume data
//        private Dictionary<DEdge, int> dictionary = null;

//        private Cell[] upBuffer = null;     // global buffer
//        private Cell[] middleBuffer = null; // global buffer
//        private Cell[] downBuffer = null;   // global buffer
//        private Edge[] localBuffer = null;  // local buffer

//        private bool paramsChanged;
//        private string pointAttributesName = "Values";
//        private string normalAttributeName = "Normal3D";
//        private bool kdTree;
//        private bool kdBuild;

//        private TriangleMesh triMesh = null;

//        #endregion

//        /// <summary>
//        /// Default constructor.
//        /// </summary>
//        public CCL()
//        {
//            AddInPort("RegGridND", typeof(RegGridND));
//            AddInPort("Treshold", typeof(Integer), false);
//            AddOutPort("Out", typeof(TriangleMesh));

//            paramsChanged = true;
//            kdTree = false;
//        }

//        /// <summary>
//        /// Entry point of module. By parameters choose CCL with or without preprocessing.
//        /// </summary>
//        public override void Execute()
//        {
//            if (!IsAllDataSame() || paramsChanged)
//            {
//                long dt;
//                System.Diagnostics.Debug.WriteLine("Centred Lattice cube");

//                System.GC.Collect();

//                data = (RegGridND)GetInput("RegGridND");
//                vData = data.GetPointAttrs(pointAttributesName);

//                if (GetInput("Treshold") != null)
//                    tresholdPercent = (Integer)GetInput("Treshold");

//                if (!IsDataSame("RegGridND"))
//                    kdBuild = false;

//                if (kdTree) //kd-tree preprocessing
//                {
//                    indicesValue = 0;

//                    if (useThresholdAsIsoValue)
//                        CCL.TresholdValue = tresholdPercent;
//                    else
//                        CCL.TresholdValue = min + (max - min) * tresholdPercent * 0.01f;

//                    if (!kdBuild)
//                    {
//                        kdCells = 0;
//                        Initialize();
//                        kdCell = new KdCell[length / 2];
//                        kdBuild = true;
//                        BuildKdTree();
//                    }

//                    dt = Environment.TickCount;
//                    WalkThroughMinLevel(0, kdCells);
//                }
//                else // CCL without preprocessing
//                {
//                    Initialize();
//                    dt = Environment.TickCount;
//                    WalkThrough();
//                }

//                vData = null;

//                System.Diagnostics.Debug.WriteLine(string.Format(" Treashold: {0}%", tresholdPercent));
//                System.Diagnostics.Debug.WriteLine(string.Format(" Iso-surface extraction time: {0} ms", Environment.TickCount - dt));
//                System.Diagnostics.Debug.WriteLine(string.Format(" Triangles: {0}", indices.Count / 3, vertices.Count));
//                System.Diagnostics.Debug.WriteLine(string.Format(" Points: {0}\n", vertices.Count));

//                CreateMesh();

//                SetOutput("Out", triMesh);

//                paramsChanged = false;
//                System.GC.Collect();
//            }
//            else
//                SetOutput("Out", triMesh, true);
//        }

//        /// <summary>
//        /// Fill triangleMesh with points, normals and trinagles.
//        /// </summary>
//        private void CreateMesh()
//        {
//            triMesh = new TriangleMesh();

//            triMesh.Points = UniformDataArray.FromList<Point3D>(vertices);
//            vertices.Clear();

//            ProgressInfo(0.86f);

//            UniformDataArray cells = new UniformDataArray(typeof(Triangle), indices.Count / 3);
//            int index = 0;
//            for (int i = 0; i < indices.Count; i += 3)
//            {
//                cells[index] = new Triangle(indices[i], indices[i + 1], indices[i + 2]);
//                index++;
//            }

//            triMesh.AddCells("Triangles", cells);
//            indices.Clear();

//            ProgressInfo(0.93f);

//            UniformDataArray normals = new UniformDataArray(typeof(Normal3D), this.normals.Count);
//            for (int i = 0; i < normals.Count; i++)
//                normals[i] = normals[i];

//            triMesh.AddPointAttrs(normalAttributeName, UniformDataArray.FromList<Normal3D>(this.normals));

//            this.normals.Clear();

//            ProgressInfo(1f);
//        }

//        /// <summary>
//        /// Inicialization structures.
//        /// </summary>
//        private void Initialize()
//        {
//            size = vData.Lengths;
//            length = size[0] * size[1] * size[2];

//            indices = new List<int>(length); // triangle indices
//            vertices = new List<Point3D>(length / 2); // points
//            normals = new List<Normal3D>(length / 2); //normals

//            slice = size[0] * size[1];
//            row = size[0];
//            indicesValue = 0;

//            // gloabal buffers
//            upBuffer = new Cell[slice];
//            middleBuffer = new Cell[slice];
//            downBuffer = new Cell[slice];
//            dictionary = new Dictionary<DEdge, int>(slice * 2);

//            // loacal buffer, used during coumputing one cell
//            localBuffer = new Edge[20]; // shared edges
//            oHedron = new Octahedron();

//            delta = new Point3D(1, 1, 1);
//            halfDelta = new Point3D(0.5f, 0.5f, 0.5f);
//            distortion = new Point3D(1, 1, 1);

//            bool allAtributes = true;

//            if (data.UserTags.ContainsKey("Min"))
//                min = (Scalar)data.UserTags["Min"];
//            else
//            {
//                min = 0;
//                allAtributes = false;
//            }

//            if (data.UserTags.ContainsKey("Max"))
//                max = (Scalar)data.UserTags["Max"];
//            else
//            {
//                max = 255;
//                allAtributes = false;
//            }

//            if (data.UserTags.ContainsKey("Distortion"))
//                distortion = (Point3D)data.UserTags["Distortion"];
//            else
//            {
//                distortion = new Point3D(1, 1, 1);
//                allAtributes = false;
//            }

//            distortion = new Point3D(1.0 / distortion.X, 1.0 / distortion.Y, 1.0 / distortion.Z);

//            if (!allAtributes)
//                throw new Exception("Attributes not set. For extraction used default values.");

//            if (useThresholdAsIsoValue)
//                CCL.TresholdValue = tresholdPercent;
//            else
//                CCL.TresholdValue = min + (max - min) * tresholdPercent * 0.01f;
//        }

//        #region CCL + Kd-tree

//        /// <summary>
//        /// Method used for recursive building of tree at min level. 
//        /// Min level = values are sort by minimal value of cell.
//        /// </summary>
//        /// <paramsChanged name="dm">lower limit</paramsChanged>
//        /// <paramsChanged name="length">number cells</paramsChanged>
//        private void MinLevel(int dm, int length)
//        {
//            Array.Sort<KdNode>(kdNode, dm, length, new SortByMin());
//            int root = dm + length / 2;
//            int hm = dm + length;

//            if (dm < root - 1) MaxLevel(dm, root - dm); // left subtree
//            if (hm > root + 1) MaxLevel(root + 1, hm - (root + 1)); // right subtree
//        }

//        /// <summary>
//        /// Method used for recursive building of tree at max level.
//        /// Max level = values are sort by maximum value of cell.
//        /// </summary>
//        /// <paramsChanged name="dm">upper limit</paramsChanged>
//        /// <paramsChanged name="length">number cells</paramsChanged>
//        private void MaxLevel(int dm, int length)
//        {
//            Array.Sort<KdNode>(kdNode, dm, length, new SortByMax());
//            int root = dm + length / 2;
//            int hm = dm + length;

//            if (dm < root - 1) MinLevel(dm, root - dm); // left subtree
//            if (hm > root + 1) MinLevel(root + 1, hm - (root + 1)); // right subtree
//        }

//        /// <summary>
//        /// Passing through the tree at min level and searching active cells.
//        /// </summary>
//        /// <paramsChanged name="dm">lower limit</paramsChanged>
//        /// <paramsChanged name="length">umber cells</paramsChanged>
//        private void WalkThroughMinLevel(int dm, int length)
//        {
//            int root = dm + length / 2;
//            int hm = dm + length;

//            // processing of root
//            if (kdNode[root].Min <= CCL.TresholdValue && kdNode[root].Max >= CCL.TresholdValue)
//                Tetrahedronize(kdCell[kdNode[root].pKdCell]);

//            if (length <= 1)
//                return;

//            if (kdNode[root].Min > CCL.TresholdValue) // test whether is necessary passing through both subtree
//            {
//                if (dm < root) WalkThroughMaxLevel(dm, root - dm);
//            }
//            else
//            {
//                if (dm < root) WalkThroughMaxLevel(dm, root - dm); // left subtree
//                if (hm > root + 1) WalkThroughMaxLevel(root + 1, hm - (root + 1)); // right subtree
//            }
//        }

//        /// <summary>
//        /// Passing through the tree at max level and searching active cells.
//        /// </summary>
//        /// <paramsChanged name="dm">lower limit</paramsChanged>
//        /// <paramsChanged name="length">umber cells</paramsChanged>
//        private void WalkThroughMaxLevel(int dm, int length)
//        {
//            int root = dm + length / 2;
//            int hm = dm + length;

//            // processing of root
//            if ((kdNode[root].Min <= CCL.TresholdValue) && kdNode[root].Max >= CCL.TresholdValue)
//                Tetrahedronize(kdCell[kdNode[root].pKdCell]);

//            if (length <= 1)
//                return;

//            if (kdNode[root].Max < CCL.TresholdValue) // test whether is necessary passing through both subtree
//            {
//                if (hm > root + 1) WalkThroughMinLevel(root + 1, hm - (root + 1)); // right subtree
//            }
//            else
//            {
//                if (dm < root) WalkThroughMinLevel(dm, root - dm); // left subtree
//                if (hm > root + 1) WalkThroughMinLevel(root + 1, hm - (root + 1)); // right subtree
//            }
//        }

//        /// <summary>
//        /// Divide cell into tetrahedrons. Determination edges which are intersect by iso-surface.
//        /// </summary>
//        /// <paramsChanged name="kdCell">cell</paramsChanged>
//        private void Tetrahedronize(KdCell kdCell)
//        {
//            Cell cell = kdCell.cell;

//            // vertices coordinates
//            Point3D e = new Point3D(kdCell.i, kdCell.j, kdCell.k);
//            Point3D f = new Point3D(kdCell.i + delta.X, kdCell.j, kdCell.k);
//            Point3D g = new Point3D(kdCell.i + delta.X, kdCell.j, kdCell.k + delta.Z);
//            Point3D h = new Point3D(kdCell.i, kdCell.j, kdCell.k + delta.Z);

//            Point3D a = new Point3D(kdCell.i, kdCell.j + delta.Y, kdCell.k);
//            Point3D b = new Point3D(kdCell.i + delta.X, kdCell.j + delta.Y, kdCell.k);
//            Point3D c = new Point3D(kdCell.i + delta.X, kdCell.j + delta.Y, kdCell.k + delta.Z);
//            Point3D d = new Point3D(kdCell.i, kdCell.j + delta.Y, kdCell.k + delta.Z);

//            Point3D s = e + halfDelta;

//            // 6 neighbouring centres 
//            Centers adjacentCenters = kdCell.centers;

//            // normals
//            Normals n = kdCell.normals;

//            Tetrahedron tHedron = new Tetrahedron();

//            int iE = (2 * kdCell.k * slice) + (2 * kdCell.j * row) + (2 * kdCell.i); // vrchol E
//            int iF = iE + 2; // F
//            int iH = iE + 2 * slice; // H
//            int iG = iH + 2; // G

//            int iA = iE + 2 * row;
//            int iB = iA + 2;
//            int iD = iH + 2 * row;
//            int iC = iD + 2;

//            int iS = iE + slice + row + 1;

//            Array.Clear(localBuffer, 0, localBuffer.Length);

//            // front side cell tetrahedrons
//            #region Front side

//            // front centers coordintes
//            Point3D front = new Point3D(s.X - delta.X, s.Y, s.Z);
//            int iFront = iS - 2;

//            // assignment values to tetrahedron
//            tHedron.SetValues(adjacentCenters.Front, cell.A, cell.E, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate()) // if values aren't same and iso-surface intersect some edge
//            {
//                tHedron.SetIndex(iFront, iA, iE, iS);
//                tHedron.SetIndices(Octahedron.topIndices); // set edges from octahedron, octahedron is used as pointer to shared edges
//                Intersections(tHedron, front, a, e, s, n.NFront, n.NA, n.NE, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Front, cell.E, cell.H, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iFront, iE, iH, iS);
//                tHedron.SetIndices(Octahedron.backIndices);
//                Intersections(tHedron, front, e, h, s, n.NFront, n.NE, n.NH, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Front, cell.H, cell.D, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iFront, iH, iD, iS);
//                tHedron.SetIndices(Octahedron.downIndices);
//                Intersections(tHedron, front, h, d, s, n.NFront, n.NH, n.ND, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Front, cell.D, cell.A, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iFront, iD, iA, iS);
//                tHedron.SetIndices(Octahedron.frontIndices);
//                Intersections(tHedron, front, d, a, s, n.NFront, n.ND, n.NA, n.NS);
//            }

//            localBuffer[0] = oHedron.Edges[5]; // copy shared edges from octahedron (defined by 4 tetrahedrons)
//            localBuffer[2] = oHedron.Edges[6]; // to local buffer. Local buffer is used for edges shared between
//            localBuffer[12] = oHedron.Edges[7]; // octahedrons
//            localBuffer[9] = oHedron.Edges[8];
//            localBuffer[16] = oHedron.Edges[9];
//            localBuffer[19] = oHedron.Edges[10];
//            localBuffer[7] = oHedron.Edges[11];
//            localBuffer[15] = oHedron.Edges[12];

//            #endregion

//            oHedron.Clear(); // clear shared edges between tetraherons

//            #region Left side

//            oHedron.Edges[6] = localBuffer[9];
//            oHedron.Edges[7] = localBuffer[16];
//            oHedron.Edges[12] = localBuffer[19];

//            // souradnice vrcholu a tetrahedronu
//            Point3D left = new Point3D(s.X, s.Y - delta.Y, s.Z);
//            int iLeft = iS - 2 * row;

//            tHedron.SetValues(adjacentCenters.Left, cell.E, cell.F, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iLeft, iE, iF, iS);
//                tHedron.SetIndices(Octahedron.topIndices);
//                Intersections(tHedron, left, e, f, s, n.NLeft, n.NE, n.NF, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Left, cell.F, cell.G, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iLeft, iF, iG, iS);
//                tHedron.SetIndices(Octahedron.backIndices);
//                Intersections(tHedron, left, f, g, s, n.NLeft, n.NF, n.NG, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Left, cell.G, cell.H, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iLeft, iG, iH, iS);
//                tHedron.SetIndices(Octahedron.downIndices);
//                Intersections(tHedron, left, g, h, s, n.NLeft, n.NG, n.NH, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Left, cell.H, cell.E, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iLeft, iH, iE, iS);
//                tHedron.SetIndices(Octahedron.frontIndices);
//                Intersections(tHedron, left, h, e, s, n.NLeft, n.NH, n.NE, n.NS);
//            }

//            localBuffer[8] = oHedron.Edges[5];
//            localBuffer[10] = oHedron.Edges[8];
//            localBuffer[17] = oHedron.Edges[9];
//            localBuffer[18] = oHedron.Edges[10];
//            localBuffer[11] = oHedron.Edges[11];

//            #endregion

//            oHedron.Clear();

//            #region Top side

//            oHedron.Edges[12] = localBuffer[16];
//            oHedron.Edges[11] = localBuffer[8];
//            oHedron.Edges[10] = localBuffer[17];
//            oHedron.Edges[7] = localBuffer[12];
//            oHedron.Edges[6] = localBuffer[0];

//            Point3D top = new Point3D(s.X, s.Y, s.Z - delta.Z);
//            int iTop = iS - 2 * slice;

//            tHedron.SetValues(adjacentCenters.Top, cell.A, cell.B, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iTop, iA, iB, iS);
//                tHedron.SetIndices(Octahedron.topIndices);
//                Intersections(tHedron, top, a, b, s, n.NTop, n.NA, n.NB, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Top, cell.B, cell.F, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iTop, iB, iF, iS);
//                tHedron.SetIndices(Octahedron.backIndices);
//                Intersections(tHedron, top, b, f, s, n.NTop, n.NB, n.NF, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Top, cell.F, cell.E, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iTop, iF, iE, iS);
//                tHedron.SetIndices(Octahedron.downIndices);
//                Intersections(tHedron, top, f, e, s, n.NTop, n.NF, n.NE, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Top, cell.E, cell.A, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iTop, iE, iA, iS);
//                tHedron.SetIndices(Octahedron.frontIndices);
//                Intersections(tHedron, top, e, a, s, n.NTop, n.NE, n.NA, n.NS);
//            }

//            localBuffer[1] = oHedron.Edges[5];
//            localBuffer[4] = oHedron.Edges[8];
//            localBuffer[13] = oHedron.Edges[9];

//            #endregion

//            oHedron.Clear();

//            #region Down side

//            oHedron.Edges[10] = localBuffer[18];
//            oHedron.Edges[9] = localBuffer[19];
//            oHedron.Edges[8] = localBuffer[11];
//            oHedron.Edges[7] = localBuffer[15];
//            oHedron.Edges[5] = localBuffer[7];

//            Point3D down = new Point3D(s.X, s.Y, s.Z + delta.Z);
//            int iDown = iS + 2 * slice;

//            tHedron.SetValues(adjacentCenters.Down, cell.D, cell.H, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iDown, iD, iH, iS);
//                tHedron.SetIndices(Octahedron.topIndices);
//                Intersections(tHedron, down, d, h, s, n.NDown, n.ND, n.NH, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Down, cell.H, cell.G, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iDown, iH, iG, iS);
//                tHedron.SetIndices(Octahedron.backIndices);
//                Intersections(tHedron, down, h, g, s, n.NDown, n.NH, n.NG, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Down, cell.G, cell.C, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iDown, iG, iC, iS);
//                tHedron.SetIndices(Octahedron.downIndices);
//                Intersections(tHedron, down, g, c, s, n.NDown, n.NG, n.NC, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Down, cell.C, cell.D, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iDown, iC, iD, iS);
//                tHedron.SetIndices(Octahedron.frontIndices);
//                Intersections(tHedron, down, c, d, s, n.NDown, n.NC, n.ND, n.NS);
//            }

//            localBuffer[5] = oHedron.Edges[6];
//            localBuffer[6] = oHedron.Edges[11];
//            localBuffer[14] = oHedron.Edges[12];

//            #endregion

//            oHedron.Clear();

//            #region Right side

//            oHedron.Edges[5] = localBuffer[1];
//            oHedron.Edges[7] = localBuffer[13];
//            oHedron.Edges[8] = localBuffer[2];
//            oHedron.Edges[9] = localBuffer[12];
//            oHedron.Edges[10] = localBuffer[15];
//            oHedron.Edges[11] = localBuffer[5];
//            oHedron.Edges[12] = localBuffer[14];

//            Point3D right = new Point3D(s.X, s.Y + delta.Y, s.Z);
//            int iRight = iS + 2 * row;

//            tHedron.SetValues(adjacentCenters.Right, cell.B, cell.A, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iRight, iB, iA, iS);
//                tHedron.SetIndices(Octahedron.topIndices);
//                Intersections(tHedron, right, b, a, s, n.NRight, n.NB, n.NA, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Right, cell.A, cell.D, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iRight, iA, iD, iS);
//                tHedron.SetIndices(Octahedron.backIndices);
//                Intersections(tHedron, right, a, d, s, n.NRight, n.NA, n.ND, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Right, cell.D, cell.C, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iRight, iD, iC, iS);
//                tHedron.SetIndices(Octahedron.downIndices);
//                Intersections(tHedron, right, d, c, s, n.NRight, n.ND, n.NC, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Right, cell.C, cell.B, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iRight, iC, iB, iS);
//                tHedron.SetIndices(Octahedron.frontIndices);
//                Intersections(tHedron, right, c, b, s, n.NRight, n.NC, n.NB, n.NS);
//            }

//            localBuffer[3] = oHedron.Edges[6];

//            #endregion

//            oHedron.Clear();

//            #region Back side

//            oHedron.Edges[5] = localBuffer[4];
//            oHedron.Edges[6] = localBuffer[3];
//            oHedron.Edges[7] = localBuffer[13];
//            oHedron.Edges[8] = localBuffer[10];
//            oHedron.Edges[9] = localBuffer[17];
//            oHedron.Edges[10] = localBuffer[18];
//            oHedron.Edges[11] = localBuffer[6];
//            oHedron.Edges[12] = localBuffer[14];

//            Point3D back = new Point3D(s.X + delta.X, s.Y, s.Z);
//            int iBack = iS + 2;

//            tHedron.SetValues(adjacentCenters.Back, cell.B, cell.F, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iBack, iB, iF, iS);
//                tHedron.SetIndices(Octahedron.topIndices);
//                Intersections(tHedron, back, b, f, s, n.NBack, n.NB, n.NF, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Back, cell.F, cell.G, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iBack, iF, iG, iS);
//                tHedron.SetIndices(Octahedron.backIndices);
//                Intersections(tHedron, back, f, g, s, n.NBack, n.NF, n.NG, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Back, cell.G, cell.C, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iBack, iG, iC, iS);
//                tHedron.SetIndices(Octahedron.downIndices);
//                Intersections(tHedron, back, g, c, s, n.NBack, n.NG, n.NC, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Back, cell.C, cell.B, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iBack, iC, iB, iS);
//                tHedron.SetIndices(Octahedron.frontIndices);
//                Intersections(tHedron, back, c, b, s, n.NBack, n.NC, n.NB, n.NS);
//            }

//            #endregion

//            oHedron.Clear();
//        }

//        /// <summary>
//        /// Decision whether apend cell into kd-tree. If are all values same then cell isn't append.
//        /// </summary>
//        /// <paramsChanged name="i">column index</paramsChanged>
//        /// <paramsChanged name="j">row index</paramsChanged>
//        /// <paramsChanged name="k">slice index</paramsChanged>
//        private void SplitToCells(int i, int j, int k)
//        {
//#if(DEBUG)
//            if (i > (row - 2) && j > (row - 2) && k > (row - 2))
//                throw new IndexOutOfRangeException("Index out of range [Tetrahedronize]");
//#endif
//            Cell cell;

//            if (middleBuffer[i + j * row].Used)
//                cell = middleBuffer[i + j * row];
//            else
//            {
//                cell = GetCell(i, j, k);
//                middleBuffer[i + j * row] = cell;
//            }

//            // 6 neigbouring centres
//            Centers adjacentCenters;
//            GetAdjacentCenters(i, j, k, out adjacentCenters);

//            // all values are same
//            if (adjacentCenters.Useless() && adjacentCenters.Front == cell.S)
//                return;

//            // normals
//            Normals n = ComputeNormals(i, j, k, adjacentCenters, cell);

//            // add cell into array of cells, which are used for kd-tree
//            kdCell[kdCells++] = new KdCell(cell, adjacentCenters, n, i, j, k);
//        }

//        /// <summary>
//        /// Create kd-tree. First is created array of cells a then is created kd-tree.
//        /// </summary>
//        private void BuildKdTree()
//        {
//            long dt = Environment.TickCount;

//            ProgressInfo(0f);

//            // cells for extraction
//            for (int k = 1; k < (size[2] - 2); k++)
//            {
//                for (int j = 1; j < size[1] - 2; j++)
//                {
//                    for (int i = 1 + (j % 2 + k % 2); i < size[0] - 2; i += 2)
//                    {
//                        SplitToCells(i, j, k);
//                    }
//                }

//                ProgressInfo(((float)k / size[2]) * 0.3f);
//                SwapBuffers();
//            }

//            // kd-tree nodes array, kd-tree is defined on this array
//            kdNode = new KdNode[kdCells];

//            int block = 10000;

//            // fill kd-nodes with parametrs of cells
//            for (int i = 0; i < kdCells; i++)
//            {
//                kdNode[i] = new KdNode(i, kdCell[i].GetMin(), kdCell[i].GetMax());

//                if (i == block)
//                {
//                    ProgressInfo(0.3f + (((float)i / kdCells) * 0.2f));
//                    block += 10000;
//                }
//            }

//            ProgressInfo(0.5f);

//            // recursive building of tree
//            MinLevel(0, kdCells);

//            ProgressInfo(0.8f);

//            Console.WriteLine(" Time of build kd-tree: {0} ms", Environment.TickCount - dt);
//        }

//        #endregion

//        #region Classic CCL

//        /// <summary>
//        /// Passing through cells.
//        /// </summary>
//        private void WalkThrough()
//        {
//            for (int k = 1; k < (size[2] - 2); k++)
//            {
//                for (int j = 1; j < (size[1] - 2); j++)
//                {
//                    for (int i = 1 + (j % 2 + k % 2); i < size[0] - 2; i += 2)
//                    {
//                        Tetrahedronize(i, j, k); // divide cell to 24 tetrahedrons
//                    }
//                }

//                SwapBuffers();
//                ProgressInfo(((float)k / size[2]) * 0.8f);
//            }
//        }

//        /// <summary>
//        /// Swap buffers.
//        /// </summary>
//        private void SwapBuffers()
//        {
//            Cell[] tmp = upBuffer;

//            upBuffer = middleBuffer;
//            middleBuffer = downBuffer;
//            downBuffer = tmp;

//            Array.Clear(downBuffer, 0, downBuffer.Length);

//            tmp = null;
//        }

//        /// <summary>
//        /// Divide cell into tetrahedrons. Determination edges which are intersect by iso-surface. 
//        /// </summary>
//        /// <paramsChanged name="i">column index</paramsChanged>
//        /// <paramsChanged name="j">row index</paramsChanged>
//        /// <paramsChanged name="k">slice index</paramsChanged>
//        private void Tetrahedronize(int i, int j, int k)
//        {
//#if(DEBUG)
//            if (i > (row - 2) && j > (row - 2) && k > (row - 2))
//                throw new IndexOutOfRangeException("Index out of range [Tetrahedronize]");
//#endif
//            Cell cell;

//            if (middleBuffer[i + j * row].Used)
//                cell = middleBuffer[i + j * row];
//            else
//            {
//                cell = GetCell(i, j, k);
//                middleBuffer[i + j * row] = cell;
//            }

//            // urceni 6 okolnich stredu
//            Centers adjacentCenters;
//            GetAdjacentCenters(i, j, k, out adjacentCenters);

//            // pokud vsechny hodnoty jsou stejne
//            if (adjacentCenters.Useless() && adjacentCenters.Front == cell.S)
//                return;

//            // souradnice vrcholu bunky
//            Point3D e = new Point3D(i, j, k);
//            Point3D f = new Point3D(i + delta.X, j, k);
//            Point3D g = new Point3D(i + delta.X, j, k + delta.Z);
//            Point3D h = new Point3D(i, j, k + delta.Z);

//            Point3D a = new Point3D(i, j + delta.Y, k);
//            Point3D b = new Point3D(i + delta.X, j + delta.Y, k);
//            Point3D c = new Point3D(i + delta.X, j + delta.Y, k + delta.Z);
//            Point3D d = new Point3D(i, j + delta.Y, k + delta.Z);

//            Normals n = ComputeNormals(i, j, k, adjacentCenters, cell);

//            Point3D s = e + halfDelta;

//            Tetrahedron tHedron = new Tetrahedron();

//            Array.Clear(localBuffer, 0, localBuffer.Length);

//            int iE = (2 * k * slice) + (2 * j * row) + (2 * i); // vrchol E
//            int iF = iE + 2; // F
//            int iH = iE + 2 * slice; // H
//            int iG = iH + 2; // G

//            int iA = iE + 2 * row;
//            int iB = iA + 2;
//            int iD = iH + 2 * row;
//            int iC = iD + 2;

//            int iS = iE + slice + row + 1;

//            // front side cell tetrahedrons
//            #region Front side

//            // front centers coordintes
//            Point3D front = new Point3D(s.X - delta.X, s.Y, s.Z);
//            int iFront = iS - 2;

//            // assignment values to tetrahedron
//            tHedron.SetValues(adjacentCenters.Front, cell.A, cell.E, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate()) // if values aren't same and iso-surface intersect some edge
//            {
//                tHedron.SetIndex(iFront, iA, iE, iS);
//                tHedron.SetIndices(Octahedron.topIndices); // set edges from octahedron, octahedron is used as pointer to shared edges
//                Intersections(tHedron, front, a, e, s, n.NFront, n.NA, n.NE, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Front, cell.E, cell.H, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iFront, iE, iH, iS);
//                tHedron.SetIndices(Octahedron.backIndices);
//                Intersections(tHedron, front, e, h, s, n.NFront, n.NE, n.NH, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Front, cell.H, cell.D, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iFront, iH, iD, iS);
//                tHedron.SetIndices(Octahedron.downIndices);
//                Intersections(tHedron, front, h, d, s, n.NFront, n.NH, n.ND, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Front, cell.D, cell.A, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iFront, iD, iA, iS);
//                tHedron.SetIndices(Octahedron.frontIndices);
//                Intersections(tHedron, front, d, a, s, n.NFront, n.ND, n.NA, n.NS);
//            }

//            localBuffer[0] = oHedron.Edges[5]; // copy shared edges from octahedron (defined by 4 tetrahedrons)
//            localBuffer[2] = oHedron.Edges[6]; // to local buffer. Local buffer is used for edges shared between
//            localBuffer[12] = oHedron.Edges[7]; // octahedrons
//            localBuffer[9] = oHedron.Edges[8];
//            localBuffer[16] = oHedron.Edges[9];
//            localBuffer[19] = oHedron.Edges[10];
//            localBuffer[7] = oHedron.Edges[11];
//            localBuffer[15] = oHedron.Edges[12];

//            #endregion

//            oHedron.Clear(); // clear shared edges between tetraherons

//            #region Left side

//            oHedron.Edges[6] = localBuffer[9];
//            oHedron.Edges[7] = localBuffer[16];
//            oHedron.Edges[12] = localBuffer[19];

//            // souradnice vrcholu a tetrahedronu
//            Point3D left = new Point3D(s.X, s.Y - delta.Y, s.Z);
//            int iLeft = iS - 2 * row;

//            tHedron.SetValues(adjacentCenters.Left, cell.E, cell.F, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iLeft, iE, iF, iS);
//                tHedron.SetIndices(Octahedron.topIndices);
//                Intersections(tHedron, left, e, f, s, n.NLeft, n.NE, n.NF, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Left, cell.F, cell.G, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iLeft, iF, iG, iS);
//                tHedron.SetIndices(Octahedron.backIndices);
//                Intersections(tHedron, left, f, g, s, n.NLeft, n.NF, n.NG, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Left, cell.G, cell.H, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iLeft, iG, iH, iS);
//                tHedron.SetIndices(Octahedron.downIndices);
//                Intersections(tHedron, left, g, h, s, n.NLeft, n.NG, n.NH, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Left, cell.H, cell.E, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iLeft, iH, iE, iS);
//                tHedron.SetIndices(Octahedron.frontIndices);
//                Intersections(tHedron, left, h, e, s, n.NLeft, n.NH, n.NE, n.NS);
//            }

//            localBuffer[8] = oHedron.Edges[5];
//            localBuffer[10] = oHedron.Edges[8];
//            localBuffer[17] = oHedron.Edges[9];
//            localBuffer[18] = oHedron.Edges[10];
//            localBuffer[11] = oHedron.Edges[11];

//            #endregion

//            oHedron.Clear();

//            #region Top side

//            oHedron.Edges[12] = localBuffer[16];
//            oHedron.Edges[11] = localBuffer[8];
//            oHedron.Edges[10] = localBuffer[17];
//            oHedron.Edges[7] = localBuffer[12];
//            oHedron.Edges[6] = localBuffer[0];

//            Point3D top = new Point3D(s.X, s.Y, s.Z - delta.Z);
//            int iTop = iS - 2 * slice;

//            tHedron.SetValues(adjacentCenters.Top, cell.A, cell.B, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iTop, iA, iB, iS);
//                tHedron.SetIndices(Octahedron.topIndices);
//                Intersections(tHedron, top, a, b, s, n.NTop, n.NA, n.NB, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Top, cell.B, cell.F, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iTop, iB, iF, iS);
//                tHedron.SetIndices(Octahedron.backIndices);
//                Intersections(tHedron, top, b, f, s, n.NTop, n.NB, n.NF, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Top, cell.F, cell.E, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iTop, iF, iE, iS);
//                tHedron.SetIndices(Octahedron.downIndices);
//                Intersections(tHedron, top, f, e, s, n.NTop, n.NF, n.NE, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Top, cell.E, cell.A, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iTop, iE, iA, iS);
//                tHedron.SetIndices(Octahedron.frontIndices);
//                Intersections(tHedron, top, e, a, s, n.NTop, n.NE, n.NA, n.NS);
//            }

//            localBuffer[1] = oHedron.Edges[5];
//            localBuffer[4] = oHedron.Edges[8];
//            localBuffer[13] = oHedron.Edges[9];

//            #endregion

//            oHedron.Clear();

//            #region Down side

//            oHedron.Edges[10] = localBuffer[18];
//            oHedron.Edges[9] = localBuffer[19];
//            oHedron.Edges[8] = localBuffer[11];
//            oHedron.Edges[7] = localBuffer[15];
//            oHedron.Edges[5] = localBuffer[7];

//            Point3D down = new Point3D(s.X, s.Y, s.Z + delta.Z);
//            int iDown = iS + 2 * slice;

//            tHedron.SetValues(adjacentCenters.Down, cell.D, cell.H, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iDown, iD, iH, iS);
//                tHedron.SetIndices(Octahedron.topIndices);
//                Intersections(tHedron, down, d, h, s, n.NDown, n.ND, n.NH, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Down, cell.H, cell.G, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iDown, iH, iG, iS);
//                tHedron.SetIndices(Octahedron.backIndices);
//                Intersections(tHedron, down, h, g, s, n.NDown, n.NH, n.NG, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Down, cell.G, cell.C, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iDown, iG, iC, iS);
//                tHedron.SetIndices(Octahedron.downIndices);
//                Intersections(tHedron, down, g, c, s, n.NDown, n.NG, n.NC, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Down, cell.C, cell.D, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iDown, iC, iD, iS);
//                tHedron.SetIndices(Octahedron.frontIndices);
//                Intersections(tHedron, down, c, d, s, n.NDown, n.NC, n.ND, n.NS);
//            }

//            localBuffer[5] = oHedron.Edges[6];
//            localBuffer[6] = oHedron.Edges[11];
//            localBuffer[14] = oHedron.Edges[12];

//            #endregion

//            oHedron.Clear();

//            #region Right side

//            oHedron.Edges[5] = localBuffer[1];
//            oHedron.Edges[7] = localBuffer[13];
//            oHedron.Edges[8] = localBuffer[2];
//            oHedron.Edges[9] = localBuffer[12];
//            oHedron.Edges[10] = localBuffer[15];
//            oHedron.Edges[11] = localBuffer[5];
//            oHedron.Edges[12] = localBuffer[14];

//            Point3D right = new Point3D(s.X, s.Y + delta.Y, s.Z);
//            int iRight = iS + 2 * row;

//            tHedron.SetValues(adjacentCenters.Right, cell.B, cell.A, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iRight, iB, iA, iS);
//                tHedron.SetIndices(Octahedron.topIndices);
//                Intersections(tHedron, right, b, a, s, n.NRight, n.NB, n.NA, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Right, cell.A, cell.D, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iRight, iA, iD, iS);
//                tHedron.SetIndices(Octahedron.backIndices);
//                Intersections(tHedron, right, a, d, s, n.NRight, n.NA, n.ND, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Right, cell.D, cell.C, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iRight, iD, iC, iS);
//                tHedron.SetIndices(Octahedron.downIndices);
//                Intersections(tHedron, right, d, c, s, n.NRight, n.ND, n.NC, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Right, cell.C, cell.B, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iRight, iC, iB, iS);
//                tHedron.SetIndices(Octahedron.frontIndices);
//                Intersections(tHedron, right, c, b, s, n.NRight, n.NC, n.NB, n.NS);
//            }

//            localBuffer[3] = oHedron.Edges[6];

//            #endregion

//            oHedron.Clear();

//            #region Back side

//            oHedron.Edges[5] = localBuffer[4];
//            oHedron.Edges[6] = localBuffer[3];
//            oHedron.Edges[7] = localBuffer[13];
//            oHedron.Edges[8] = localBuffer[10];
//            oHedron.Edges[9] = localBuffer[17];
//            oHedron.Edges[10] = localBuffer[18];
//            oHedron.Edges[11] = localBuffer[6];
//            oHedron.Edges[12] = localBuffer[14];

//            Point3D back = new Point3D(s.X + delta.X, s.Y, s.Z);
//            int iBack = iS + 2;

//            tHedron.SetValues(adjacentCenters.Back, cell.B, cell.F, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iBack, iB, iF, iS);
//                tHedron.SetIndices(Octahedron.topIndices);
//                Intersections(tHedron, back, b, f, s, n.NBack, n.NB, n.NF, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Back, cell.F, cell.G, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iBack, iF, iG, iS);
//                tHedron.SetIndices(Octahedron.backIndices);
//                Intersections(tHedron, back, f, g, s, n.NBack, n.NF, n.NG, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Back, cell.G, cell.C, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iBack, iG, iC, iS);
//                tHedron.SetIndices(Octahedron.downIndices);
//                Intersections(tHedron, back, g, c, s, n.NBack, n.NG, n.NC, n.NS);
//            }

//            tHedron.SetValues(adjacentCenters.Back, cell.C, cell.B, cell.S);
//            if (!tHedron.Useless && tHedron.Evaluate())
//            {
//                tHedron.SetIndex(iBack, iC, iB, iS);
//                tHedron.SetIndices(Octahedron.frontIndices);
//                Intersections(tHedron, back, c, b, s, n.NBack, n.NC, n.NB, n.NS);
//            }

//            #endregion

//            oHedron.Clear();
//        }

//        /// <summary>
//        /// Determination intersetion of tetrahedron.
//        /// </summary>
//        private void Intersections(Tetrahedron hedron, Point3D a, Point3D b, Point3D c, Point3D d, Normal3D na, Normal3D nb, Normal3D nc, Normal3D nd)
//        {
//            DEdge ac, bc, ab, bd, ad, cd;
//            int indicia, index0, index1;

//            switch (hedron.Flags.Value)
//            {
//                case 1: // d
//                case 14:
//                    if (oHedron.Edges[hedron.EdgeIndices[4]].HasIntersection)
//                        indices.Add(oHedron.Edges[hedron.EdgeIndices[4]].IndexIB);
//                    else
//                    {
//                        bd = new DEdge(hedron.ib, hedron.id);

//                        if (dictionary.TryGetValue(bd, out indicia)) // slice
//                            indices.Add(indicia);
//                        else
//                        {
//                            oHedron.Edges[hedron.EdgeIndices[4]].IndexIB = indicesValue;
//                            oHedron.Edges[hedron.EdgeIndices[4]].HasIntersection = true;
//                            Intersection(b, d, nb, nd, hedron.B, hedron.D);
//                            dictionary.Add(bd, indicesValue);
//                            indices.Add(indicesValue++);
//                        }
//                    }

//                    if (oHedron.Edges[hedron.EdgeIndices[2]].HasIntersection)
//                        indices.Add(oHedron.Edges[hedron.EdgeIndices[2]].IndexIB);
//                    else
//                    {
//                        ad = new DEdge(hedron.ia, hedron.id);

//                        if (dictionary.TryGetValue(ad, out indicia))
//                            indices.Add(indicia);
//                        else
//                        {
//                            oHedron.Edges[hedron.EdgeIndices[2]].IndexIB = indicesValue;
//                            oHedron.Edges[hedron.EdgeIndices[2]].HasIntersection = true;
//                            Intersection(a, d, na, nd, hedron.A, hedron.D);
//                            dictionary.Add(ad, indicesValue);
//                            indices.Add(indicesValue++);
//                        }
//                    }

//                    if (oHedron.Edges[hedron.EdgeIndices[5]].HasIntersection)
//                        indices.Add(oHedron.Edges[hedron.EdgeIndices[5]].IndexIB);
//                    else
//                    {
//                        cd = new DEdge(hedron.ic, hedron.id);

//                        if (dictionary.TryGetValue(cd, out indicia))
//                            indices.Add(indicia);
//                        else
//                        {
//                            oHedron.Edges[hedron.EdgeIndices[5]].IndexIB = indicesValue;
//                            oHedron.Edges[hedron.EdgeIndices[5]].HasIntersection = true;
//                            Intersection(c, d, nc, nd, hedron.C, hedron.D);
//                            dictionary.Add(cd, indicesValue);
//                            indices.Add(indicesValue++);
//                        }
//                    }

//                    break;

//                case 2: // c
//                case 13:
//                    if (oHedron.Edges[hedron.EdgeIndices[1]].HasIntersection)
//                        indices.Add(oHedron.Edges[hedron.EdgeIndices[1]].IndexIB);
//                    else
//                    {
//                        ac = new DEdge(hedron.ia, hedron.ic);

//                        if (dictionary.TryGetValue(ac, out indicia))
//                            indices.Add(indicia);
//                        else
//                        {
//                            oHedron.Edges[hedron.EdgeIndices[1]].IndexIB = indicesValue;
//                            oHedron.Edges[hedron.EdgeIndices[1]].HasIntersection = true;
//                            Intersection(a, c, na, nc, hedron.A, hedron.C);
//                            dictionary.Add(ac, indicesValue);
//                            indices.Add(indicesValue++);
//                        }
//                    }

//                    if (oHedron.Edges[hedron.EdgeIndices[3]].HasIntersection)
//                    {
//                        indices.Add(oHedron.Edges[hedron.EdgeIndices[3]].IndexIB);
//                    }
//                    else
//                    {
//                        bc = new DEdge(hedron.ib, hedron.ic);

//                        if (dictionary.TryGetValue(bc, out indicia))
//                            indices.Add(indicia);
//                        else
//                        {
//                            oHedron.Edges[hedron.EdgeIndices[3]].IndexIB = indicesValue;
//                            oHedron.Edges[hedron.EdgeIndices[3]].HasIntersection = true;
//                            Intersection(b, c, nb, nc, hedron.B, hedron.C);
//                            dictionary.Add(bc, indicesValue);
//                            indices.Add(indicesValue++);
//                        }
//                    }

//                    if (oHedron.Edges[hedron.EdgeIndices[5]].HasIntersection)
//                        indices.Add(oHedron.Edges[hedron.EdgeIndices[5]].IndexIB);
//                    else
//                    {
//                        cd = new DEdge(hedron.ic, hedron.id);

//                        if (dictionary.TryGetValue(cd, out indicia))
//                            indices.Add(indicia);
//                        else
//                        {
//                            oHedron.Edges[hedron.EdgeIndices[5]].IndexIB = indicesValue;
//                            oHedron.Edges[hedron.EdgeIndices[5]].HasIntersection = true;
//                            Intersection(c, d, nc, nd, hedron.C, hedron.D);
//                            dictionary.Add(cd, indicesValue);
//                            indices.Add(indicesValue++);
//                        }
//                    }

//                    break;

//                case 4: // b
//                case 11:
//                    if (oHedron.Edges[hedron.EdgeIndices[0]].HasIntersection)
//                        indices.Add(oHedron.Edges[hedron.EdgeIndices[0]].IndexIB);
//                    else
//                    {
//                        ab = new DEdge(hedron.ia, hedron.ib);
//                        if (dictionary.TryGetValue(ab, out indicia))
//                            indices.Add(indicia);
//                        else
//                        {
//                            oHedron.Edges[hedron.EdgeIndices[0]].IndexIB = indicesValue;
//                            oHedron.Edges[hedron.EdgeIndices[0]].HasIntersection = true;
//                            Intersection(a, b, na, nb, hedron.A, hedron.B);
//                            dictionary.Add(ab, indicesValue);
//                            indices.Add(indicesValue++);
//                        }
//                    }

//                    if (oHedron.Edges[hedron.EdgeIndices[3]].HasIntersection)
//                        indices.Add(oHedron.Edges[hedron.EdgeIndices[3]].IndexIB);
//                    else
//                    {
//                        bc = new DEdge(hedron.ib, hedron.ic);

//                        if (dictionary.TryGetValue(bc, out indicia))
//                            indices.Add(indicia);
//                        else
//                        {
//                            oHedron.Edges[hedron.EdgeIndices[3]].IndexIB = indicesValue;
//                            oHedron.Edges[hedron.EdgeIndices[3]].HasIntersection = true;
//                            Intersection(b, c, nb, nc, hedron.B, hedron.C);
//                            dictionary.Add(bc, indicesValue);
//                            indices.Add(indicesValue++);
//                        }
//                    }

//                    if (oHedron.Edges[hedron.EdgeIndices[4]].HasIntersection) // local
//                        indices.Add(oHedron.Edges[hedron.EdgeIndices[4]].IndexIB);
//                    else
//                    {
//                        bd = new DEdge(hedron.ib, hedron.id);

//                        if (dictionary.TryGetValue(bd, out indicia)) // slice
//                            indices.Add(indicia);
//                        else
//                        {
//                            oHedron.Edges[hedron.EdgeIndices[4]].IndexIB = indicesValue;
//                            oHedron.Edges[hedron.EdgeIndices[4]].HasIntersection = true;
//                            Intersection(b, d, nb, nd, hedron.B, hedron.D);
//                            dictionary.Add(bd, indicesValue);
//                            indices.Add(indicesValue++);
//                        }
//                    }
//                    break;

//                case 7: // a
//                case 8:
//                    if (oHedron.Edges[hedron.EdgeIndices[0]].HasIntersection)
//                        indices.Add(oHedron.Edges[hedron.EdgeIndices[0]].IndexIB);
//                    else
//                    {
//                        ab = new DEdge(hedron.ia, hedron.ib);
//                        if (dictionary.TryGetValue(ab, out indicia))
//                            indices.Add(indicia);
//                        else
//                        {
//                            oHedron.Edges[hedron.EdgeIndices[0]].IndexIB = indicesValue;
//                            oHedron.Edges[hedron.EdgeIndices[0]].HasIntersection = true;
//                            Intersection(a, b, na, nb, hedron.A, hedron.B);
//                            dictionary.Add(ab, indicesValue);
//                            indices.Add(indicesValue++);
//                        }
//                    }

//                    if (oHedron.Edges[hedron.EdgeIndices[1]].HasIntersection)
//                        indices.Add(oHedron.Edges[hedron.EdgeIndices[1]].IndexIB);
//                    else
//                    {
//                        ac = new DEdge(hedron.ia, hedron.ic);

//                        if (dictionary.TryGetValue(ac, out indicia))
//                            indices.Add(indicia);
//                        else
//                        {
//                            oHedron.Edges[hedron.EdgeIndices[1]].IndexIB = indicesValue;
//                            oHedron.Edges[hedron.EdgeIndices[1]].HasIntersection = true;
//                            Intersection(a, c, na, nc, hedron.A, hedron.C);
//                            dictionary.Add(ac, indicesValue);
//                            indices.Add(indicesValue++);
//                        }
//                    }

//                    if (oHedron.Edges[hedron.EdgeIndices[2]].HasIntersection)
//                        indices.Add(oHedron.Edges[hedron.EdgeIndices[2]].IndexIB);
//                    else
//                    {
//                        ad = new DEdge(hedron.ia, hedron.id);

//                        if (dictionary.TryGetValue(ad, out indicia))
//                            indices.Add(indicia);
//                        else
//                        {
//                            oHedron.Edges[hedron.EdgeIndices[2]].IndexIB = indicesValue;
//                            oHedron.Edges[hedron.EdgeIndices[2]].HasIntersection = true;
//                            Intersection(a, d, na, nd, hedron.A, hedron.D);
//                            dictionary.Add(ad, indicesValue);
//                            indices.Add(indicesValue++);
//                        }
//                    }
//                    break;

//                case 3: // AC AD, BC, BD
//                case 12:
//                    if (oHedron.Edges[hedron.EdgeIndices[1]].HasIntersection)
//                        indices.Add(oHedron.Edges[hedron.EdgeIndices[1]].IndexIB);
//                    else
//                    {
//                        ac = new DEdge(hedron.ia, hedron.ic);

//                        if (dictionary.TryGetValue(ac, out indicia))
//                            indices.Add(indicia);
//                        else
//                        {
//                            oHedron.Edges[hedron.EdgeIndices[1]].IndexIB = indicesValue;
//                            oHedron.Edges[hedron.EdgeIndices[1]].HasIntersection = true;
//                            Intersection(a, c, na, nc, hedron.A, hedron.C);
//                            dictionary.Add(ac, indicesValue);
//                            indices.Add(indicesValue++);
//                        }
//                    }

//                    if (oHedron.Edges[hedron.EdgeIndices[2]].HasIntersection)
//                    {
//                        index0 = oHedron.Edges[hedron.EdgeIndices[2]].IndexIB;
//                        indices.Add(index0);
//                    }
//                    else
//                    {
//                        ad = new DEdge(hedron.ia, hedron.id);

//                        if (dictionary.TryGetValue(ad, out indicia))
//                        {
//                            index0 = indicia;
//                            indices.Add(index0);
//                        }
//                        else
//                        {
//                            oHedron.Edges[hedron.EdgeIndices[2]].IndexIB = indicesValue;
//                            oHedron.Edges[hedron.EdgeIndices[2]].HasIntersection = true;
//                            Intersection(a, d, na, nd, hedron.A, hedron.D);
//                            index0 = indicesValue;
//                            dictionary.Add(ad, indicesValue);
//                            indices.Add(indicesValue++);
//                        }
//                    }

//                    if (oHedron.Edges[hedron.EdgeIndices[3]].HasIntersection)
//                    {
//                        index1 = oHedron.Edges[hedron.EdgeIndices[3]].IndexIB;
//                        indices.Add(index1);
//                    }
//                    else
//                    {
//                        bc = new DEdge(hedron.ib, hedron.ic);

//                        if (dictionary.TryGetValue(bc, out indicia))
//                        {
//                            index1 = indicia;
//                            indices.Add(index1);
//                        }
//                        else
//                        {
//                            oHedron.Edges[hedron.EdgeIndices[3]].IndexIB = indicesValue;
//                            oHedron.Edges[hedron.EdgeIndices[3]].HasIntersection = true;
//                            Intersection(b, c, nb, nc, hedron.B, hedron.C);
//                            index1 = indicesValue;
//                            dictionary.Add(bc, indicesValue);
//                            indices.Add(indicesValue++);
//                        }
//                    }

//                    if (oHedron.Edges[hedron.EdgeIndices[4]].HasIntersection)
//                    {
//                        indices.Add(oHedron.Edges[hedron.EdgeIndices[4]].IndexIB);
//                        indices.Add(index1);
//                        indices.Add(index0);
//                    }
//                    else
//                    {
//                        bd = new DEdge(hedron.ib, hedron.id);

//                        if (dictionary.TryGetValue(bd, out indicia))
//                        {
//                            indices.Add(dictionary[bd]);
//                            indices.Add(index1);
//                            indices.Add(index0);
//                        }
//                        else
//                        {
//                            oHedron.Edges[hedron.EdgeIndices[4]].IndexIB = indicesValue;
//                            oHedron.Edges[hedron.EdgeIndices[4]].HasIntersection = true;
//                            Intersection(b, d, nb, nd, hedron.B, hedron.D);
//                            dictionary.Add(bd, indicesValue);
//                            indices.Add(index1);
//                            indices.Add(index0);
//                            indices.Add(indicesValue++);
//                        }
//                    }
//                    break;

//                case 5: // AB, AD, BC, CD
//                case 10:
//                    if (oHedron.Edges[hedron.EdgeIndices[0]].HasIntersection)
//                        indices.Add(oHedron.Edges[hedron.EdgeIndices[0]].IndexIB);
//                    else
//                    {
//                        ab = new DEdge(hedron.ia, hedron.ib);
//                        if (dictionary.TryGetValue(ab, out indicia))
//                            indices.Add(indicia);
//                        else
//                        {
//                            oHedron.Edges[hedron.EdgeIndices[0]].IndexIB = indicesValue;
//                            oHedron.Edges[hedron.EdgeIndices[0]].HasIntersection = true;
//                            Intersection(a, b, na, nb, hedron.A, hedron.B);
//                            dictionary.Add(ab, indicesValue);
//                            indices.Add(indicesValue++);
//                        }
//                    }

//                    if (oHedron.Edges[hedron.EdgeIndices[2]].HasIntersection)
//                    {
//                        index0 = oHedron.Edges[hedron.EdgeIndices[2]].IndexIB;
//                        indices.Add(index0);
//                    }
//                    else
//                    {
//                        ad = new DEdge(hedron.ia, hedron.id);

//                        if (dictionary.TryGetValue(ad, out indicia))
//                        {
//                            index0 = indicia;
//                            indices.Add(index0);
//                        }
//                        else
//                        {
//                            oHedron.Edges[hedron.EdgeIndices[2]].IndexIB = indicesValue;
//                            oHedron.Edges[hedron.EdgeIndices[2]].HasIntersection = true;
//                            index0 = indicesValue;
//                            Intersection(a, d, na, nd, hedron.A, hedron.D);
//                            dictionary.Add(ad, indicesValue);
//                            indices.Add(indicesValue++);
//                        }
//                    }

//                    if (oHedron.Edges[hedron.EdgeIndices[3]].HasIntersection)
//                    {
//                        index1 = oHedron.Edges[hedron.EdgeIndices[3]].IndexIB;
//                        indices.Add(index1);
//                    }
//                    else
//                    {
//                        bc = new DEdge(hedron.ib, hedron.ic);

//                        if (dictionary.TryGetValue(bc, out indicia))
//                        {
//                            index1 = indicia;
//                            indices.Add(index1);
//                        }
//                        else
//                        {
//                            oHedron.Edges[hedron.EdgeIndices[3]].IndexIB = indicesValue;
//                            oHedron.Edges[hedron.EdgeIndices[3]].HasIntersection = true;
//                            index1 = indicesValue;
//                            Intersection(b, c, nb, nc, hedron.B, hedron.C);
//                            dictionary.Add(bc, indicesValue);
//                            indices.Add(indicesValue++);
//                        }
//                    }

//                    if (oHedron.Edges[hedron.EdgeIndices[5]].HasIntersection)
//                    {
//                        indices.Add(oHedron.Edges[hedron.EdgeIndices[5]].IndexIB);
//                        indices.Add(index1);
//                        indices.Add(index0);
//                    }
//                    else
//                    {
//                        cd = new DEdge(hedron.ic, hedron.id);

//                        if (dictionary.TryGetValue(cd, out indicia))
//                        {
//                            indices.Add(dictionary[cd]);
//                            indices.Add(index1);
//                            indices.Add(index0);
//                        }
//                        else
//                        {
//                            oHedron.Edges[hedron.EdgeIndices[5]].IndexIB = indicesValue;
//                            oHedron.Edges[hedron.EdgeIndices[5]].HasIntersection = true;
//                            Intersection(c, d, nc, nd, hedron.C, hedron.D);
//                            dictionary.Add(cd, indicesValue);
//                            indices.Add(indicesValue++);
//                            indices.Add(index1);
//                            indices.Add(index0);
//                        }
//                    }
//                    break;

//                case 6: // AB, AC, BD, CD
//                case 9:
//                    if (oHedron.Edges[hedron.EdgeIndices[0]].HasIntersection)
//                        indices.Add(oHedron.Edges[hedron.EdgeIndices[0]].IndexIB);
//                    else
//                    {
//                        ab = new DEdge(hedron.ia, hedron.ib);
//                        if (dictionary.TryGetValue(ab, out indicia))
//                            indices.Add(indicia);
//                        else
//                        {
//                            oHedron.Edges[hedron.EdgeIndices[0]].IndexIB = indicesValue;
//                            oHedron.Edges[hedron.EdgeIndices[0]].HasIntersection = true;
//                            Intersection(a, b, na, nb, hedron.A, hedron.B);
//                            dictionary.Add(ab, indicesValue);
//                            indices.Add(indicesValue++);
//                        }
//                    }

//                    if (oHedron.Edges[hedron.EdgeIndices[1]].HasIntersection)
//                    {
//                        index0 = oHedron.Edges[hedron.EdgeIndices[1]].IndexIB;
//                        indices.Add(index0);
//                    }
//                    else
//                    {
//                        ac = new DEdge(hedron.ia, hedron.ic);

//                        if (dictionary.TryGetValue(ac, out indicia))
//                        {
//                            index0 = indicia;
//                            indices.Add(indicia);
//                        }
//                        else
//                        {
//                            oHedron.Edges[hedron.EdgeIndices[1]].IndexIB = indicesValue;
//                            oHedron.Edges[hedron.EdgeIndices[1]].HasIntersection = true;
//                            Intersection(a, c, na, nc, hedron.A, hedron.C);
//                            index0 = indicesValue;
//                            dictionary.Add(ac, indicesValue);
//                            indices.Add(indicesValue++);
//                        }
//                    }

//                    if (oHedron.Edges[hedron.EdgeIndices[4]].HasIntersection) // local
//                    {
//                        index1 = oHedron.Edges[hedron.EdgeIndices[4]].IndexIB;
//                        indices.Add(index1);
//                    }
//                    else
//                    {
//                        bd = new DEdge(hedron.ib, hedron.id);

//                        if (dictionary.TryGetValue(bd, out indicia)) // slice
//                        {
//                            indices.Add(indicia);
//                            index1 = indicia;
//                        }
//                        else
//                        {
//                            oHedron.Edges[hedron.EdgeIndices[4]].IndexIB = indicesValue;
//                            oHedron.Edges[hedron.EdgeIndices[4]].HasIntersection = true;
//                            Intersection(b, d, nb, nd, hedron.B, hedron.D);
//                            dictionary.Add(bd, indicesValue);
//                            index1 = indicesValue;
//                            indices.Add(indicesValue++);
//                        }
//                    }

//                    if (oHedron.Edges[hedron.EdgeIndices[5]].HasIntersection)
//                    {
//                        indices.Add(oHedron.Edges[hedron.EdgeIndices[5]].IndexIB);
//                        indices.Add(index1);
//                        indices.Add(index0);
//                    }
//                    else
//                    {
//                        cd = new DEdge(hedron.ic, hedron.id);

//                        if (dictionary.TryGetValue(cd, out indicia))
//                        {
//                            indices.Add(dictionary[cd]);
//                            indices.Add(index1);
//                            indices.Add(index0);
//                        }
//                        else
//                        {
//                            oHedron.Edges[hedron.EdgeIndices[5]].IndexIB = indicesValue;
//                            oHedron.Edges[hedron.EdgeIndices[5]].HasIntersection = true;
//                            Intersection(c, d, nc, nd, hedron.C, hedron.D);
//                            dictionary.Add(cd, indicesValue);
//                            indices.Add(indicesValue++);
//                            indices.Add(index1);
//                            indices.Add(index0);
//                        }
//                    }
//                    break;
//            }
//        }

//        /// <summary>
//        /// Edge intersection a determiantion normal in intersection.
//        /// </summary>
//        /// <paramsChanged name="a">coordinates vertices a</paramsChanged>
//        /// <paramsChanged name="b">coordinates vertices b</paramsChanged>
//        /// <paramsChanged name="na">normal coordinates in vertices a</paramsChanged>
//        /// <paramsChanged name="nb">normal coordinates in vertices b</paramsChanged>
//        /// <paramsChanged name="va">values in vertices a</paramsChanged>
//        /// <paramsChanged name="vb">values in vertices b</paramsChanged>
//        public void Intersection(Point3D a, Point3D b, Normal3D na, Normal3D nb, double va, double vb)
//        {
//            double tmp = (CCL.TresholdValue - va) / (vb - va);

//            Point3D p = a + (b - a) * tmp;
//            Normal3D n = na + (nb - na) * tmp;

//            p.X *= distortion.X;
//            p.Y *= distortion.Y;
//            p.Z *= distortion.Z;

//            n.Nx *= distortion.X;
//            n.Ny *= distortion.Y;
//            n.Nz *= distortion.Z;

//            vertices.Add(p);
//            normals.Add(n);
//        }

//        /// <summary>
//        /// Determination values of six neighbouring centres.
//        /// </summary>
//        private void GetAdjacentCenters(int i, int j, int k, out Centers adjacentCenters)
//        {
//#if(DEBUG)
//            if ((i < 1 && j < 1 && k < 1) || (i > (row - 2) && j > (row - 2) && k > (row - 2)))
//                throw new IndexOutOfRangeException("Index out of range [GetAdjecentCentres]");
//#endif
//            Cell cell;

//            adjacentCenters = new Centers();

//            long index = i + j * row;

//            // front
//            if (middleBuffer[index - 1].Used)
//                adjacentCenters.Front = middleBuffer[index - 1].S;
//            else
//            {
//                cell = GetCell(i - 1, j, k);
//                middleBuffer[index - 1] = cell;
//                adjacentCenters.Front = cell.S;
//            }

//            // back
//            if (middleBuffer[index + 1].Used)
//                adjacentCenters.Back = middleBuffer[index + 1].S;
//            else
//            {
//                cell = GetCell(i + 1, j, k);
//                middleBuffer[index + 1] = cell;
//                adjacentCenters.Back = cell.S;
//            }

//            // left
//            if (middleBuffer[index - row].Used)
//                adjacentCenters.Left = middleBuffer[index - row].S;
//            else
//            {
//                cell = GetCell(i, j - 1, k);
//                middleBuffer[index - row] = cell;
//                adjacentCenters.Left = cell.S;
//            }

//            // right
//            if (middleBuffer[index + row].Used)
//                adjacentCenters.Right = middleBuffer[index + row].S;
//            else
//            {
//                cell = GetCell(i, j + 1, k);
//                middleBuffer[index + row] = cell;
//                adjacentCenters.Right = cell.S;
//            }

//            // down
//            if (downBuffer[index].Used)
//                adjacentCenters.Down = downBuffer[index].S;
//            else
//            {
//                cell = GetCell(i, j, k + 1);
//                downBuffer[index] = cell;
//                adjacentCenters.Down = cell.S;
//            }

//            // up
//            if (upBuffer[index].Used)
//                adjacentCenters.Top = upBuffer[index].S;
//            else
//            {
//                cell = GetCell(i, j, k - 1);
//                upBuffer[index] = cell;
//                adjacentCenters.Top = cell.S;
//            }
//        }

//        /// <summary>
//        /// Determination normals.
//        /// </summary>
//        private Normals ComputeNormals(int i, int j, int k, Centers adjacentCenters, Cell cx)
//        {
//            Cell front, back, left, right, top, down;
//            Cell a, b, c, d, e, f, g, h;
//            Normals normals = new Normals();

//            #region Cells necessary for normals determination

//            int iPlus = i + 1;
//            int iMinus = i - 1;
//            int jPlus = j + 1;
//            int jMinus = j - 1;
//            int iPlusPlus = iPlus + 1;
//            int iMinusMinus = iMinus - 1;
//            int jPlusPlus = jPlus + 1;
//            int jMinusMinus = jMinus - 1;

//            int jRow = j * row;


//            // front
//            if ((iMinus) > 0)
//            {
//                if (middleBuffer[iMinusMinus + jRow].Used)
//                    front = middleBuffer[iMinusMinus + jRow];
//                else
//                {
//                    front = GetCell(iMinusMinus, j, k);
//                    middleBuffer[iMinusMinus + jRow] = front;
//                }
//            }
//            else
//                front = middleBuffer[iMinus + jRow];

//            // back
//            if ((iPlus) < (size[0] - 2))
//            {
//                if (middleBuffer[iPlusPlus + jRow].Used)
//                    back = middleBuffer[iPlusPlus + jRow];
//                else
//                {
//                    back = GetCell(iPlusPlus, j, k);
//                    middleBuffer[iPlusPlus + jRow] = back;
//                }
//            }
//            else
//                back = middleBuffer[iPlus + jRow];

//            // left
//            if (jMinus > 0)
//            {
//                if (middleBuffer[i + jMinusMinus * row].Used)
//                    left = middleBuffer[i + jMinusMinus * row];
//                else
//                {
//                    left = GetCell(i, jMinusMinus, k);
//                    middleBuffer[i + jMinusMinus * row] = left;
//                }
//            }
//            else
//                left = middleBuffer[i + jMinus * row];

//            // right
//            if (jPlus < (size[1] - 2))
//            {
//                if (middleBuffer[i + jPlusPlus * row].Used)
//                    right = middleBuffer[i + jPlusPlus * row];
//                else
//                {
//                    right = GetCell(i, jPlusPlus, k);
//                    middleBuffer[i + jPlusPlus * row] = right;
//                }
//            }
//            else
//                right = middleBuffer[i + jPlus * row];

//            // top
//            if ((k - 1) > 0)
//                top = GetCell(i, j, k - 2);
//            else
//                top = upBuffer[i + jRow];

//            // down
//            if ((k + 1) < (size[2] - 2))
//                down = GetCell(i, j, k + 2);
//            else
//                down = upBuffer[i + jRow];

//            // e
//            if (upBuffer[iMinus + jMinus * row].Used)
//                e = upBuffer[iMinus + jMinus * row];
//            else
//            {
//                e = GetCell(iMinus, jMinus, k - 1);
//                upBuffer[iMinus + jMinus * row] = e;
//            }

//            // f
//            if (upBuffer[iPlus + jMinus * row].Used)
//                f = upBuffer[iPlus + jMinus * row];
//            else
//            {
//                f = GetCell(iPlus, jMinus, k - 1);
//                upBuffer[(iPlus) + jMinus * row] = f;
//            }

//            // g
//            if (downBuffer[iPlus + jMinus * row].Used)
//                g = downBuffer[iPlus + jMinus * row];
//            else
//            {
//                g = GetCell(iPlus, jMinus, k + 1);
//                downBuffer[iPlus + jMinus * row] = g;
//            }

//            // h
//            if (downBuffer[iMinus + jMinus * row].Used)
//                h = downBuffer[iMinus + jMinus * row];
//            else
//            {
//                h = GetCell(iMinus, jMinus, k + 1);
//                downBuffer[(iMinus) + jMinus * row] = h;
//            }

//            // a
//            if (upBuffer[(iMinus) + jPlus * row].Used)
//                a = upBuffer[(iMinus) + jPlus * row];
//            else
//            {
//                a = GetCell(iMinus, jPlus, k - 1);
//                upBuffer[iMinus + jPlus * row] = a;
//            }

//            // b
//            if (upBuffer[iPlus + jPlus * row].Used)
//                b = upBuffer[iPlus + jPlus * row];
//            else
//            {
//                b = GetCell(iPlus, jPlus, k - 1);
//                upBuffer[iPlus + jPlus * row] = b;
//            }

//            // c
//            if (downBuffer[(iPlus) + jPlus * row].Used)
//                c = downBuffer[(iPlus) + jPlus * row];
//            else
//            {
//                c = GetCell(iPlus, jPlus, k + 1);
//                downBuffer[iPlus + jPlus * row] = c;
//            }

//            // d
//            if (downBuffer[iMinus + jPlus * row].Used)
//                d = downBuffer[iMinus + jPlus * row];
//            else
//            {
//                d = GetCell(iMinus, jPlus, k + 1);
//                downBuffer[iMinus + jPlus * row] = d;
//            }

//            #endregion

//            #region Normlas in vertices

//            normals.NA = new Normal3D(cx.B - front.B, right.E - cx.E, cx.D - top.D);
//            normals.NA *= 0.5f;

//            normals.NS = normals.NA;

//            normals.NB = new Normal3D(back.A - cx.A, right.F - cx.F, cx.C - top.C);
//            normals.NB *= 0.5f;
//            normals.NS += normals.NB;

//            normals.NC = new Normal3D(back.D - cx.D, right.G - cx.G, down.B - cx.B);
//            normals.NC *= 0.5f;
//            normals.NS += normals.NC;

//            normals.ND = new Normal3D(cx.C - front.C, right.H - cx.H, down.A - cx.A);
//            normals.ND *= 0.5f;
//            normals.NS += normals.ND;

//            normals.NE = new Normal3D(cx.F - front.F, cx.A - left.A, cx.H - top.H);
//            normals.NE *= 0.5f;
//            normals.NS += normals.NE;

//            normals.NF = new Normal3D(back.E - cx.E, cx.B - left.B, cx.G - top.G);
//            normals.NF *= 0.5f;
//            normals.NS += normals.NF;

//            normals.NG = new Normal3D(back.H - cx.H, cx.C - left.C, down.F - cx.F);
//            normals.NG *= 0.5f;
//            normals.NS += normals.NG;

//            normals.NH = new Normal3D(cx.G - front.G, cx.D - left.D, down.E - cx.E);
//            normals.NH *= 0.5f;
//            normals.NS += normals.NH;

//            normals.NS *= 0.125f;

//            #endregion

//            #region Normal neighbourin centers

//            // front center normal
//            Normal3D x0 = new Normal3D(cx.A - front.A, a.D - front.F, front.C - a.E);
//            Normal3D x1 = new Normal3D(cx.E - front.E, front.B - e.H, front.G - e.A);
//            Normal3D x2 = new Normal3D(cx.H - front.H, front.C - h.E, h.D - front.F);
//            Normal3D x3 = new Normal3D(cx.D - front.D, d.A - front.G, d.H - front.B);

//            Normal3D x = (x0 + x1 + x2 + x3) * 0.5f;
//            normals.NFront = (x + normals.NA + normals.NE + normals.NH + normals.ND) * 0.125f;

//            // back center normal
//            x0 = new Normal3D(back.B - cx.B, b.C - back.E, back.D - b.F);
//            x1 = new Normal3D(back.F - cx.F, back.A - f.G, back.H - f.B);
//            x2 = new Normal3D(back.G - cx.G, back.D - g.F, g.C - back.E);
//            x3 = new Normal3D(back.C - cx.C, c.B - back.H, c.G - back.A);

//            x = (x0 + x1 + x2 + x3) * 0.5f;
//            normals.NBack = (x + normals.NB + normals.NF + normals.NG + normals.NC) * 0.125f;

//            // left center normal
//            x0 = new Normal3D(left.B - e.H, cx.E - left.E, left.D - e.F);
//            x1 = new Normal3D(f.G - left.A, cx.F - left.F, left.C - f.E);
//            x2 = new Normal3D(g.F - left.D, cx.G - left.G, g.H - left.B);
//            x3 = new Normal3D(left.C - h.E, cx.H - left.H, h.G - left.A);

//            x = (x0 + x1 + x2 + x3) * 0.5f;
//            normals.NLeft = (x + normals.NE + normals.NF + normals.NG + normals.NH) * 0.125f;

//            // right center normal
//            x0 = new Normal3D(b.C - right.E, right.B - cx.B, right.G - b.A);
//            x1 = new Normal3D(right.F - a.D, right.A - cx.A, right.H - a.B);
//            x2 = new Normal3D(right.G - d.A, right.D - cx.D, d.C - right.E);
//            x3 = new Normal3D(c.B - right.H, right.C - cx.C, c.D - right.F);

//            x = (x0 + x1 + x2 + x3) * 0.5f;
//            normals.NRight = (x + normals.NA + normals.NB + normals.NC + normals.ND) * 0.125f;

//            // top center normal
//            x0 = new Normal3D(top.G - e.A, top.D - e.F, cx.E - top.E);
//            x1 = new Normal3D(f.B - top.H, top.C - f.E, cx.F - top.F);
//            x2 = new Normal3D(b.F - top.D, b.A - top.G, cx.B - top.B);
//            x3 = new Normal3D(top.C - a.E, a.B - top.H, cx.A - top.A);

//            x = (x0 + x1 + x2 + x3) * 0.5f;
//            normals.NTop = (x + normals.NA + normals.NB + normals.NE + normals.NF) * 0.125f;

//            // down center normal
//            x0 = new Normal3D(down.F - h.D, down.A - h.G, down.H - cx.H);
//            x1 = new Normal3D(g.C - down.E, down.B - g.H, down.G - cx.G);
//            x2 = new Normal3D(c.G - down.A, c.D - down.F, down.C - cx.C);
//            x3 = new Normal3D(down.B - d.H, d.C - down.E, down.D - cx.D);

//            x = (x0 + x1 + x2 + x3) * 0.5f;
//            normals.NDown = (x + normals.NC + normals.ND + normals.NG + normals.NH) * 0.125f;

//            #endregion

//            return normals;
//        }

//        /// <summary>
//        /// Determination values of the cell.
//        /// </summary>
//        /// <paramsChanged name="i">column index</paramsChanged>
//        /// <paramsChanged name="j">row index</paramsChanged>
//        /// <paramsChanged name="k">slice index</paramsChanged>
//        /// <returns>cell at position [i, j, k]</returns>
//        private Cell GetCell(int i, int j, int k)
//        {
//            Cell cell = new Cell();
//            cell.Used = true;

//            cell.SetValues(
//                            (Scalar)vData[i, j + 1, k],            // A
//                            (Scalar)vData[i + 1, j + 1, k],        // B
//                            (Scalar)vData[i + 1, j + 1, k + 1],    // C
//                            (Scalar)vData[i, j + 1, k + 1],        // D
//                            (Scalar)vData[i, j, k],                // E
//                            (Scalar)vData[i + 1, j, k],            // F
//                            (Scalar)vData[i + 1, j, k + 1],        // G
//                            (Scalar)vData[i, j, k + 1]             // H
//                        );

//            return cell;
//        }

//        #endregion

//        #region Properties

//        /// <summary>
//        /// Define treshold value in %. Value for which wil iso-surface extract.
//        /// </summary> 
//        [Description("Treshold value. Range is 1% - 100%. \nIf 'useThresholdAsIsoValue' flag is set to True,"
//        + " is this value directly used as iso value to be searched (used for implicit surfaces visualization).")]
//        public double Treshold
//        {
//            set
//            {
//                if (value >= 0 && value <= 100)
//                {
//                    tresholdPercent = value;
//                    CCL.TresholdValue = min + (max - min) * tresholdPercent * 0.01f;
//                    paramsChanged = true;
//                }
//                else
//                {
//                    throw new MveException(this, "Values should be in range 1% - 100% !");
//                }
//            }

//            get { return tresholdPercent; }
//        }

//        /// <summary>
//        /// Enable kd-tree preprocessing.
//        /// </summary>
//        [Description("Enable using kd-tree. Kd-tree is used for searching active cells, but is most memory expensive !!")]
//        public bool UseKd_tree
//        {
//            set
//            {
//                kdTree = value;
//                kdBuild = false;
//                paramsChanged = true;
//            }

//            get { return kdTree; }
//        }

//        /// <summary>
//        /// Set name of RegGridND point attribute. Default set to Values.
//        /// </summary>
//        [Description("Set name of RegGridND point attribute. Default set to \"Values\"")]
//        public string PointAttributeName
//        {
//            set
//            {
//                if (value == string.Empty)
//                {
//                    throw new MveException(this, "Attribut property cannot be set to an empty string!");
//                }
//                else
//                {
//                    pointAttributesName = value;
//                    paramsChanged = true;
//                }
//            }

//            get { return pointAttributesName; }
//        }

//        /// <summary>
//        /// Set name of RegGridND normal attribute. Default set to Values.
//        /// </summary>
//        [Description("Set name of normals point attribute. Default set to \"Normal3D\"")]
//        public string NormalAttributeName
//        {
//            set
//            {
//                if (value == string.Empty)
//                {
//                    throw new MveException(this, "Attribut property cannot be set to an empty string!");
//                }
//                else
//                {
//                    normalAttributeName = value;
//                    paramsChanged = true;
//                }
//            }
//            get { return normalAttributeName; }
//        }

//        /// <summary>
//        /// Property for linearInterpolation variable.
//        /// </summary>
//        [Description("Specifies whether to use input threshold as percentual value from min/max interval or directly as iso value.")]
//        public bool UseThresholdAsIsoValue
//        {
//            get { return (useThresholdAsIsoValue); }
//            set
//            {
//                useThresholdAsIsoValue = value;
//                paramsChanged = true;
//            }
//        }

//        #endregion

//        #region Internal structures

//        /// <summary>
//        /// Structure used for keep vol. data which define cell. Values A ... H are read from vol. data source and
//        /// sample S is calculated: S = A + B + ... + H /8 (aritmetic mean).
//        /// </summary>
//        /// <remarks>
//        ///   E ------ F
//        ///  /|       /|
//        /// A ------ B |
//        /// | |      | |
//        /// | H -----| G
//        /// |/       |/
//        /// D ------ C
//        /// 
//        /// </remarks>
//        private struct Cell
//        {
//            /// <summary>
//            /// Define whether cell was used. Atribut is used in buffers.
//            /// </summary>
//            public bool Used;

//            /// <summary>
//            /// Values in corners.
//            /// </summary>
//            public double A, B, C, D, E, F, G, H, S;

//            /// <summary>
//            /// Assingning values.
//            /// </summary>
//            public void SetValues(double a, double b, double c, double d, double e, double f, double g, double h)
//            {
//                Used = true;

//                S = A = a;
//                S += B = b;
//                S += C = c;
//                S += D = d;
//                S += E = e;
//                S += F = f;
//                S += G = g;
//                S += H = h;

//                S /= 8;
//            }
//        }

//        /// <summary>
//        /// Sturcture used for storage six vaules of surrounding centres.
//        /// </summary>
//        /// <remarks>
//        /// 	      Top
//        ///            |  Left          
//        ///	           |/                 
//        ///  Front ----|---- Back        +--- x
//        ///           /|                /|
//        ///     Right  |               y |
//        ///          Down                z
//        /// </remarks>                   
//        private struct Centers
//        {
//            /// <summary>
//            /// Centres vaules of surrounding cells.
//            /// </summary>
//            public double Top, Down, Front, Back, Right, Left;

//            /// <summary>
//            /// Test whether values are same.
//            /// </summary>
//            /// <returns>true if values are same</returns>
//            public bool Useless()
//            {
//                if ((Top == Down) && (Down == Front) && (Front == Back) && (Back == Right) && (Right == Left))
//                    return true;
//                else
//                    return false;
//            }
//        }



//        /// <summary>
//        /// Structure used for storage information about edge.
//        /// </summary>
//        private struct Edge
//        {
//            /// <summary>
//            /// End vertices.
//            /// </summary>
//            public Point3D A, B;
//            /// <summary>
//            /// Define whether intersection has alredy count.
//            /// </summary>
//            public bool HasIntersection;

//            /// <summary>
//            /// Index of intersection. Is valid only if HasIntersetion == true.
//            /// </summary>
//            public int IndexIB;

//            public Edge(Point3D a, Point3D b)
//            {
//                if (a.X <= b.X && a.Y <= b.Y && a.Z <= b.Z)
//                {
//                    A = a;
//                    B = b;
//                }
//                else
//                {
//                    A = b;
//                    B = a;
//                }

//                HasIntersection = false;
//                IndexIB = 0;
//            }
//        }

//        /// <summary>
//        /// Structure used for storage normals necessary for determination normals in cell.
//        /// </summary>
//        private struct Normals
//        {
//            /// <summary>
//            /// Normals in vertices of cell and centres.
//            /// </summary>
//            public Normal3D NA, NB, NC, ND, NE, NF, NG, NH, NS, NFront, NBack, NLeft, NRight, NTop, NDown;
//        }

//        /// <summary>
//        /// Class define 4 tetrahedron where each 2 neighbouring tetrahedrons share 3 edges.
//        /// </summary>
//        private class Octahedron
//        {
//            //         0   1   2   3   4   5   6  7   8   9   10  11  12
//            // edges: AB, AC, AD, AE, AF, BC, BE, BF, CD, CF, DF, DE, EF;

//            /// <summary>
//            /// Octahedron edges.
//            /// </summary>
//            public Edge[] Edges;

//            //oHedron.AB, oHedron.AC, oHedron.AF, oHedron.BC, oHedron.BF, oHedron.CF
//            public static byte [] topIndices = { 0, 1, 4, 5, 7, 9 };

//            //oHedron.AC, oHedron.AD, oHedron.AF, oHedron.CD, oHedron.CF, oHedron.DF
//            public static byte [] backIndices = { 1, 2, 4, 8, 9, 10 };

//            //oHedron.AD, oHedron.AE, oHedron.AF, oHedron.DE, oHedron.DF, oHedron.EF
//            public static byte [] downIndices = { 2, 3, 4, 11, 10, 12 };

//            //oHedron.AE, oHedron.AB, oHedron.AF, oHedron.BE, oHedron.EF, oHedron.BF
//            public static byte [] frontIndices = { 3, 0, 4, 6, 12, 7 };

//            /// <summary>
//            /// Create instance.
//            /// </summary>
//            public Octahedron()
//            {
//                Edges = new Edge[13]; // 13 ... 4 tetrahedron where each 2 neighbouring share 3 edges 
//            }

//            /// <summary>
//            /// Clear values.
//            /// </summary>
//            public void Clear()
//            {
//                Array.Clear(Edges, 0, Edges.Length);
//            }
//        }

//        /// <summary>
//        /// Edge of the cube or tetrahedron.
//        /// </summary>
//        public struct DEdge : IEquatable<DEdge>
//        {
//            int p1, p2;

//            /// <summary>
//            /// Constructor.
//            /// </summary>
//            /// <paramsChanged name="x"></paramsChanged>
//            /// <paramsChanged name="y"></paramsChanged>
//            public DEdge(int x, int y)
//            {
//                p1 = x;
//                p2 = y;
//            }

//            /// <summary>
//            /// Get hash code.
//            /// </summary>
//            /// <returns>hash code</returns>
//            public override int GetHashCode()
//            {
//                return p1 * p2;
//            }

//            #region IEquatable<Edge> Members

//            /// <summary>
//            /// Define equals.
//            /// </summary>
//            /// <paramsChanged name="other"></paramsChanged>
//            /// <returns>true if equals</returns>
//            public bool Equals(DEdge other)
//            {
//                if ((p1 == other.p1) && (p2 == other.p2) || (p1 == other.p2) && (p2 == other.p1))
//                    return (true);
//                else
//                    return (false);
//            }

//            #endregion
//        }

//        /// <summary>
//        /// Bit valuation tetrahedron vertices.
//        /// </summary>
//        private struct Flags
//        {
//            /// <summary>
//            /// Value indicate which edges of tetrahedron are intersect.
//            /// </summary>
//            public byte Value;

//            public bool Set0 { set { Value |= 1; } } // D
//            public bool Set1 { set { Value |= 2; } } // C
//            public bool Set2 { set { Value |= 4; } } // B
//            public bool Set3 { set { Value |= 8; } } // A
//        }

//        /// <summary>
//        /// Structure used for storage tetrahedron values.
//        /// </summary>
//        /// <remarks>
//        ///    C
//        ///  / | \
//        /// A -+- D
//        ///  \ | /
//        ///    B
//        /// </remarks>
//        private struct Tetrahedron
//        {
//            /// <summary>
//            /// Values of tetrahedron.
//            /// </summary>
//            public double A, B, C, D;

//            /// <summary>
//            /// indexy pro hash funkci
//            /// </summary>
//            public int ia, ib, ic, id;

//            /// <summary>
//            /// Edge indices.
//            /// </summary>
//            public byte[] EdgeIndices;

//            /// <summary>
//            /// Define if vertices are same.
//            /// </summary>
//            public bool Useless;

//            /// <summary>
//            /// Attribute of intersect edges.
//            /// </summary>
//            public Flags Flags;

//            /// <summary>
//            /// Set indexs.
//            /// </summary>
//            public void SetIndex(int ia, int ib, int ic, int id)
//            {
//                ia = ia;
//                ib = ib;
//                ic = ic;
//                id = id;
//            }

//            /// <summary>
//            /// Values in vertices.
//            /// </summary>
//            public void SetValues(double a, double b, double c, double d)
//            {
//                if (a == b && b == c && c == d)
//                    Useless = true;
//                else
//                    Useless = false;

//                A = a;
//                B = b;
//                C = c;
//                D = d;

//                Flags = new Flags();
//            }

//            /// <summary>
//            /// Setting edge indices of octahedron which define tetrahedron. 
//            /// </summary>
//            /// <paramsChanged name="indices">edege indices</paramsChanged>
//            public void SetIndices(byte[] indices)
//            {
//                EdgeIndices = indices;
//            }

//            /// <summary>
//            /// Attribute of intersect edges.
//            /// </summary>
//            /// <returns>TRUE if no one edege is intersect</returns>
//            public bool Evaluate()
//            {
//                if (A < CCL.TresholdValue) Flags.Set3 = true;
//                if (B < CCL.TresholdValue) Flags.Set2 = true;
//                if (C < CCL.TresholdValue) Flags.Set1 = true;
//                if (D < CCL.TresholdValue) Flags.Set0 = true;

//                return (Flags.Value > 0) && (Flags.Value < 15);
//            }
//        }

//        /// <summary>
//        /// Structure define kd-tree node.
//        /// </summary>
//        private struct KdNode
//        {
//            public int pKdCell; // index to array of kdCells
//            public double Min;   // minimum vol. value of cell
//            public double Max;   // maximum vol. value of cell

//            public KdNode(int pKdCell, double min, double max)
//            {
//                this.pKdCell = pKdCell;
//                Min = min;
//                Max = max;
//            }
//        }

//        /// <summary>
//        /// Structure contain cell data.
//        /// </summary>
//        private struct KdCell
//        {
//            public Cell cell;       // values in the conrners
//            public Centers centers; // 6 neighbouring centres
//            public int i, j, k;     // indexs in the grid .. used for determ. position
//            public Normals normals; // normals

//            public KdCell(Cell cell, Centers centers, Normals normals, int i, int j, int k)
//            {
//                this.cell = cell;
//                this.centers = centers;
//                this.normals = normals;
//                this.i = i;
//                this.j = j;
//                this.k = k;
//            }

//            /// <summary>
//            /// Minimum value from cell values.
//            /// </summary>
//            /// <returns>minimum value</returns>
//            public double GetMin()
//            {
//                double min = cell.A;

//                if (cell.B < min) min = cell.B;
//                if (cell.C < min) min = cell.C;
//                if (cell.D < min) min = cell.D;
//                if (cell.E < min) min = cell.E;
//                if (cell.F < min) min = cell.F;
//                if (cell.G < min) min = cell.G;
//                if (cell.H < min) min = cell.H;

//                if (centers.Back < min) min = centers.Back;
//                if (centers.Front < min) min = centers.Front;

//                if (centers.Left < min) min = centers.Left;
//                if (centers.Right < min) min = centers.Right;

//                if (centers.Top < min) min = centers.Top;
//                if (centers.Down < min) min = centers.Down;

//                return min;
//            }

//            /// <summary>
//            /// Maximum values from cell values.
//            /// </summary>
//            /// <returns>maximum values</returns>
//            public double GetMax()
//            {
//                double max = cell.A;

//                if (cell.B > max) max = cell.B;
//                if (cell.C > max) max = cell.C;
//                if (cell.D > max) max = cell.D;
//                if (cell.E > max) max = cell.E;
//                if (cell.F > max) max = cell.F;
//                if (cell.G > max) max = cell.G;
//                if (cell.H > max) max = cell.H;

//                if (centers.Back > max) max = centers.Back;
//                if (centers.Front > max) max = centers.Front;

//                if (centers.Left > max) max = centers.Left;
//                if (centers.Right > max) max = centers.Right;

//                if (centers.Top > max) max = centers.Top;
//                if (centers.Down > max) max = centers.Down;

//                return max;
//            }
//        }

//        /// <summary>
//        /// Class used for sorting kd-tree nodes by minimum.
//        /// </summary>
//        private class SortByMin : IComparer<KdNode>
//        {
//            public int Compare(KdNode x, KdNode y)
//            {
//                double o1 = x.Min;
//                double o2 = y.Min;

//                if (o1 > o2)
//                    return 1;

//                if (o1 < o2)
//                    return -1;
//                else
//                    return 0;
//            }
//        }

//        /// <summary>
//        /// Class used for sorting kd-tree nodes by maximum.
//        /// </summary>
//        private class SortByMax : IComparer<KdNode>
//        {
//            public int Compare(KdNode x, KdNode y)
//            {
//                double o1 = x.Max;
//                double o2 = y.Max;

//                if (o1 > o2)
//                    return 1;

//                if (o1 < o2)
//                    return -1;
//                else
//                    return 0;
//            }
//        }

//        #endregion
//    }
//}
