using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;

using Zcu.Mve.Core;
using Zcu.Mve.Numerics;
using Zcu.Mve.Visualization;


namespace Zcu.Mve.Visualization.Processing
{
    /// <summary>
    /// Module extracts iso-surface from input volume data with Marching Tetrahedra algorithm. Generates even more triangles/points than Marching. Cubes algorithm - it would be nice
    /// to use some module for triangle mesh simplification or another suitable post-processing. Module shall be used with 'VolLoader' module, which provides desired input data. 
    /// </summary>
    [ModuleInfo("Ji Zapletal", "Marching Tetrahedra iso-surface extractor.\nModule shall be used with 'VolLoader' module, which provides desired input data.", CategoryType.Filter, CategoryData.Volume, IconName = "MT5.ico")]
    [PortInfo("InThreshold", "This value defines iso-surface threshold (in percents).\nNon-required input port.")]
    [PortInfo("InRegGridND", "Input volume data.")]
    [PortInfo("Out", "Output data - triangle mesh.")]
    public class MarchingTetrahedra5 : MarchingTetrahedra
    {
        /// <summary> Initializes variables.  </summary>
        protected override void Initialize() { base.Initialize(); }

        /// <summary> Checks whether a cube is intersected by searched iso-surface and eventually generates appropriate triangles. </summary>
        protected override void GenerateTriangles(Cube cube, int i, int j, int k)
        {
            //we must regularly alternate both types of this division
            if ((i + j + k) % 2 != 0)
            {
                CalculateTetraIndex(cube.p[0], cube.p[1], cube.p[3], cube.p[4]);
                CalculateTetraIndex(cube.p[5], cube.p[1], cube.p[4], cube.p[6]);
                CalculateTetraIndex(cube.p[1], cube.p[4], cube.p[6], cube.p[3]);
                CalculateTetraIndex(cube.p[2], cube.p[1], cube.p[6], cube.p[3]);
                CalculateTetraIndex(cube.p[7], cube.p[3], cube.p[6], cube.p[4]);
            }
            else
            {
                CalculateTetraIndex(cube.p[4], cube.p[0], cube.p[7], cube.p[5]);
                CalculateTetraIndex(cube.p[1], cube.p[2], cube.p[0], cube.p[5]);
                CalculateTetraIndex(cube.p[0], cube.p[2], cube.p[7], cube.p[5]);
                CalculateTetraIndex(cube.p[6], cube.p[2], cube.p[5], cube.p[7]);
                CalculateTetraIndex(cube.p[3], cube.p[0], cube.p[2], cube.p[7]);
            }
        } // of GenerateTriangles

    } // of MarchingTetrahedra5 class
} // of namespace Zcu.Gsvd.Extractions